// Flashlight
// Dynamic Light
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRange;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fAreaLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 4) uniform sampler g_xTrilinearWrap;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 3, binding = 0) uniform texture2D g_xTextureArray[];
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec3 _210;
vec4 _211;
vec2 _212;

void main()
{
    ivec2 _222 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _230 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _222, 0).x);
    vec4 _245 = vec4((FrameConstants.g_xCameraTanHalfFOV * _230) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _230, 1.0);
    
    _245.x -= vk3d_params[VIEW].stereo.x * (_230 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0]; 
    
    vec4 _296 = TEXEL_FETCH(g_xLighting_NormalBuffer, _222, 0);
    vec4 _298 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _222, 0);
    vec4 _300 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _222, 0);
    vec3 _304 = normalize((_296.xyz * 2.0) - vec3(1.0));
    vec3 _305 = _298.xyz;
    float _306 = _300.y;
    float _307 = _298.w;
    float _308 = _300.x;
    vec2 _311 = (_300.zw * 2.0) - vec2(1.0);
    vec2 _313 = _311.xy;
    float _317 = sqrt(clamp(1.0 - dot(_313, _313), 0.0, 1.0));
    vec3 _322 = vec3(_311.x, _311.y, _210.z);
    _322.z = (_296.w != 0.0) ? _317 : (-_317);
    vec3 _323 = normalize(in_var_CamDir);
    vec4 _1036;
    bool _1037;
    do
    {
        vec4 _348;
        bool _349;
        do
        {
            vec4 _328 = _245 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            float _329 = _328.z;
            if (_329 < 0.0)
            {
                _348 = _211;
                _349 = false;
                break;
            }
            vec2 _336 = _328.xy / vec2(_328.w);
            vec4 _339 = vec4(_336, _329, _328.w);
            if (any(lessThan(vec2(0.5) - abs(_336.xy - vec2(0.5)), vec2(0.0))))
            {
                _348 = _339;
                _349 = false;
                break;
            }
            _348 = _339;
            _349 = true;
            break;
        } while(false);
        if (!_349)
        {
            _1036 = vec4(0.0, 0.0, 0.0, 1.0);
            _1037 = false;
            break;
        }
        vec3 _357 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_245 * FrameConstants.g_xInvView).xyz;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_bCustomBoundingVolumeSet != 0u)
        {
            float _362 = -dot(_357, g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz);
            if ((_362 < g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fMinRange) || (_362 > g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange))
            {
                _1036 = vec4(0.0, 0.0, 0.0, 1.0);
                _1037 = false;
                break;
            }
        }
        float _368 = length(_357);
        vec3 _370 = _357 / vec3(_368);
        float _379 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_368 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_379 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _1036 = vec4(0.0, 0.0, 0.0, 1.0);
            _1037 = false;
            break;
        }
        vec4 _751;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex != (-1))
        {
            float _395 = (_348.z + max(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias * (1.0 - clamp(dot(_304, _370), 0.0, 1.0)), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias)) / _348.w;
            float _401 = (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane;
            float _402 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _403 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _409 = clamp(((_401 / (_402 - ((((1.0 - _395) * 2.0) - 1.0) * _403))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _403, 0.0, 1.0);
            vec4 _750;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength > 0.0)
            {
                float _746;
                if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness > 0.0)
                {
                    float _484 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                    float _485 = sin(_484);
                    float _486 = cos(_484);
                    float _488 = (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov;
                    float _490 = float(uint(0));
                    float _493 = sqrt(_490 + 0.5) * 0.5;
                    float _495 = (_490 * 2.400000095367431640625) + _484;
                    uint _502 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    vec4 _507 = texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(_502)], g_xBilinearClamp)), _348.xy + (vec2(_493 * cos(_495), _493 * sin(_495)) * _488), FrameConstants.g_fGlobalSamplerBias);
                    float _517 = clamp(((_401 / (_402 - ((((1.0 - _507.x) * 2.0) - 1.0) * _403))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _403, 0.0, 1.0);
                    vec2 _523;
                    if (_517 < _409)
                    {
                        vec2 _521 = _212;
                        _521.x = _517;
                        vec2 _522 = _521;
                        _522.y = 1.0;
                        _523 = _522;
                    }
                    else
                    {
                        _523 = vec2(0.0);
                    }
                    float _525 = float(uint(1));
                    float _528 = sqrt(_525 + 0.5) * 0.5;
                    float _530 = (_525 * 2.400000095367431640625) + _484;
                    vec4 _541 = texture(sampler2D(g_xTextureArray[_502], g_xBilinearClamp), _348.xy + (vec2(_528 * cos(_530), _528 * sin(_530)) * _488), FrameConstants.g_fGlobalSamplerBias);
                    float _551 = clamp(((_401 / (_402 - ((((1.0 - _541.x) * 2.0) - 1.0) * _403))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _403, 0.0, 1.0);
                    vec2 _561;
                    if (_551 < _409)
                    {
                        vec2 _557 = _212;
                        _557.x = _523.x + _551;
                        vec2 _560 = _557;
                        _560.y = _523.y + 1.0;
                        _561 = _560;
                    }
                    else
                    {
                        _561 = _523;
                    }
                    float _563 = float(uint(2));
                    float _566 = sqrt(_563 + 0.5) * 0.5;
                    float _568 = (_563 * 2.400000095367431640625) + _484;
                    vec4 _577 = texture(sampler2D(g_xTextureArray[_502], g_xBilinearClamp), _348.xy + (vec2(_566 * cos(_568), _566 * sin(_568)) * _488), FrameConstants.g_fGlobalSamplerBias);
                    float _587 = clamp(((_401 / (_402 - ((((1.0 - _577.x) * 2.0) - 1.0) * _403))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _403, 0.0, 1.0);
                    vec2 _597;
                    if (_587 < _409)
                    {
                        vec2 _593 = _212;
                        _593.x = _561.x + _587;
                        vec2 _596 = _593;
                        _596.y = _561.y + 1.0;
                        _597 = _596;
                    }
                    else
                    {
                        _597 = _561;
                    }
                    float _599 = float(uint(3));
                    float _602 = sqrt(_599 + 0.5) * 0.5;
                    float _604 = (_599 * 2.400000095367431640625) + _484;
                    vec4 _613 = texture(sampler2D(g_xTextureArray[_502], g_xBilinearClamp), _348.xy + (vec2(_602 * cos(_604), _602 * sin(_604)) * _488), FrameConstants.g_fGlobalSamplerBias);
                    float _623 = clamp(((_401 / (_402 - ((((1.0 - _613.x) * 2.0) - 1.0) * _403))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _403, 0.0, 1.0);
                    vec2 _633;
                    if (_623 < _409)
                    {
                        vec2 _629 = _212;
                        _629.x = _597.x + _623;
                        vec2 _632 = _629;
                        _632.y = _597.y + 1.0;
                        _633 = _632;
                    }
                    else
                    {
                        _633 = _597;
                    }
                    float _745;
                    if (_633.y > 0.0)
                    {
                        float _639 = _633.x / _633.y;
                        float _641 = (_409 - _639) / _639;
                        float _648 = (((clamp(_641 * _641, 0.0, 1.0) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness) * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov) + (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize);
                        _745 = (((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_502)], g_xShadowCmpSampler)), vec3(_348.xy + (vec2(((-0.326211988925933837890625) * _485) - ((-0.40580499172210693359375) * _486), ((-0.40580499172210693359375) * _485) + ((-0.326211988925933837890625) * _486)) * _648), _395)) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2(((-0.840143978595733642578125) * _485) - ((-0.0735799968242645263671875) * _486), ((-0.0735799968242645263671875) * _485) + ((-0.840143978595733642578125) * _486)) * _648), _395))) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2(((-0.695913970470428466796875) * _485) - (0.45713698863983154296875 * _486), (0.45713698863983154296875 * _485) + ((-0.695913970470428466796875) * _486)) * _648), _395))) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2(((-0.203345000743865966796875) * _485) - (0.62071597576141357421875 * _486), (0.62071597576141357421875 * _485) + ((-0.203345000743865966796875) * _486)) * _648), _395))) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2((0.96233999729156494140625 * _485) - ((-0.194983005523681640625) * _486), ((-0.194983005523681640625) * _485) + (0.96233999729156494140625 * _486)) * _648), _395))) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2((0.4734340012073516845703125 * _485) - ((-0.4800260066986083984375) * _486), ((-0.4800260066986083984375) * _485) + (0.4734340012073516845703125 * _486)) * _648), _395))) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2((0.51945602893829345703125 * _485) - (0.76702201366424560546875 * _486), (0.76702201366424560546875 * _485) + (0.51945602893829345703125 * _486)) * _648), _395))) + texture(sampler2DShadow(g_xTextureArray[_502], g_xShadowCmpSampler), vec3(_348.xy + (vec2((0.18546099960803985595703125 * _485) - ((-0.89312398433685302734375) * _486), ((-0.89312398433685302734375) * _485) + (0.18546099960803985595703125 * _486)) * _648), _395))) * 0.125;
                    }
                    else
                    {
                        _745 = 0.0;
                    }
                    _746 = _745;
                }
                else
                {
                    uint _418 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    _746 = (((((((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_418)], g_xShadowCmpSampler)), vec3(_348.xy + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395)) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) + texture(sampler2DShadow(g_xTextureArray[_418], g_xShadowCmpSampler), vec3(_348.xy + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _395))) * 0.083333335816860198974609375;
                }
                vec4 _749 = vec4(0.0, 0.0, 0.0, 1.0);
                _749.w = 1.0 - (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength * _746);
                _750 = _749;
            }
            else
            {
                _750 = vec4(0.0, 0.0, 0.0, 1.0);
            }
            _751 = _750;
        }
        else
        {
            _751 = vec4(0.0, 0.0, 0.0, 1.0);
        }
        vec4 _1035;
        if (_751.w > 0.0)
        {
            vec4 _764;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iTextureIndex > 0)
            {
                _764 = textureLod(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iTextureIndex))], g_xTrilinearClamp)), _348.xy, 0.0);
            }
            else
            {
                _764 = vec4(1.0);
            }
            vec4 _777;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iDetailTextureIndex > 0)
            {
                _777 = _764 * textureLod(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iDetailTextureIndex))], g_xTrilinearWrap)), (_348.xy * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xDetailOffsetScale.zw) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xDetailOffsetScale.xy, 0.0);
            }
            else
            {
                _777 = _764;
            }
            float _798;
            vec3 _799;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius > 0.0)
            {
                vec3 _785 = -reflect(_323, _304);
                vec3 _788 = (_785 * dot(_357, _785)) - _357;
                _798 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius * inversesqrt(dot(_357, _357));
                _799 = normalize(_357 + (_788 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius / length(_788), 0.0, 1.0)));
            }
            else
            {
                _798 = 0.0;
                _799 = _370;
            }
            float _801 = (_307 != 0.01960784383118152618408203125) ? _308 : 0.0;
            vec3 _803 = normalize(_323 + _799);
            float _805 = clamp(dot(_304, _370), 0.0, 1.0);
            float _807 = clamp(dot(_304, _323), 0.0, 1.0);
            float _809 = clamp(dot(_304, _803), 0.0, 1.0);
            float _811 = clamp(dot(_323, _803), 0.0, 1.0);
            bool _812 = _307 == 0.01568627543747425079345703125;
            vec3 _849;
            vec3 _850;
            vec3 _851;
            if (_812)
            {
                _849 = vec3(0.046999998390674591064453125);
                _850 = vec3(_805);
                _851 = _305;
            }
            else
            {
                vec3 _845;
                vec3 _846;
                vec3 _847;
                if (_307 == 0.007843137718737125396728515625)
                {
                    _845 = vec3(0.0280000008642673492431640625);
                    _846 = vec3(_805);
                    _847 = _305;
                }
                else
                {
                    vec3 _841;
                    vec3 _842;
                    vec3 _843;
                    if (_307 == 0.0039215688593685626983642578125)
                    {
                        _841 = vec3(0.0280000008642673492431640625);
                        _842 = vec3((clamp((clamp(dot(_370, -_304), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_370, -_323), 0.0, 1.0), 7.0), 0.0, 1.0) * _801) + _805);
                        _843 = _305;
                    }
                    else
                    {
                        vec3 _824 = vec3(_801);
                        _841 = mix(vec3(0.039999999105930328369140625), _305, _824);
                        _842 = vec3(_805);
                        _843 = mix(_305, vec3(0.0), _824);
                    }
                    _845 = _841;
                    _846 = _842;
                    _847 = _843;
                }
                _849 = _845;
                _850 = _846;
                _851 = _847;
            }
            vec3 _852 = _850 * _851;
            float _853 = max(_306, 0.0500000007450580596923828125);
            float _854 = _853 * _853;
            vec3 _967;
            if (_812)
            {
                float _905 = clamp(1.0 - _306, 0.0, 1.0);
                float _907 = ((_308 * 2.0) - 1.0) * 0.300000011920928955078125;
                float _913 = pow(_905, 8.0) * 500.0;
                float _916 = pow(_905, 6.0) * 2000.0;
                float _930 = dot(normalize(_322 + (_304 * ((-0.0500000007450580596923828125) + _907))), _803);
                float _950 = dot(normalize(_322 + (_304 * ((0.0500000007450580596923828125 + _907) - (_905 * 0.0500000007450580596923828125)))), _803);
                _967 = _852 + ((((vec3(((smoothstep(-1.0, 0.0, _930) * pow(sqrt(clamp(1.0 - (_930 * _930), 0.0, 1.0)), 20.0 + _913)) * ((((_308 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _905))) * ((_913 + 28.0) * 0.03978873789310455322265625)) * (_305 / vec3(max(dot(_305, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _950) * pow(sqrt(clamp(1.0 - (_950 * _950), 0.0, 1.0)), 50.0 + _916)) * ((((_308 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _905)) * ((_916 + 58.0) * 0.03978873789310455322265625))) * _805) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
            }
            else
            {
                float _860 = clamp(_798 * (1.0 - _854), 0.0, 1.0);
                float _880;
                if (_860 > 0.0)
                {
                    float _871 = _854 * _854;
                    _880 = _871 / (_871 + (((_860 * 0.25) * ((3.0 * _854) + _860)) / (_811 + 0.001000000047497451305389404296875)));
                }
                else
                {
                    _880 = 1.0;
                }
                float _881 = 2.0 - _854;
                float _888 = _854 * _854;
                float _892 = (((_809 * _888) - _809) * _809) + 1.0;
                _967 = _852 + (((((vec3(1.0) - _849) * exp2((((-5.554729938507080078125) * _811) - 6.9831600189208984375) * _811)) + _849) * ((((1.0 / (((_807 * _881) + _854) * ((_805 * _881) + _854))) * (_888 * (1.0 / (_892 * _892)))) * _805) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _880);
            }
            vec3 _1017;
            if (_307 == 0.01960784383118152618408203125)
            {
                bool _971 = _308 > 0.4980392158031463623046875;
                vec3 _989 = sqrt(abs(_851));
                float _1003 = 0.5 / _854;
                _1017 = mix(_967, (_852 + (((((vec3(1.0) - _989) * exp2((((-5.554729938507080078125) * _811) - 6.9831600189208984375) * _811)) + _989) * ((1.0 / ((4.0 * ((_805 + _807) - (_805 * _807))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _1003) * pow(1.0 - (_809 * _809), _1003)))) * (_805 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_851 * (clamp((clamp(dot(_370, -_304), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_370, -_323), 0.0, 1.0), 7.0), 0.0, 1.0) * (_971 ? 0.5 : 0.0))), vec3((_971 ? (_308 - 0.501960813999176025390625) : _308) * 2.007874011993408203125));
            }
            else
            {
                _1017 = _967;
            }
            vec3 _1020 = min(_1017 * ((g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * _379) * _777.xyz), vec3(1000.0)).xyz;
            vec3 _1033;
            if ((_307 == 0.007843137718737125396728515625) && (_230 < 3.0))
            {
                _1033 = _1020 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_230 - 2.0, 0.0)));
            }
            else
            {
                _1033 = _1020;
            }
            _1035 = vec4(_1033.x, _1033.y, _1033.z, _751.w);
        }
        else
        {
            _1035 = _751;
        }
        _1036 = _1035;
        _1037 = true;
        break;
    } while(false);
    if (false == _1037)
    {
        discard;
    }
    out_var_SV_Target = _1036;
}


