#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 1, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 1, std140) uniform type_RainData
{
    mat4 g_xReachabilityViewProj;
    mat4 g_xCameraViewProjPrevFrame;
} RainData;

layout(set = 1, binding = 2, std140) uniform type_MiscParams
{
    float g_fCurrentFrameWeight;
    float g_fScreenScaleX;
    float g_fScreenScaleY;
    float g_fpad;
} MiscParams;

layout(set = 0, binding = 1) uniform sampler g_xPointClamp;
layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 1, binding = 49) uniform TEXTURE_2D g_xFrameDepthStencil;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xRainReachabilityMap;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xMaskPreviousFrame;

layout(location = 0) out vec2 out_var_SV_Target;

uint _87;
vec3 _88;

void main()
{
    vec2 _93 = gl_FragCoord.xy * 2.0;
    float _108 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xFrameDepthStencil, ivec3(int(_93.x), int(_93.y), 0).xy, 0).x);
    vec2 _109 = _93.xy;
    vec4 _126 = vec4((FrameConstants.g_xCameraTanHalfFOV * _108) * (((_109 + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _108, 1.0) * FrameConstants.g_xInvView;
    vec3 _127 = _126.xyz;
    vec4 _133 = vec4(_126.xyz, 1.0);
    float _140 = float(FrameConstants.g_xScreenDimensions.y);
    uvec3 _151 = uvec3(vec3(FrameConstants.g_fScaledTime * 3478.0) + (_127 * 123456.0));
    uvec3 _156 = uvec3(1103515245u) * ((_151 >> (uvec3(1u) & uvec3(31u))) ^ _151.yzx);
    uint _163 = 1103515245u * ((_156.x ^ _156.z) ^ (_156.y >> 3u));
    uint _165 = _163 ^ (_163 >> 16u);
    vec2 _174 = _109 + (((vec3(uvec3(_165, _87, _165 * 48271u) & uvec3(2147483647u)) * vec3(4.656612873077392578125e-10)).xz - vec2(0.5)) * min((0.25 * _140) / _108, 0.100000001490116119384765625 * _140));
    float _184 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xFrameDepthStencil, ivec3(int(_174.x), int(_174.y), 0).xy, 0).x);
    vec4 _193 = vec4((FrameConstants.g_xCameraTanHalfFOV * _184) * (((_174 + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _184, 1.0) * FrameConstants.g_xInvView;
    vec4 _199 = RainData.g_xReachabilityViewProj * vec4(_193.xyz, 1.0);
    vec3 _201 = _193.xyz - _127;
    vec3 _210 = mix((RainData.g_xReachabilityViewProj * _133).xyz, _199.xyz, bvec3(all(lessThanEqual(abs(_199.xy), vec2(1.0))) && (dot(_201, _201) < 4.0)));
    vec3 _214 = _88;
    _214.x = 0.5 + (_210.x * 0.5);
    vec3 _218 = _214;
    _218.y = 0.5 - (_210.y * 0.5);
    vec4 _225 = SAMPLE_TEXTURE1(SAMPLER_2D(g_xRainReachabilityMap, g_xPointClamp), _218.xy, FrameConstants.g_fGlobalSamplerBias);
    float _229 = (_210.z < _225.x) ? 0.0 : FrameConstants.g_fRainBaseWetness;
    vec4 _232 = _133 * RainData.g_xCameraViewProjPrevFrame;
    float _235 = _232.w;
    vec4 _257;
    vec2 _258;
    if (all(lessThan(abs(_232.xy), vec2(_235))))
    {
        vec4 _245 = _232;
        _245.x = 0.5 + ((0.5 * _232.x) / _235);
        vec4 _250 = _245;
        _250.y = 0.5 - ((0.5 * _232.y) / _235);
        _257 = _250;
        _258 = SAMPLE_TEXTURE1(SAMPLER_2D(g_xMaskPreviousFrame, g_xBilinearClamp), _250.xy, FrameConstants.g_fGlobalSamplerBias).xy;
    }
    else
    {
        _257 = _232;
        _258 = vec2(-1.0);
    }
    float _273;
    if ((_108 > 0.0) && (abs((_258.y / _257.w) - 1.0) < 0.00999999977648258209228515625))
    {
        _273 = mix(_258.x, _229, MiscParams.g_fCurrentFrameWeight);
    }
    else
    {
        _273 = _229;
    }
    out_var_SV_Target = vec2(_273, _108);
}

