// Lights 4
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRangeUnused;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 3, binding = 0) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xTextureArray[];
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec3 _198;
vec4 _199;
vec2 _200;

void main()
{
    ivec2 _210 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _218 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _210, 0).x);
    vec4 _233 = vec4((FrameConstants.g_xCameraTanHalfFOV * _218) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _218, 1.0);
    
    _233.x -= vk3d_params[VIEW].stereo.x * (_218 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];	
    
    vec4 _277 = TEXEL_FETCH(g_xLighting_NormalBuffer, _210, 0);
    vec4 _279 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _210, 0);
    vec4 _281 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _210, 0);
    vec3 _285 = normalize((_277.xyz * 2.0) - vec3(1.0));
    vec3 _286 = _279.xyz;
    float _287 = _281.y;
    float _288 = _279.w;
    float _289 = _281.x;
    vec2 _292 = (_281.zw * 2.0) - vec2(1.0);
    vec2 _294 = _292.xy;
    float _298 = sqrt(clamp(1.0 - dot(_294, _294), 0.0, 1.0));
    vec3 _303 = vec3(_292.x, _292.y, _198.z);
    _303.z = (_277.w != 0.0) ? _298 : (-_298);
    vec3 _304 = normalize(in_var_CamDir);
    vec4 _990;
    bool _991;
    do
    {
        vec4 _329;
        bool _330;
        do
        {
            vec4 _309 = _233 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            float _310 = _309.z;
            if (_310 < 0.0)
            {
                _329 = _199;
                _330 = false;
                break;
            }
            vec2 _317 = _309.xy / vec2(_309.w);
            vec4 _320 = vec4(_317, _310, _309.w);
            if (any(lessThan(vec2(0.5) - abs(_317.xy - vec2(0.5)), vec2(0.0))))
            {
                _329 = _320;
                _330 = false;
                break;
            }
            _329 = _320;
            _330 = true;
            break;
        } while(false);
        if (!_330)
        {
            _990 = vec4(0.0, 0.0, 0.0, 1.0);
            _991 = false;
            break;
        }
        vec3 _338 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_233 * FrameConstants.g_xInvView).xyz;
        float _339 = length(_338);
        vec3 _341 = _338 / vec3(_339);
        float _350 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_339 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_350 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _990 = vec4(0.0, 0.0, 0.0, 1.0);
            _991 = false;
            break;
        }
        vec4 _722;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex != (-1))
        {
            float _366 = (_329.z + max(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias * (1.0 - clamp(dot(_285, _341), 0.0, 1.0)), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias)) / _329.w;
            float _372 = (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane;
            float _373 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _374 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _380 = clamp(((_372 / (_373 - ((((1.0 - _366) * 2.0) - 1.0) * _374))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _374, 0.0, 1.0);
            vec4 _721;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength > 0.0)
            {
                float _717;
                if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness > 0.0)
                {
                    float _455 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                    float _456 = sin(_455);
                    float _457 = cos(_455);
                    float _459 = (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov;
                    float _461 = float(uint(0));
                    float _464 = sqrt(_461 + 0.5) * 0.5;
                    float _466 = (_461 * 2.400000095367431640625) + _455;
                    uint _473 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    vec4 _478 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(_473)], g_xBilinearClamp)), _329.xy + (vec2(_464 * cos(_466), _464 * sin(_466)) * _459), FrameConstants.g_fGlobalSamplerBias);
                    float _488 = clamp(((_372 / (_373 - ((((1.0 - _478.x) * 2.0) - 1.0) * _374))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _374, 0.0, 1.0);
                    vec2 _494;
                    if (_488 < _380)
                    {
                        vec2 _492 = _200;
                        _492.x = _488;
                        vec2 _493 = _492;
                        _493.y = 1.0;
                        _494 = _493;
                    }
                    else
                    {
                        _494 = vec2(0.0);
                    }
                    float _496 = float(uint(1));
                    float _499 = sqrt(_496 + 0.5) * 0.5;
                    float _501 = (_496 * 2.400000095367431640625) + _455;
                    vec4 _512 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_473], g_xBilinearClamp), _329.xy + (vec2(_499 * cos(_501), _499 * sin(_501)) * _459), FrameConstants.g_fGlobalSamplerBias);
                    float _522 = clamp(((_372 / (_373 - ((((1.0 - _512.x) * 2.0) - 1.0) * _374))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _374, 0.0, 1.0);
                    vec2 _532;
                    if (_522 < _380)
                    {
                        vec2 _528 = _200;
                        _528.x = _494.x + _522;
                        vec2 _531 = _528;
                        _531.y = _494.y + 1.0;
                        _532 = _531;
                    }
                    else
                    {
                        _532 = _494;
                    }
                    float _534 = float(uint(2));
                    float _537 = sqrt(_534 + 0.5) * 0.5;
                    float _539 = (_534 * 2.400000095367431640625) + _455;
                    vec4 _548 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_473], g_xBilinearClamp), _329.xy + (vec2(_537 * cos(_539), _537 * sin(_539)) * _459), FrameConstants.g_fGlobalSamplerBias);
                    float _558 = clamp(((_372 / (_373 - ((((1.0 - _548.x) * 2.0) - 1.0) * _374))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _374, 0.0, 1.0);
                    vec2 _568;
                    if (_558 < _380)
                    {
                        vec2 _564 = _200;
                        _564.x = _532.x + _558;
                        vec2 _567 = _564;
                        _567.y = _532.y + 1.0;
                        _568 = _567;
                    }
                    else
                    {
                        _568 = _532;
                    }
                    float _570 = float(uint(3));
                    float _573 = sqrt(_570 + 0.5) * 0.5;
                    float _575 = (_570 * 2.400000095367431640625) + _455;
                    vec4 _584 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_473], g_xBilinearClamp), _329.xy + (vec2(_573 * cos(_575), _573 * sin(_575)) * _459), FrameConstants.g_fGlobalSamplerBias);
                    float _594 = clamp(((_372 / (_373 - ((((1.0 - _584.x) * 2.0) - 1.0) * _374))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _374, 0.0, 1.0);
                    vec2 _604;
                    if (_594 < _380)
                    {
                        vec2 _600 = _200;
                        _600.x = _568.x + _594;
                        vec2 _603 = _600;
                        _603.y = _568.y + 1.0;
                        _604 = _603;
                    }
                    else
                    {
                        _604 = _568;
                    }
                    float _716;
                    if (_604.y > 0.0)
                    {
                        float _610 = _604.x / _604.y;
                        float _612 = (_380 - _610) / _610;
                        float _619 = (((clamp(_612 * _612, 0.0, 1.0) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness) * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov) + (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize);
                        _716 = (((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_473)], g_xShadowCmpSampler)), vec3(_329.xy + (vec2(((-0.326211988925933837890625) * _456) - ((-0.40580499172210693359375) * _457), ((-0.40580499172210693359375) * _456) + ((-0.326211988925933837890625) * _457)) * _619), _366)) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2(((-0.840143978595733642578125) * _456) - ((-0.0735799968242645263671875) * _457), ((-0.0735799968242645263671875) * _456) + ((-0.840143978595733642578125) * _457)) * _619), _366))) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2(((-0.695913970470428466796875) * _456) - (0.45713698863983154296875 * _457), (0.45713698863983154296875 * _456) + ((-0.695913970470428466796875) * _457)) * _619), _366))) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2(((-0.203345000743865966796875) * _456) - (0.62071597576141357421875 * _457), (0.62071597576141357421875 * _456) + ((-0.203345000743865966796875) * _457)) * _619), _366))) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2((0.96233999729156494140625 * _456) - ((-0.194983005523681640625) * _457), ((-0.194983005523681640625) * _456) + (0.96233999729156494140625 * _457)) * _619), _366))) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2((0.4734340012073516845703125 * _456) - ((-0.4800260066986083984375) * _457), ((-0.4800260066986083984375) * _456) + (0.4734340012073516845703125 * _457)) * _619), _366))) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2((0.51945602893829345703125 * _456) - (0.76702201366424560546875 * _457), (0.76702201366424560546875 * _456) + (0.51945602893829345703125 * _457)) * _619), _366))) + texture(sampler2DShadow(g_xTextureArray[_473], g_xShadowCmpSampler), vec3(_329.xy + (vec2((0.18546099960803985595703125 * _456) - ((-0.89312398433685302734375) * _457), ((-0.89312398433685302734375) * _456) + (0.18546099960803985595703125 * _457)) * _619), _366))) * 0.125;
                    }
                    else
                    {
                        _716 = 0.0;
                    }
                    _717 = _716;
                }
                else
                {
                    uint _389 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    _717 = (((((((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_389)], g_xShadowCmpSampler)), vec3(_329.xy + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366)) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) + texture(sampler2DShadow(g_xTextureArray[_389], g_xShadowCmpSampler), vec3(_329.xy + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _366))) * 0.083333335816860198974609375;
                }
                vec4 _720 = vec4(0.0, 0.0, 0.0, 1.0);
                _720.w = 1.0 - (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength * _717);
                _721 = _720;
            }
            else
            {
                _721 = vec4(0.0, 0.0, 0.0, 1.0);
            }
            _722 = _721;
        }
        else
        {
            _722 = vec4(0.0, 0.0, 0.0, 1.0);
        }
        vec4 _989;
        if (_722.w > 0.0)
        {
            float _752;
            vec3 _753;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius > 0.0)
            {
                vec3 _739 = -reflect(_304, _285);
                vec3 _742 = (_739 * dot(_338, _739)) - _338;
                _752 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius * inversesqrt(dot(_338, _338));
                _753 = normalize(_338 + (_742 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius / length(_742), 0.0, 1.0)));
            }
            else
            {
                _752 = 0.0;
                _753 = _341;
            }
            float _755 = (_288 != 0.01960784383118152618408203125) ? _289 : 0.0;
            vec3 _757 = normalize(_304 + _753);
            float _759 = clamp(dot(_285, _341), 0.0, 1.0);
            float _761 = clamp(dot(_285, _304), 0.0, 1.0);
            float _763 = clamp(dot(_285, _757), 0.0, 1.0);
            float _765 = clamp(dot(_304, _757), 0.0, 1.0);
            bool _766 = _288 == 0.01568627543747425079345703125;
            vec3 _803;
            vec3 _804;
            vec3 _805;
            if (_766)
            {
                _803 = vec3(0.046999998390674591064453125);
                _804 = vec3(_759);
                _805 = _286;
            }
            else
            {
                vec3 _799;
                vec3 _800;
                vec3 _801;
                if (_288 == 0.007843137718737125396728515625)
                {
                    _799 = vec3(0.0280000008642673492431640625);
                    _800 = vec3(_759);
                    _801 = _286;
                }
                else
                {
                    vec3 _795;
                    vec3 _796;
                    vec3 _797;
                    if (_288 == 0.0039215688593685626983642578125)
                    {
                        _795 = vec3(0.0280000008642673492431640625);
                        _796 = vec3((clamp((clamp(dot(_341, -_285), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_341, -_304), 0.0, 1.0), 7.0), 0.0, 1.0) * _755) + _759);
                        _797 = _286;
                    }
                    else
                    {
                        vec3 _778 = vec3(_755);
                        _795 = mix(vec3(0.039999999105930328369140625), _286, _778);
                        _796 = vec3(_759);
                        _797 = mix(_286, vec3(0.0), _778);
                    }
                    _799 = _795;
                    _800 = _796;
                    _801 = _797;
                }
                _803 = _799;
                _804 = _800;
                _805 = _801;
            }
            vec3 _806 = _804 * _805;
            float _807 = max(_287, 0.0500000007450580596923828125);
            float _808 = _807 * _807;
            vec3 _921;
            if (_766)
            {
                float _859 = clamp(1.0 - _287, 0.0, 1.0);
                float _861 = ((_289 * 2.0) - 1.0) * 0.300000011920928955078125;
                float _867 = pow(_859, 8.0) * 500.0;
                float _870 = pow(_859, 6.0) * 2000.0;
                float _884 = dot(normalize(_303 + (_285 * ((-0.0500000007450580596923828125) + _861))), _757);
                float _904 = dot(normalize(_303 + (_285 * ((0.0500000007450580596923828125 + _861) - (_859 * 0.0500000007450580596923828125)))), _757);
                _921 = _806 + ((((vec3(((smoothstep(-1.0, 0.0, _884) * pow(sqrt(clamp(1.0 - (_884 * _884), 0.0, 1.0)), 20.0 + _867)) * ((((_289 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _859))) * ((_867 + 28.0) * 0.03978873789310455322265625)) * (_286 / vec3(max(dot(_286, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _904) * pow(sqrt(clamp(1.0 - (_904 * _904), 0.0, 1.0)), 50.0 + _870)) * ((((_289 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _859)) * ((_870 + 58.0) * 0.03978873789310455322265625))) * _759) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
            }
            else
            {
                float _814 = clamp(_752 * (1.0 - _808), 0.0, 1.0);
                float _834;
                if (_814 > 0.0)
                {
                    float _825 = _808 * _808;
                    _834 = _825 / (_825 + (((_814 * 0.25) * ((3.0 * _808) + _814)) / (_765 + 0.001000000047497451305389404296875)));
                }
                else
                {
                    _834 = 1.0;
                }
                float _835 = 2.0 - _808;
                float _842 = _808 * _808;
                float _846 = (((_763 * _842) - _763) * _763) + 1.0;
                _921 = _806 + (((((vec3(1.0) - _803) * exp2((((-5.554729938507080078125) * _765) - 6.9831600189208984375) * _765)) + _803) * ((((1.0 / (((_761 * _835) + _808) * ((_759 * _835) + _808))) * (_842 * (1.0 / (_846 * _846)))) * _759) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _834);
            }
            vec3 _971;
            if (_288 == 0.01960784383118152618408203125)
            {
                bool _925 = _289 > 0.4980392158031463623046875;
                vec3 _943 = sqrt(abs(_805));
                float _957 = 0.5 / _808;
                _971 = mix(_921, (_806 + (((((vec3(1.0) - _943) * exp2((((-5.554729938507080078125) * _765) - 6.9831600189208984375) * _765)) + _943) * ((1.0 / ((4.0 * ((_759 + _761) - (_759 * _761))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _957) * pow(1.0 - (_763 * _763), _957)))) * (_759 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_805 * (clamp((clamp(dot(_341, -_285), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_341, -_304), 0.0, 1.0), 7.0), 0.0, 1.0) * (_925 ? 0.5 : 0.0))), vec3((_925 ? (_289 - 0.501960813999176025390625) : _289) * 2.007874011993408203125));
            }
            else
            {
                _971 = _921;
            }
            vec3 _974 = min(_971 * (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * (_350 * clamp(((-dot(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz, _341)) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffOfs) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffScale, 0.0, 1.0))), vec3(1000.0)).xyz;
            vec3 _987;
            if ((_288 == 0.007843137718737125396728515625) && (_218 < 3.0))
            {
                _987 = _974 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_218 - 2.0, 0.0)));
            }
            else
            {
                _987 = _974;
            }
            _989 = vec4(_987.x, _987.y, _987.z, _722.w);
        }
        else
        {
            _989 = _722;
        }
        _990 = _989;
        _991 = true;
        break;
    } while(false);
    if (false == _991)
    {
        discard;
    }
    out_var_SV_Target = _990;
}


