// More Shadows
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRangeUnused;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 0, binding = 4) uniform sampler g_xTrilinearWrap;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 3, binding = 0) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xTextureArray[];
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec4 _115;
float _132;
vec3 _145;

void main()
{
    ivec2 _155 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _163 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _155, 0).x);
    vec4 _178 = vec4((FrameConstants.g_xCameraTanHalfFOV * _163) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _163, 1.0);
    
   	_178.x -= vk3d_params[VIEW].stereo.x * (_163 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];
    
    vec4 _206 = TEXEL_FETCH(g_xLighting_NormalBuffer, _155, 0);
    vec4 _208 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _155, 0);
    vec4 _210 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _155, 0);
    vec3 _214 = normalize((_206.xyz * 2.0) - vec3(1.0));
    vec3 _215 = _208.xyz;
    float _216 = _210.y;
    float _217 = _208.w;
    float _218 = _210.x;
    vec2 _221 = (_210.zw * 2.0) - vec2(1.0);
    vec2 _223 = _221.xy;
    float _227 = sqrt(clamp(1.0 - dot(_223, _223), 0.0, 1.0));
    vec3 _232 = vec3(_221.x, _221.y, _145.z);
    _232.z = (_206.w != 0.0) ? _227 : (-_227);
    vec3 _233 = normalize(in_var_CamDir);
    vec4 _567;
    bool _568;
    do
    {
        vec4 _258;
        bool _259;
        do
        {
            vec4 _238 = _178 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            if (_238.z < 0.0)
            {
                _258 = _115;
                _259 = false;
                break;
            }
            vec2 _246 = _238.xy / vec2(_238.w);
            vec4 _249 = vec4(_246, _132, _132);
            if (any(lessThan(vec2(0.5) - abs(_246.xy - vec2(0.5)), vec2(0.0))))
            {
                _258 = _249;
                _259 = false;
                break;
            }
            _258 = _249;
            _259 = true;
            break;
        } while(false);
        if (!_259)
        {
            _567 = vec4(0.0, 0.0, 0.0, 1.0);
            _568 = false;
            break;
        }
        vec3 _267 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_178 * FrameConstants.g_xInvView).xyz;
        float _268 = length(_267);
        vec3 _270 = _267 / vec3(_268);
        float _279 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_268 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_279 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _567 = vec4(0.0, 0.0, 0.0, 1.0);
            _568 = false;
            break;
        }
        vec4 _294;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iTextureIndex > 0)
        {
            _294 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iTextureIndex))], g_xTrilinearClamp)), _258.xy, FrameConstants.g_fGlobalSamplerBias);
        }
        else
        {
            _294 = vec4(1.0);
        }
        vec4 _309;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iDetailTextureIndex > 0)
        {
            _309 = _294 * 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iDetailTextureIndex))], g_xTrilinearWrap)), (_258.xy * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xDetailOffsetScale.zw) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xDetailOffsetScale.xy, FrameConstants.g_fGlobalSamplerBias);
        }
        else
        {
            _309 = _294;
        }
        float _330;
        vec3 _331;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius > 0.0)
        {
            vec3 _317 = -reflect(_233, _214);
            vec3 _320 = (_317 * dot(_267, _317)) - _267;
            _330 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius * inversesqrt(dot(_267, _267));
            _331 = normalize(_267 + (_320 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius / length(_320), 0.0, 1.0)));
        }
        else
        {
            _330 = 0.0;
            _331 = _270;
        }
        float _333 = (_217 != 0.01960784383118152618408203125) ? _218 : 0.0;
        vec3 _335 = normalize(_233 + _331);
        float _337 = clamp(dot(_214, _270), 0.0, 1.0);
        float _339 = clamp(dot(_214, _233), 0.0, 1.0);
        float _341 = clamp(dot(_214, _335), 0.0, 1.0);
        float _343 = clamp(dot(_233, _335), 0.0, 1.0);
        bool _344 = _217 == 0.01568627543747425079345703125;
        vec3 _381;
        vec3 _382;
        vec3 _383;
        if (_344)
        {
            _381 = vec3(0.046999998390674591064453125);
            _382 = vec3(_337);
            _383 = _215;
        }
        else
        {
            vec3 _377;
            vec3 _378;
            vec3 _379;
            if (_217 == 0.007843137718737125396728515625)
            {
                _377 = vec3(0.0280000008642673492431640625);
                _378 = vec3(_337);
                _379 = _215;
            }
            else
            {
                vec3 _373;
                vec3 _374;
                vec3 _375;
                if (_217 == 0.0039215688593685626983642578125)
                {
                    _373 = vec3(0.0280000008642673492431640625);
                    _374 = vec3((clamp((clamp(dot(_270, -_214), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_270, -_233), 0.0, 1.0), 7.0), 0.0, 1.0) * _333) + _337);
                    _375 = _215;
                }
                else
                {
                    vec3 _356 = vec3(_333);
                    _373 = mix(vec3(0.039999999105930328369140625), _215, _356);
                    _374 = vec3(_337);
                    _375 = mix(_215, vec3(0.0), _356);
                }
                _377 = _373;
                _378 = _374;
                _379 = _375;
            }
            _381 = _377;
            _382 = _378;
            _383 = _379;
        }
        vec3 _384 = _382 * _383;
        float _385 = max(_216, 0.0500000007450580596923828125);
        float _386 = _385 * _385;
        vec3 _499;
        if (_344)
        {
            float _437 = clamp(1.0 - _216, 0.0, 1.0);
            float _439 = ((_218 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _445 = pow(_437, 8.0) * 500.0;
            float _448 = pow(_437, 6.0) * 2000.0;
            float _462 = dot(normalize(_232 + (_214 * ((-0.0500000007450580596923828125) + _439))), _335);
            float _482 = dot(normalize(_232 + (_214 * ((0.0500000007450580596923828125 + _439) - (_437 * 0.0500000007450580596923828125)))), _335);
            _499 = _384 + ((((vec3(((smoothstep(-1.0, 0.0, _462) * pow(sqrt(clamp(1.0 - (_462 * _462), 0.0, 1.0)), 20.0 + _445)) * ((((_218 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _437))) * ((_445 + 28.0) * 0.03978873789310455322265625)) * (_215 / vec3(max(dot(_215, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _482) * pow(sqrt(clamp(1.0 - (_482 * _482), 0.0, 1.0)), 50.0 + _448)) * ((((_218 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _437)) * ((_448 + 58.0) * 0.03978873789310455322265625))) * _337) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
        }
        else
        {
            float _392 = clamp(_330 * (1.0 - _386), 0.0, 1.0);
            float _412;
            if (_392 > 0.0)
            {
                float _403 = _386 * _386;
                _412 = _403 / (_403 + (((_392 * 0.25) * ((3.0 * _386) + _392)) / (_343 + 0.001000000047497451305389404296875)));
            }
            else
            {
                _412 = 1.0;
            }
            float _413 = 2.0 - _386;
            float _420 = _386 * _386;
            float _424 = (((_341 * _420) - _341) * _341) + 1.0;
            _499 = _384 + (((((vec3(1.0) - _381) * exp2((((-5.554729938507080078125) * _343) - 6.9831600189208984375) * _343)) + _381) * ((((1.0 / (((_339 * _413) + _386) * ((_337 * _413) + _386))) * (_420 * (1.0 / (_424 * _424)))) * _337) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _412);
        }
        vec3 _549;
        if (_217 == 0.01960784383118152618408203125)
        {
            bool _503 = _218 > 0.4980392158031463623046875;
            vec3 _521 = sqrt(abs(_383));
            float _535 = 0.5 / _386;
            _549 = mix(_499, (_384 + (((((vec3(1.0) - _521) * exp2((((-5.554729938507080078125) * _343) - 6.9831600189208984375) * _343)) + _521) * ((1.0 / ((4.0 * ((_337 + _339) - (_337 * _339))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _535) * pow(1.0 - (_341 * _341), _535)))) * (_337 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_383 * (clamp((clamp(dot(_270, -_214), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_270, -_233), 0.0, 1.0), 7.0), 0.0, 1.0) * (_503 ? 0.5 : 0.0))), vec3((_503 ? (_218 - 0.501960813999176025390625) : _218) * 2.007874011993408203125));
        }
        else
        {
            _549 = _499;
        }
        vec3 _552 = min(_549 * ((g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * _279) * _309.xyz), vec3(1000.0)).xyz;
        vec3 _565;
        if ((_217 == 0.007843137718737125396728515625) && (_163 < 3.0))
        {
            _565 = _552 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_163 - 2.0, 0.0)));
        }
        else
        {
            _565 = _552;
        }
        _567 = vec4(_565.x, _565.y, _565.z, vec4(0.0, 0.0, 0.0, 1.0).w);
        _568 = true;
        break;
    } while(false);
    if (false == _568)
    {
        discard;
    }
    out_var_SV_Target = _567;
}


