// Major Water Shader
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_scalar_block_layout : require

struct CascadeLayer
{
    vec2 m_xWorldPos;
    vec2 m_xWorldPosShadow;
    float m_fWorldSize;
    float m_fMinWavelength;
    vec2 _padding;
};

struct WaterMeshInstanceData
{
    vec3 m_xWorldMeshPosition;
    float m_fWorldMeshScale;
    uint m_uCascadeIndex;
    float m_fWorldVertexSize;
    float m_fWorldTexelSize;
    float m_fPadding;
};

struct CubemapData
{
    mat4 m_xObbSpace;
    mat4 m_xInvObbSpace;
    vec3 m_xCubemapPosition;
    float m_fPad;
};

const vec4 _246[4] = vec4[](vec4(0.5, 0.0, 0.0, 0.00999999977648258209228515625), vec4(-0.5299999713897705078125, 0.5, 0.5, 0.0), vec4(0.699999988079071044921875, 0.25, 0.100000001490116119384765625, 0.3300000131130218505859375), vec4(-0.300000011920928955078125, 0.75, 0.100000001490116119384765625, 0.660000026226043701171875));


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 7, std140) uniform type_ProjectedCloudParams
{
    uint g_bSunlightCloudsEnabled;
    float g_fSunlightCloudsVelocityX;
    float g_fSunlightCloudsVelocityY;
    float g_fSunlightCloudsScale;
    vec4 g_xSunlightCloudsProjectionDir;
} ProjectedCloudParams;

layout(set = 1, binding = 6, scalar) uniform type_FogParams
{
    vec3 g_xFogColour;
    float g_fFogNearPlane;
    float g_fRcpFogVolumeDepth;
    float g_fMaxFogDensity;
    uint g_bFogEnabled;
    uint g_bTexturedFogEnabled;
    float g_fHorizonOffset;
    float g_fHorizonScale;
    float g_fTextureHDRScale;
    float g_fTextureUOffset;
    float g_fDensityPower;
    float g_fVerticalTextureScale;
    float g_fHeightFogStrength;
    float g_fHeightFogFalloff;
    float g_fHeightFogHeight;
    vec3 g_xHeightFogColour;
    uint g_bHeightFogEnabled;
    float g_fHeightFogMaxOpacity;
    float g_fHeightFogStartDistance;
    float g_fCosScatteringAngle;
    vec3 g_xSunDirection;
    float g_fDirectScatterStartDistance;
    float g_fDirectScatterFadeDistance;
    vec3 g_xDirectColour;
} FogParams;

layout(set = 3, binding = 3, std140) uniform type_CascadeDefinition
{
    CascadeLayer m_xLayer[10];
    int m_iLayerCount;
    float m_fResolution;
    float g_fAltitudeAlpha;
    int g_iCascadeShiftSinceLastUpdate;
} CascadeDefinition;

layout(set = 3, binding = 5, std140) uniform type_WaterDataTextureParams
{
    vec4 m_xWorldToUVScaleAndOffset;
    vec2 m_xTileLookupUVScale;
    vec2 m_xInputUVScale;
    vec2 m_xTiledUVScale;
    vec2 m_xWetnessMaskTileLookupUVScale;
    vec2 m_xWetnessMaskInputUVScale;
    vec2 m_xWetnessMaskTiledUVScale;
    vec2 m_xTileClassificationBoundsMin;
    vec2 m_xOneOverTileClassificationBoundsRange;
    int m_iTileClassificationMipOffset;
    float m_fWaterLevelMin;
    float m_fWaterLevelMax;
    float m_fOpenWaterLevel;
    uint m_uOpenWaterMaterialID;
    uint m_uOpenWaterSpectrumID;
    uint m_uLevelHasOpenWater;
    uint m_uPadding;
} WaterDataTextureParams;

layout(set = 4, binding = 72, std430) readonly buffer type_StructuredBuffer_WaterMeshInstanceData
{
    WaterMeshInstanceData _m0[];
} g_axWaterMeshInstanceData;

layout(set = 3, binding = 12, std140) uniform type_WaterMiscParams
{
    vec3 g_xDetailCenter;
    uint g_uMinQuadMeshCascade;
    vec2 g_xNormalMapScrollSpeeds1;
    vec2 g_xNormalMapScrollSpeeds2;
    vec2 g_xNormalMapScales;
} WaterMiscParams;

layout(set = 3, binding = 79, std430) readonly buffer type_StructuredBuffer_CubemapData
{
    layout(row_major) CubemapData _m0[];
} g_axPBRCubemapDatas;

layout(set = 3, binding = 13, std140) uniform type_RainData
{
    mat4 g_xRainReachabilityViewProj;
} RainData;

layout(set = 1, binding = 11, std140) uniform type_ShadingParameters
{
    vec3 g_xLightDir;
    float g_fEnableCaustics;
    vec3 g_xSunColour;
    float g_fLightingParamsPad;
    uint m_uNumRainRippleLayers;
} ShadingParameters;

layout(set = 0, binding = 1) uniform sampler g_xPointClamp;
layout(set = 0, binding = 2) uniform sampler g_xBilinearWrap;
layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 0, binding = 6) uniform sampler g_xFilteredWrap;
layout(set = 0, binding = 7) uniform sampler g_xFilteredClamp;
layout(set = 1, binding = 100) uniform TEXTURE_2D g_xLighting_Clouds;
layout(set = 1, binding = 49) uniform texture1D g_xFogTexture;
layout(set = 1, binding = 81) uniform TEXTURE_2D g_xFogColourTexture;
layout(set = 3, binding = 60) uniform TEXTURE_2D g_xWaterTileLookupTexture;
layout(set = 3, binding = 65) uniform TEXTURE_2D g_xWaterMaskTexture;
layout(set = 3, binding = 69) uniform TEXTURE_2D g_xNormalMapTexture;
layout(set = 3, binding = 70) uniform TEXTURE_2D g_xCubeMapLookupTexture;
layout(set = 3, binding = 71) uniform TEXTURE_2D g_xWetnessMaskTileLookupTexture;
layout(set = 3, binding = 63) uniform texture2DArray g_xGerstnerTexture;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xDepthBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xAlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xReflectionBuffer;
layout(set = 3, binding = 68) uniform TEXTURE_2D g_xFoamDetailTexture;
layout(set = 3, binding = 76) uniform TEXTURE_2D g_xRainRippleTexture;
layout(set = 3, binding = 78) uniform TEXTURE_2D g_xRainReachabilityTexture;
layout(set = 3, binding = 77) uniform textureCubeArray g_axPBRCubeMaps;

layout(location = 0) in vec4 in_var_WorldSpacePos_Distance;
layout(location = 1) in float in_var_RoughnessFactor;
layout(location = 2) in vec4 in_var_ReflectionColour_WaterLevelDx;
layout(location = 3) in vec4 in_var_FogTransmissionAlpha_WaterLevelDz;
layout(location = 4) in vec4 in_var_OriginalWorldPos_LodAlpha;
layout(location = 5) in vec4 in_var_FlowVelocity_FoamSim_Shadow;
layout(location = 6) in vec4 in_var_SubsurfaceScatterColour_NormalMapStrength;
layout(location = 7) in vec4 in_var_SubsurfaceScatterColourFastFlow_EdgeBlendRadius;
layout(location = 8) flat in uint in_var_InstanceId;
layout(location = 9) in vec4 in_var_FoamColour;
layout(location = 0) out vec4 out_var_SV_Target;

vec2 _264;
vec3 _265;

void main()
{
    vec2 _290;
    vec2 _296;
    bool _300;
    bool _316;
    bool _317;
    vec2 _318;
    do
    {
        _290 = in_var_OriginalWorldPos_LodAlpha.xz;
        _296 = (_290 * WaterDataTextureParams.m_xWorldToUVScaleAndOffset.xy) + WaterDataTextureParams.m_xWorldToUVScaleAndOffset.zw;
        _300 = all(lessThan(abs(_296 - vec2(0.5)), vec2(0.5)));
        if (_300)
        {
            _316 = true;
            _317 = true;
            _318 = ((_296 * WaterDataTextureParams.m_xWetnessMaskInputUVScale) + (TEXTURE_LOD(SAMPLER_2D(g_xWetnessMaskTileLookupTexture, g_xPointClamp), clamp(_296, vec2(0.0), vec2(1.0)) * WaterDataTextureParams.m_xWetnessMaskTileLookupUVScale, 0.0).xy * 32767.0)) * WaterDataTextureParams.m_xWetnessMaskTiledUVScale;
            break;
        }
        _316 = false;
        _317 = WaterDataTextureParams.m_uLevelHasOpenWater != 0u;
        _318 = vec2(0.0);
        break;
    } while(false);
    bool _332;
    do
    {
        if (_317 == false)
        {
            _332 = false;
            break;
        }
        if (_316)
        {
            _332 = TEXTURE_LOD(SAMPLER_2D(g_xWaterMaskTexture, g_xFilteredClamp), _318, 0.0).x > 0.5;
            break;
        }
        _332 = true;
        break;
    } while(false);
    if (_332 == false)
    {
        discard;
    }
    vec2 _656;
    vec3 _657;
    vec3 _658;
    float _659;
    vec3 _660;
    if (in_var_OriginalWorldPos_LodAlpha.w < 0.00999999977648258209228515625)
    {
        int _584 = int(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex);
        vec2 _590 = vec2(CascadeDefinition.m_xLayer[_584].m_fWorldSize);
        float _592 = float(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex);
        vec3 _602 = in_var_OriginalWorldPos_LodAlpha.xyz + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_290 - CascadeDefinition.m_xLayer[_584].m_xWorldPos) / _590, _592) + vec3(0.0, 0.0, 1.0), 0.0).xyz;
        vec3 _604 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize, 0.0, 0.0);
        vec3 _617 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(0.0, 0.0, g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
        vec3 _629 = (_604 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_604.xz - CascadeDefinition.m_xLayer[_584].m_xWorldPos) / _590, _592) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _602;
        vec3 _630 = (_617 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_617.xz - CascadeDefinition.m_xLayer[_584].m_xWorldPos) / _590, _592) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _602;
        float _631 = _629.y;
        vec2 _632 = _264;
        _632.x = _631;
        float _633 = _630.y;
        vec2 _634 = _632;
        _634.y = _633;
        vec3 _639 = _629;
        _639.y = _631 + (in_var_ReflectionColour_WaterLevelDx.w * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
        vec3 _642 = _630;
        _642.y = _633 + (in_var_FogTransmissionAlpha_WaterLevelDz.w * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
        _656 = _634 / vec2(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
        _657 = _639;
        _658 = _642;
        _659 = clamp(1.0 - (0.3499999940395355224609375 * (((_629.x * _630.z) - (_629.z * _630.x)) / (g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize))), 0.0, 1.0);
        _660 = cross(_639, _642);
    }
    else
    {
        vec2 _579;
        vec3 _580;
        vec3 _581;
        float _582;
        vec3 _583;
        if (in_var_OriginalWorldPos_LodAlpha.w > 0.9900000095367431640625)
        {
            float _504 = 2.0 * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize;
            uint _506 = min((g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex + 1u), 7u);
            int _507 = int(_506);
            vec2 _513 = vec2(CascadeDefinition.m_xLayer[_507].m_fWorldSize);
            float _515 = float(_506);
            vec3 _525 = in_var_OriginalWorldPos_LodAlpha.xyz + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_290 - CascadeDefinition.m_xLayer[_507].m_xWorldPos) / _513, _515) + vec3(0.0, 0.0, 1.0), 0.0).xyz;
            vec3 _527 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(_504, 0.0, 0.0);
            vec3 _540 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(0.0, 0.0, _504);
            vec3 _552 = (_527 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_527.xz - CascadeDefinition.m_xLayer[_507].m_xWorldPos) / _513, _515) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _525;
            vec3 _553 = (_540 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_540.xz - CascadeDefinition.m_xLayer[_507].m_xWorldPos) / _513, _515) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _525;
            float _554 = _552.y;
            vec2 _555 = _264;
            _555.x = _554;
            float _556 = _553.y;
            vec2 _557 = _555;
            _557.y = _556;
            vec3 _562 = _552;
            _562.y = _554 + (in_var_ReflectionColour_WaterLevelDx.w * _504);
            vec3 _565 = _553;
            _565.y = _556 + (in_var_FogTransmissionAlpha_WaterLevelDz.w * _504);
            _579 = _557 / vec2(_504);
            _580 = _562;
            _581 = _565;
            _582 = clamp(1.0 - (0.3499999940395355224609375 * (((_552.x * _553.z) - (_552.z * _553.x)) / (_504 * _504))), 0.0, 1.0);
            _583 = cross(_562, _565);
        }
        else
        {
            int _352 = int(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex);
            vec2 _358 = vec2(CascadeDefinition.m_xLayer[_352].m_fWorldSize);
            float _360 = float(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex);
            vec3 _370 = in_var_OriginalWorldPos_LodAlpha.xyz + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_290 - CascadeDefinition.m_xLayer[_352].m_xWorldPos) / _358, _360) + vec3(0.0, 0.0, 1.0), 0.0).xyz;
            vec3 _372 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize, 0.0, 0.0);
            vec3 _385 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(0.0, 0.0, g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
            vec3 _397 = (_372 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_372.xz - CascadeDefinition.m_xLayer[_352].m_xWorldPos) / _358, _360) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _370;
            vec3 _398 = (_385 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_385.xz - CascadeDefinition.m_xLayer[_352].m_xWorldPos) / _358, _360) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _370;
            float _399 = _397.y;
            vec2 _400 = _264;
            _400.x = _399;
            float _401 = _398.y;
            vec2 _402 = _400;
            _402.y = _401;
            vec3 _407 = _397;
            _407.y = _399 + (in_var_ReflectionColour_WaterLevelDx.w * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
            vec3 _410 = _398;
            _410.y = _401 + (in_var_FogTransmissionAlpha_WaterLevelDz.w * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize);
            float _424 = 2.0 * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize;
            uint _426 = min((g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex + 1u), 7u);
            int _427 = int(_426);
            vec2 _433 = vec2(CascadeDefinition.m_xLayer[_427].m_fWorldSize);
            float _435 = float(_426);
            vec3 _443 = in_var_OriginalWorldPos_LodAlpha.xyz + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_290 - CascadeDefinition.m_xLayer[_427].m_xWorldPos) / _433, _435) + vec3(0.0, 0.0, 1.0), 0.0).xyz;
            vec3 _445 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(_424, 0.0, 0.0);
            vec3 _458 = in_var_OriginalWorldPos_LodAlpha.xyz + vec3(0.0, 0.0, _424);
            vec3 _470 = (_445 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_445.xz - CascadeDefinition.m_xLayer[_427].m_xWorldPos) / _433, _435) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _443;
            vec3 _471 = (_458 + textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), vec3((_458.xz - CascadeDefinition.m_xLayer[_427].m_xWorldPos) / _433, _435) + vec3(0.0, 0.0, 1.0), 0.0).xyz) - _443;
            float _472 = _470.y;
            vec2 _473 = _264;
            _473.x = _472;
            float _474 = _471.y;
            vec2 _475 = _473;
            _475.y = _474;
            vec3 _480 = _470;
            _480.y = _472 + (in_var_ReflectionColour_WaterLevelDx.w * _424);
            vec3 _483 = _471;
            _483.y = _474 + (in_var_FogTransmissionAlpha_WaterLevelDz.w * _424);
            vec3 _497 = vec3(in_var_OriginalWorldPos_LodAlpha.w);
            _579 = mix(_402 / vec2(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize), _475 / vec2(_424), vec2(in_var_OriginalWorldPos_LodAlpha.w));
            _580 = mix(_407, _480, _497);
            _581 = mix(_410, _483, _497);
            _582 = mix(clamp(1.0 - (0.3499999940395355224609375 * (((_397.x * _398.z) - (_397.z * _398.x)) / (g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize * g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize))), 0.0, 1.0), clamp(1.0 - (0.3499999940395355224609375 * (((_470.x * _471.z) - (_470.z * _471.x)) / (_424 * _424))), 0.0, 1.0), in_var_OriginalWorldPos_LodAlpha.w);
            _583 = mix(cross(_407, _410), cross(_480, _483), _497);
        }
        _656 = _579;
        _657 = _580;
        _658 = _581;
        _659 = _582;
        _660 = _583;
    }
    float _686;
    float _822;
    vec3 _661 = normalize(_660);
    float _670 = dot(in_var_FlowVelocity_FoamSim_Shadow.xy, in_var_FlowVelocity_FoamSim_Shadow.xy);
    bool _671 = _670 > 0.0;
    vec3 _1031;
    do
    {
        float _678 = sqrt(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldMeshScale);
        float _681 = WaterMiscParams.g_xNormalMapScales.x / g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldMeshScale;
        float _682 = 0.5 * _681;
        float _683 = 0.5 * _678;
        _686 = FrameConstants.g_fScaledTime;
        vec2 _688 = _290 + ((WaterMiscParams.g_xNormalMapScrollSpeeds1 * _683) * _686);
        float _689 = 0.70710670948028564453125 * _678;
        vec2 _692 = _290 + ((WaterMiscParams.g_xNormalMapScrollSpeeds1 * _689) * _686);
        float _695 = WaterMiscParams.g_xNormalMapScales.y / g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldMeshScale;
        float _696 = 0.5 * _695;
        vec2 _699 = _290 + ((WaterMiscParams.g_xNormalMapScrollSpeeds2 * _683) * _686);
        vec2 _702 = _290 + ((WaterMiscParams.g_xNormalMapScrollSpeeds2 * _689) * _686);
        vec3 _821;
        if (_671)
        {
            vec2 _740 = fract(vec2(_686 * 0.666666686534881591796875) + vec2(0.0, 0.5)) - vec2(0.5);
            vec2 _743 = vec2(1.0) - (abs(_740) * 2.0);
            vec2 _744 = _740 * 1.5;
            vec2 _748 = in_var_FlowVelocity_FoamSim_Shadow.xy * _744.x;
            float _754 = 1.0 - in_var_OriginalWorldPos_LodAlpha.w;
            float _755 = _743.x;
            float _756 = _754 * _755;
            vec2 _759 = in_var_FlowVelocity_FoamSim_Shadow.xy * _744.y;
            float _765 = _743.y;
            float _766 = _754 * _765;
            float _791 = in_var_OriginalWorldPos_LodAlpha.w * _755;
            float _800 = in_var_OriginalWorldPos_LodAlpha.w * _765;
            vec2 _819 = (((((((SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_688 + _748) * _681).xy * _756).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_688 + _759) * _681).xy * _766)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_699 + _748) * _695).xy * _756)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_699 + _759) * _695).xy * _766)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_692 + _748) * _682).xy * _791)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_692 + _759) * _682).xy * _800)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_702 + _748) * _696).xy * _791)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), (_702 + _759) * _696).xy * _800);
            _821 = vec3(_819.x, _819.y, vec3(0.0, 0.0, 1.0).z);
        }
        else
        {
            float _712 = 1.0 - in_var_OriginalWorldPos_LodAlpha.w;
            vec2 _734 = (((SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), _688 * _681).xy * _712).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), _699 * _695).xy * _712)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), _692 * _682).xy * in_var_OriginalWorldPos_LodAlpha.w)).xy + (SAMPLE_TEXTURE(SAMPLER_2D(g_xNormalMapTexture, g_xFilteredWrap), _702 * _696).xy * in_var_OriginalWorldPos_LodAlpha.w);
            _821 = vec3(_734.x, _734.y, vec3(0.0, 0.0, 1.0).z);
        }
        _822 = _821.x;
        vec2 _826 = (_821.xy - vec2(1.0)).xy * in_var_SubsurfaceScatterColour_NormalMapStrength.w;
        vec3 _830 = vec3(_826.x, _826.y, _821.z);
        _830.y = -_826.y;
        vec3 _836 = _830;
        _836.z = sqrt(abs(1.0 - dot(_830.xy, _830.xy)));
        vec4 _850 = RainData.g_xRainReachabilityViewProj * vec4(in_var_OriginalWorldPos_LodAlpha.xyz, 1.0);
        vec2 _853 = (_850.xy * 0.5) + vec2(0.5);
        vec3 _857 = vec3(_853.x, _853.y, _850.z);
        _857.y = 1.0 - _853.y;
        vec3 _1018;
        if (((ShadingParameters.m_uNumRainRippleLayers > 0u) && (FrameConstants.g_fRainBaseWetness > 0.0)) && ((_850.z - (float(all(lessThanEqual(abs((_857 * vec3(0.5, 0.5, 1.0)) + vec3(0.0, 0.0, -0.5)), vec3(0.5)))) * TEXTURE_LOD(SAMPLER_2D(g_xRainReachabilityTexture, g_xPointClamp), _857.xy, 0.0).x)) > 0.0))
        {
            vec3 _878;
            _878 = vec3(0.0, 0.0, 1.0);
            vec3 _879;
            for (uint _881 = 0u; _881 < ShadingParameters.m_uNumRainRippleLayers; _878 = _879, _881++)
            {
                if (_881 == 0u)
                {
                    vec2 _960 = vec2(_246[_881].y, _246[_881].z);
                    vec3 _1009;
                    do
                    {
                        float _965 = _686 + _246[_881].w;
                        vec2 _973;
                        if (ShadingParameters.m_uNumRainRippleLayers > 1u)
                        {
                            _973 = _960 + vec2(floor(_965) * 3.141592502593994140625);
                        }
                        else
                        {
                            _973 = _960;
                        }
                        vec4 _979 = SAMPLE_TEXTURE(SAMPLER_2D(g_xRainRippleTexture, g_xBilinearWrap), (_290 + _973) * _246[_881].x);
                        float _980 = _979.w;
                        if (_980 > FrameConstants.g_fRainBaseWetness)
                        {
                            _1009 = vec3(0.0, 0.0, 1.0);
                            break;
                        }
                        float _989;
                        if (ShadingParameters.m_uNumRainRippleLayers <= 1u)
                        {
                            _989 = fract(_980 * 12455.123046875);
                        }
                        else
                        {
                            _989 = 0.0;
                        }
                        float _991 = fract(_965 + _989);
                        _1009 = vec3((((_979.yz * 2.0) - vec2(1.0)) * 0.1500000059604644775390625) * (sin(clamp(((_991 - 1.0) + _979.x) * 20.0, 0.0, 5.0) * 3.141592502593994140625) * (1.0 - _991)), 1.0);
                        break;
                    } while(false);
                    _879 = _1009;
                }
                else
                {
                    vec2 _896 = vec2(_246[_881].y, _246[_881].z);
                    vec3 _945;
                    do
                    {
                        float _901 = _686 + _246[_881].w;
                        vec2 _909;
                        if (ShadingParameters.m_uNumRainRippleLayers > 1u)
                        {
                            _909 = _896 + vec2(floor(_901) * 3.141592502593994140625);
                        }
                        else
                        {
                            _909 = _896;
                        }
                        vec4 _915 = SAMPLE_TEXTURE(SAMPLER_2D(g_xRainRippleTexture, g_xBilinearWrap), (_290 + _909) * _246[_881].x);
                        float _916 = _915.w;
                        if (_916 > FrameConstants.g_fRainBaseWetness)
                        {
                            _945 = vec3(0.0, 0.0, 1.0);
                            break;
                        }
                        float _925;
                        if (ShadingParameters.m_uNumRainRippleLayers <= 1u)
                        {
                            _925 = fract(_916 * 12455.123046875);
                        }
                        else
                        {
                            _925 = 0.0;
                        }
                        float _927 = fract(_901 + _925);
                        _945 = vec3((((_915.yz * 2.0) - vec2(1.0)) * 0.1500000059604644775390625) * (sin(clamp(((_927 - 1.0) + _915.x) * 20.0, 0.0, 5.0) * 3.141592502593994140625) * (1.0 - _927)), 1.0);
                        break;
                    } while(false);
                    vec3 _946 = _878 + vec3(0.0, 0.0, 1.0);
                    vec3 _947 = _945 * vec3(-1.0, -1.0, 1.0);
                    _879 = ((_946 * dot(_946, _947)) / vec3(_946.z)) - _947;
                }
            }
            vec3 _1010 = _836 + vec3(0.0, 0.0, 1.0);
            vec3 _1011 = _878 * vec3(-1.0, -1.0, 1.0);
            _1018 = ((_1010 * dot(_1010, _1011)) / vec3(_1010.z)) - _1011;
        }
        else
        {
            _1018 = _836;
        }
        if ((g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex + 1u) == WaterMiscParams.g_uMinQuadMeshCascade)
        {
            _1031 = mix(_1018, vec3(0.0, 0.0, 1.0), vec3(in_var_OriginalWorldPos_LodAlpha.w));
            break;
        }
        else
        {
            if (g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex >= WaterMiscParams.g_uMinQuadMeshCascade)
            {
                _1031 = vec3(0.0, 0.0, 1.0);
                break;
            }
        }
        _1031 = _1018;
        break;
    } while(false);
    vec3 _1039 = ((normalize(_658) * _1031.x) + (normalize(_657) * _1031.y)) + (_661 * _1031.z);
    bool _1060;
    vec2 _1061;
    do
    {
        if (_300)
        {
            _1060 = true;
            _1061 = ((_296 * WaterDataTextureParams.m_xInputUVScale) + (TEXTURE_LOD(SAMPLER_2D(g_xWaterTileLookupTexture, g_xPointClamp), clamp(_296, vec2(0.0), vec2(1.0)) * WaterDataTextureParams.m_xTileLookupUVScale, 0.0).xy * 32767.0)) * WaterDataTextureParams.m_xTiledUVScale;
            break;
        }
        _1060 = false;
        _1061 = vec2(0.0);
        break;
    } while(false);
    uint _1074;
    uint _1075;
    uint _1076;
    if (_1060)
    {
        uvec2 _1069 = uvec2(TEXTURE_LOD(SAMPLER_2D(g_xCubeMapLookupTexture, g_xPointClamp), _1061, 0.0).xy * 255.0);
        uint _1071 = _1069.y;
        _1074 = _1071 >> 1u;
        _1075 = _1071 & 1u;
        _1076 = _1069.x;
    }
    else
    {
        _1074 = 0u;
        _1075 = 0u;
        _1076 = 0u;
    }
    vec4 _1081 = vec4(in_var_WorldSpacePos_Distance.xyz, 1.0);
    vec3 _1085 = (_1081 * g_axPBRCubemapDatas._m0[_1076].m_xObbSpace).xyz;
    float _1097;
    if (_1075 == 1u)
    {
        vec3 _1093 = clamp((clamp(abs(_1085), vec3(0.0), vec3(1.0)) - vec3(0.89999997615814208984375)) * vec3(9.9999980926513671875), vec3(0.0), vec3(1.0));
        _1097 = max(_1093.x, _1093.z);
    }
    else
    {
        _1097 = 0.0;
    }
    float _1101;
    bool _1111;
    vec3 _1121;
    do
    {
        vec3 _1100 = _661 * vec3(1.0, -1.0, 1.0);
        _1101 = float(_1076);
        float _1102 = _1100.x;
        vec3 _1110 = textureLod(samplerCubeArray(g_axPBRCubeMaps, g_xFilteredClamp), vec4(_1102, _1100.yz, _1101), 6.0).xyz;
        _1111 = _1097 == 0.0;
        if (_1111)
        {
            _1121 = _1110;
            break;
        }
        _1121 = mix(_1110, textureLod(samplerCubeArray(g_axPBRCubeMaps, g_xFilteredClamp), vec4(_1102, _1100.yz, float(_1074)), 6.0).xyz, vec3(_1097));
        break;
    } while(false);
    vec3 _1126 = FrameConstants.g_xCamPos - in_var_WorldSpacePos_Distance.xyz;
    vec3 _1127 = normalize(_1126);
    float _1178;
    if (ProjectedCloudParams.g_bSunlightCloudsEnabled != 0u)
    {
        float _1177;
        if (abs(ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y) > 9.9999999747524270787835121154785e-07)
        {
            vec3 _1141 = _1081.xyz;
            float _1144 = (vec3(0.0, -10000.0, 0.0) - _1141).y / ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y;
            float _1176;
            if (_1144 > 0.0)
            {
                vec3 _1150 = _1141 + (ProjectedCloudParams.g_xSunlightCloudsProjectionDir.xyz * _1144);
                vec2 _1157 = _1150.xz;
                _1157.x = _1150.x + (_686 * ProjectedCloudParams.g_fSunlightCloudsVelocityX);
                vec2 _1163 = _1157;
                _1163.y = _1150.z + (_686 * ProjectedCloudParams.g_fSunlightCloudsVelocityY);
                vec2 _1167 = _1163 / vec2(ProjectedCloudParams.g_fSunlightCloudsScale);
                vec2 _1170 = _1167;
                _1170.x = _1167.x * (-1.0);
                _1176 = TEXTURE_LOD(SAMPLER_2D(g_xLighting_Clouds, g_xBilinearWrap), _1170, 0.0).x;
            }
            else
            {
                _1176 = 1.0;
            }
            _1177 = _1176;
        }
        else
        {
            _1177 = 1.0;
        }
        _1178 = _1177;
    }
    else
    {
        _1178 = 1.0;
    }
    float _1179 = in_var_FlowVelocity_FoamSim_Shadow.w * _1178;
    bool _1183 = in_var_FlowVelocity_FoamSim_Shadow.z > 0.00999999977648258209228515625;
    vec3 _1188 = -_1127;
    vec3 _1197 = ((ShadingParameters.g_xSunColour * max(0.070000000298023223876953125 * mix(0.5, 1.0, clamp(dot(ShadingParameters.g_xLightDir, _661), 0.0, 1.0)), (0.3499999940395355224609375 * pow(clamp(dot(ShadingParameters.g_xLightDir, _1188), 0.0, 1.0), 7.0)) * _659)) * _1179) * in_var_SubsurfaceScatterColour_NormalMapStrength.xyz;
    vec3 _1201 = mix(_1121, vec3(dot(vec3(0.20999999344348907470703125, 0.7200000286102294921875, 0.070000000298023223876953125), _1121)), vec3(0.699999988079071044921875)) * 0.100000001490116119384765625;
    vec3 _1218;
    if (_671)
    {
        _1218 = _1197 + (_1201 * mix(in_var_SubsurfaceScatterColour_NormalMapStrength.xyz, (in_var_SubsurfaceScatterColourFastFlow_EdgeBlendRadius.xyz * 5.0) * max(_1179, 0.5), vec3(clamp((1.2999999523162841796875 * _822) * _670, 0.0, 1.0) * 0.5)));
    }
    else
    {
        _1218 = _1197 + (_1201 * in_var_SubsurfaceScatterColour_NormalMapStrength.xyz);
    }
    vec2 _1222 = gl_FragCoord.xy * FrameConstants.g_xRcpScreenDims;
    vec4 _1226 = TEXTURE_LOD(SAMPLER_2D(g_xDepthBuffer, g_xPointClamp), _1222, 0.0);
    float _1227 = _1226.x;
    float _1229 = max(_1227 - in_var_WorldSpacePos_Distance.w, 0.0);
    float _1231 = clamp(dot(_1039, _1127), 0.0, 1.0);
    float _1235 = 0.0203731842339038848876953125 + (0.979626834392547607421875 * pow(1.0 - _1231, 5.0));
    vec3 _1240 = vec3(_656.x, -1.0, _656.y);
    vec2 _1244 = _1240.xz + (_1031.xy * 5.0);
    float _1287;
    vec3 _1288;
    do
    {
        if ((!(_1235 > 0.300000011920928955078125)) == false)
        {
            _1287 = _1229;
            _1288 = TEXTURE_LOD(SAMPLER_2D(g_xAlbedoBuffer, g_xTrilinearClamp), _1222, 0.0).xyz;
            break;
        }
        else
        {
            vec2 _1258 = (normalize(vec3(_1244.x, _1240.y, _1244.y)).xz * (0.20000000298023223876953125 * clamp(_1229, 0.0, 0.949999988079071044921875))) / vec2(in_var_WorldSpacePos_Distance.w);
            vec4 _1261 = TEXTURE_LOD(SAMPLER_2D(g_xDepthBuffer, g_xPointClamp), _1222 + _1258, 0.0);
            float _1262 = _1261.x;
            bool _1268 = (_1262 < (in_var_WorldSpacePos_Distance.w + 0.100000001490116119384765625)) || (((_1227 - _1262) / _1262) > 0.5);
            float _1273;
            if (_1268)
            {
                _1273 = _1229;
            }
            else
            {
                _1273 = _1262 - in_var_WorldSpacePos_Distance.w;
            }
            _1287 = _1273;
            _1288 = TEXTURE_LOD(SAMPLER_2D(g_xAlbedoBuffer, g_xTrilinearClamp), _1222 + mix(_1258, vec2(0.0), bvec2(_1268)), 0.0).xyz;
            break;
        }
    } while(false);
    float _1293 = _1287 + (clamp((in_var_WorldSpacePos_Distance.w - 30.0) * 0.014999999664723873138427734375, 0.0, 1.0) * 0.20000000298023223876953125);
    vec3 _1412;
    if (ShadingParameters.g_fEnableCaustics > 0.0)
    {
        vec3 _1411;
        do
        {
            float _1303 = abs(_1293 * _1127.y);
            if (_1303 > 1.5)
            {
                _1411 = _1288;
                break;
            }
            float _1315 = abs(ShadingParameters.g_xLightDir.y);
            int _1321 = int(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex);
            float _1329 = float(g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_uCascadeIndex);
            vec3 _1332 = vec3((((in_var_WorldSpacePos_Distance.xz - (_1127.xz * _1293)) + ((ShadingParameters.g_xLightDir.xz * _1303) / vec2(max(_1315, 0.0500000007450580596923828125)))) - CascadeDefinition.m_xLayer[_1321].m_xWorldPos) / vec2(CascadeDefinition.m_xLayer[_1321].m_fWorldSize), _1329);
            vec2 _1333 = _1332.xy;
            vec3 _1346;
            float _1347;
            if (any(greaterThan(abs(_1333 - vec2(0.5)), vec2(0.4799999892711639404296875))))
            {
                vec2 _1341 = (_1333 * 0.5) + vec2(0.25);
                vec3 _1344 = vec3(_1341.x, _1341.y, _1332.z);
                _1344.z = _1329 + 1.0;
                _1346 = _1344;
                _1347 = g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize * 2.0;
            }
            else
            {
                _1346 = _1332;
                _1347 = g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldTexelSize;
            }
            vec3 _1350 = _1346;
            _1350.z = _1346.z + 1.0;
            vec2 _1354 = vec2(1.0 / CascadeDefinition.m_fResolution, 0.0);
            vec3 _1358 = textureLod(sampler2DArray(g_xGerstnerTexture, g_xBilinearClamp), _1350, 0.0).xyz;
            vec3 _1369 = textureLod(sampler2DArray(g_xGerstnerTexture, g_xBilinearClamp), _1350 + _1354.xyy, 0.0).xyz - _1358;
            vec3 _1372 = textureLod(sampler2DArray(g_xGerstnerTexture, g_xBilinearClamp), _1350 + _1354.yxy, 0.0).xyz - _1358;
            float _1384 = clamp(((-0.125) * ((((_1369.x + _1347) * (_1372.z + _1347)) - (_1369.z * _1372.x)) / (_1347 * _1347))) + 1.0, 0.0, 1.0);
            float _1388 = clamp(dFdx(_1384) + dFdy(_1384), -0.00999999977648258209228515625, 0.00999999977648258209228515625);
            vec3 _1390 = _265;
            _1390.x = _1384 + _1388;
            vec3 _1393 = _1390;
            _1393.y = _1384 - (2.0 * _1388);
            vec3 _1395 = _1393;
            _1395.z = _1384 - _1388;
            _1411 = _1288 * mix(vec3(1.0), min((log((vec3(1.2000000476837158203125) + (pow(_1395, vec3(15.0)) * 2.0)) + (pow(_1395, vec3(90.0)) * 500.0)) * 2.5) * _1315, vec3(5.0)), vec3(_1179 * smoothstep(_1303, 1.5, 1.2000000476837158203125)));
            break;
        } while(false);
        _1412 = _1411;
    }
    else
    {
        _1412 = _1288;
    }
    vec3 _1421;
    if (_671)
    {
        _1421 = in_var_FogTransmissionAlpha_WaterLevelDz.xyz * (1.0 + min(0.5, (_822 * 2.0) * _670));
    }
    else
    {
        _1421 = in_var_FogTransmissionAlpha_WaterLevelDz.xyz;
    }
    vec4 _1548;
    if (_1183)
    {
        float _1428 = 1.08000004291534423828125 / g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldMeshScale;
        float _1429 = 2.1600000858306884765625 / g_axWaterMeshInstanceData._m0[in_var_InstanceId].m_fWorldMeshScale;
        vec3 _1494;
        if (_671)
        {
            vec2 _1451 = fract(vec2(_686 * 1.369863033294677734375) + vec2(0.0, 0.5)) - vec2(0.5);
            vec2 _1454 = vec2(1.0) - (abs(_1451) * 2.0);
            vec2 _1455 = _1451 * 0.730000019073486328125;
            vec2 _1458 = _290 + (in_var_FlowVelocity_FoamSim_Shadow.xy * _1455.x);
            vec2 _1463 = _290 + (in_var_FlowVelocity_FoamSim_Shadow.xy * _1455.y);
            float _1471 = _1454.x;
            float _1477 = 1.0 - in_var_OriginalWorldPos_LodAlpha.w;
            float _1484 = _1454.y;
            _1494 = (((SAMPLE_TEXTURE(SAMPLER_2D(g_xFoamDetailTexture, g_xFilteredWrap), _1458 * _1428).xyz * (in_var_OriginalWorldPos_LodAlpha.w * _1471)) + (SAMPLE_TEXTURE(SAMPLER_2D(g_xFoamDetailTexture, g_xFilteredWrap), _1458 * _1429).xyz * (_1477 * _1471))) + (SAMPLE_TEXTURE(SAMPLER_2D(g_xFoamDetailTexture, g_xFilteredWrap), _1463 * _1428).xyz * (in_var_OriginalWorldPos_LodAlpha.w * _1484))) + (SAMPLE_TEXTURE(SAMPLER_2D(g_xFoamDetailTexture, g_xFilteredWrap), _1463 * _1429).xyz * (_1477 * _1484));
        }
        else
        {
            _1494 = (SAMPLE_TEXTURE(SAMPLER_2D(g_xFoamDetailTexture, g_xFilteredWrap), _290 * _1428).xyz * in_var_OriginalWorldPos_LodAlpha.w) + (SAMPLE_TEXTURE(SAMPLER_2D(g_xFoamDetailTexture, g_xFilteredWrap), _290 * _1429).xyz * (1.0 - in_var_OriginalWorldPos_LodAlpha.w));
        }
        float _1496 = pow(in_var_FlowVelocity_FoamSim_Shadow.z, 1.7000000476837158203125);
        vec2 _1504 = _1494.yz - vec2(0.5);
        vec3 _1510 = normalize(vec3(-_1504.x, -0.300000011920928955078125, -_1504.y));
        vec3 _1536;
        do
        {
            vec3 _1519 = _1510 * vec3(1.0, -1.0, 1.0);
            float _1520 = _1519.x;
            vec3 _1526 = textureLod(samplerCubeArray(g_axPBRCubeMaps, g_xFilteredClamp), vec4(_1520, _1519.yz, _1101), 6.0).xyz;
            if (_1111)
            {
                _1536 = _1526;
                break;
            }
            _1536 = mix(_1526, textureLod(samplerCubeArray(g_axPBRCubeMaps, g_xFilteredClamp), vec4(_1520, _1519.yz, float(_1074)), 6.0).xyz, vec3(_1097));
            break;
        } while(false);
        _1548 = vec4(((_1536 + (((ShadingParameters.g_xSunColour * max(0.0, dot(_1510, ShadingParameters.g_xLightDir))) * max(_1179, 0.0199999995529651641845703125)) * vec3(0.31830990314483642578125))) * 0.89999997615814208984375) * in_var_FoamColour.xyz, in_var_FoamColour.w * min(smoothstep((1.2000000476837158203125 - _1496) - min(0.25, in_var_WorldSpacePos_Distance.w * 0.0030000000260770320892333984375), 1.39999997615814208984375 - _1496, _1494.x), 1.0));
    }
    else
    {
        _1548 = vec4(0.0);
    }
    vec3 _1549 = reflect(_1188, _1039);
    vec3 _1552 = _1549;
    _1552.y = min(_1549.y, 0.0);
    vec3 _1668;
    do
    {
        vec3 _1563 = _1552 * mat3(g_axPBRCubemapDatas._m0[_1076].m_xObbSpace[0].xyz, g_axPBRCubemapDatas._m0[_1076].m_xObbSpace[1].xyz, g_axPBRCubemapDatas._m0[_1076].m_xObbSpace[2].xyz);
        vec3 _1571 = (vec3(1.0) - (mix(vec3(0.0), vec3(1.0), lessThan(_1563, vec3(0.0))) * 2.0)) * (vec3(1.0) / (abs(_1563) + vec3(1.0000000133514319600180897396058e-10)));
        vec3 _1576 = max((vec3(1.0) - _1085) * _1571, (vec3(-1.0) - _1085) * _1571);
        vec3 _1586 = (_1085 + (_1563 * min(_1576.x, min(_1576.y, _1576.z)))) - g_axPBRCubemapDatas._m0[_1076].m_xCubemapPosition;
        vec3 _1589 = _1586;
        _1589.y = -_1586.y;
        vec4 _1605 = texture(samplerCubeArray(g_axPBRCubeMaps, g_xFilteredClamp), vec4(_1589 * mat3(g_axPBRCubemapDatas._m0[_1076].m_xInvObbSpace[0].xyz, g_axPBRCubemapDatas._m0[_1076].m_xInvObbSpace[1].xyz, g_axPBRCubemapDatas._m0[_1076].m_xInvObbSpace[2].xyz), _1101));
        vec3 _1606 = _1605.xyz;
        if (_1111)
        {
            _1668 = _1606;
            break;
        }
        vec3 _1612 = (_1081 * g_axPBRCubemapDatas._m0[_1074].m_xObbSpace).xyz;
        vec3 _1621 = _1552 * mat3(g_axPBRCubemapDatas._m0[_1074].m_xObbSpace[0].xyz, g_axPBRCubemapDatas._m0[_1074].m_xObbSpace[1].xyz, g_axPBRCubemapDatas._m0[_1074].m_xObbSpace[2].xyz);
        vec3 _1629 = (vec3(1.0) - (mix(vec3(0.0), vec3(1.0), lessThan(_1621, vec3(0.0))) * 2.0)) * (vec3(1.0) / (abs(_1621) + vec3(1.0000000133514319600180897396058e-10)));
        vec3 _1634 = max((vec3(1.0) - _1612) * _1629, (vec3(-1.0) - _1612) * _1629);
        vec3 _1644 = (_1612 + (_1621 * min(_1634.x, min(_1634.y, _1634.z)))) - g_axPBRCubemapDatas._m0[_1074].m_xCubemapPosition;
        vec3 _1647 = _1644;
        _1647.y = -_1644.y;
        _1668 = mix(_1606, texture(samplerCubeArray(g_axPBRCubeMaps, g_xFilteredClamp), vec4(_1647 * mat3(g_axPBRCubemapDatas._m0[_1074].m_xInvObbSpace[0].xyz, g_axPBRCubemapDatas._m0[_1074].m_xInvObbSpace[1].xyz, g_axPBRCubemapDatas._m0[_1074].m_xInvObbSpace[2].xyz), float(_1074))).xyz, vec3(_1097));
        break;
    } while(false);
    vec2 _1679;
    if ((_1126.x == 0.0) && (_1126.z == 0.0))
    {
        _1679 = vec2(0.0);
    }
    else
    {
        _1679 = normalize(_1126.xz);
    }
    vec4 _1692 = SAMPLE_TEXTURE(SAMPLER_2D(g_xReflectionBuffer, g_xTrilinearClamp), _1222 + ((_1039.xz * mat2(vec2(-_1679.y, _1679.x), _1679)) * 2.0));
    vec3 _1696 = mix(_1668, _1692.xyz, vec3(_1692.w));
    vec3 _1704;
    if (_1183)
    {
        _1704 = _1696 * (1.0 - clamp(2.5 * _1548.w, 0.0, 1.0));
    }
    else
    {
        _1704 = _1696;
    }
    float _1714 = mix(max(0.02999999932944774627685546875, in_var_RoughnessFactor * 0.20000000298023223876953125), max(0.02999999932944774627685546875, in_var_RoughnessFactor), clamp((in_var_WorldSpacePos_Distance.w - 150.0) * 0.000624999986030161380767822265625, 0.0, 1.0));
    float _1715 = _1714 * _1714;
    vec3 _1717 = normalize(_1127 + ShadingParameters.g_xLightDir);
    float _1719 = clamp(dot(_1039, ShadingParameters.g_xLightDir), 0.0, 1.0);
    float _1721 = clamp(dot(_1039, _1717), 0.0, 1.0);
    float _1723 = clamp(dot(_1127, _1717), 0.0, 1.0);
    float _1731 = 2.0 - _1715;
    float _1738 = _1715 * _1715;
    float _1742 = (((_1721 * _1738) - _1721) * _1721) + 1.0;
    vec3 _1756 = mix((_1412 * exp(_1421 * (_1293 + 0.100000001490116119384765625))) + _1218, _1704, in_var_ReflectionColour_WaterLevelDx.xyz * _1235).xyz + (sqrt(max(vec3(((vec3(0.980000019073486328125) * exp2((((-5.554729938507080078125) * _1723) - 6.9831600189208984375) * _1723)) + vec3(0.0199999995529651641845703125)).x * (((1.0 / (((_1231 * _1731) + _1715) * ((_1719 * _1731) + _1715))) * (_1738 * (1.0 / (_1742 * _1742)))) * _1719)) * ShadingParameters.g_xSunColour, vec3(0.0))) * (_1179 * _1179));
    vec4 _1766;
    if (_1183)
    {
        vec3 _1764 = mix(_1756.xyz, _1548.xyz, vec3(_1548.w));
        _1766 = vec4(_1764.x, _1764.y, _1764.z, vec4(0.0, 0.0, 0.0, 1.0).w);
    }
    else
    {
        _1766 = vec4(_1756.x, _1756.y, _1756.z, vec4(0.0, 0.0, 0.0, 1.0).w);
    }
    vec4 _2075;
    if (FogParams.g_bHeightFogEnabled != 0u)
    {
        vec3 _1892 = (vec4((FrameConstants.g_xCameraTanHalfFOV * in_var_WorldSpacePos_Distance.w) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), in_var_WorldSpacePos_Distance.w, 1.0) * FrameConstants.g_xInvView).xyz - FrameConstants.g_xCamPos;
        
        //vec4 temp = vec4((FrameConstants.g_xCameraTanHalfFOV * in_var_WorldSpacePos_Distance.w) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), in_var_WorldSpacePos_Distance.w, 1.0);
    
		//temp.x -= vk3d_params[VIEW].stereo.x * (_158 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];	
//		vec3 _179 = (temp * FrameConstants.g_xInvView).xyz;
        
        float _1893 = length(_1892);
        vec3 _1895 = -normalize(_1892);
        vec4 _1999;
        if (_1893 > FogParams.g_fHeightFogStartDistance)
        {
            float _1903 = max(_1893 - FogParams.g_fHeightFogStartDistance, 0.0);
            float _1927;
            if (_1893 >= FrameConstants.g_xCameraFarClipValues.x)
            {
                _1927 = float(int(sign(_1895.y))) * 1.57079601287841796875;
            }
            else
            {
                _1927 = atan((((_1895.y * _1903) - FrameConstants.g_xCamPos.y) - FogParams.g_fHeightFogHeight) / FogParams.g_fHeightFogFalloff);
            }
            float _1928 = _1895.y;
            float _1963;
            if (abs(_1928) <= 9.9999999747524270787835121154785e-07)
            {
                float _1958 = (-FrameConstants.g_xCamPos.y) - FogParams.g_fHeightFogHeight;
                _1963 = (((FogParams.g_fHeightFogStrength * FogParams.g_fHeightFogFalloff) * _1903) * FogParams.g_fHeightFogFalloff) / ((_1958 * _1958) + (FogParams.g_fHeightFogFalloff * FogParams.g_fHeightFogFalloff));
            }
            else
            {
                _1963 = ((FogParams.g_fHeightFogStrength * FogParams.g_fHeightFogFalloff) * (_1927 - atan(((-FogParams.g_fHeightFogHeight) - FrameConstants.g_xCamPos.y) / FogParams.g_fHeightFogFalloff))) / _1928;
            }
            float _1974 = clamp(max(-dot(FogParams.g_xSunDirection, _1895), 0.0) - FogParams.g_fCosScatteringAngle, 0.0, 1.0) / (1.0 - FogParams.g_fCosScatteringAngle);
            _1999 = vec4(FogParams.g_xHeightFogColour + ((FogParams.g_xDirectColour * clamp(max(_1893 - FogParams.g_fDirectScatterStartDistance, 0.0) / FogParams.g_fDirectScatterFadeDistance, 0.0, 1.0)) * (_1974 * _1974)), clamp(_1963, 0.0, 1.0) * FogParams.g_fHeightFogMaxOpacity);
        }
        else
        {
            _1999 = vec4(0.0);
        }
        vec4 _2074;
        if (FogParams.g_bTexturedFogEnabled != 0u)
        {
            float _2010 = (in_var_WorldSpacePos_Distance.w - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
            float _2014 = clamp(_2010, 0.0, 1.0);
            float _2023 = (FogParams.g_fDensityPower > 0.0) ? (pow(_2014, FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _2010).x;
            float _2038 = atan(-_1127.z, -_1127.x) * 0.159155070781707763671875;
            vec2 _2048 = vec2(_2038, (acos(min(max((_1127.y * FogParams.g_fHorizonScale) + FogParams.g_fHorizonOffset, -1.0), 1.0)) * FogParams.g_fVerticalTextureScale) * 0.31831014156341552734375);
            _2048.x = _2038 + FogParams.g_fTextureUOffset;
            _2074 = vec4(mix(vec4(TEXTURE_LOD(SAMPLER_2D(g_xFogColourTexture, g_xFilteredWrap), _2048, (1.0 - _2014) * 8.0).xyz * FogParams.g_fTextureHDRScale, _2023).xyz, _1999.xyz, vec3(_1999.w)), clamp(_1999.w + _2023, 0.0, 1.0));
        }
        else
        {
            _2074 = _1999;
        }
        _2075 = _2074;
    }
    else
    {
        vec4 _1873;
        if (FogParams.g_bFogEnabled != 0u)
        {
            float _1855 = (in_var_WorldSpacePos_Distance.w - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
            _1873 = vec4(FogParams.g_xFogColour, (FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_1855, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _1855).x);
        }
        else
        {
            vec4 _1847;
            if (FogParams.g_bTexturedFogEnabled != 0u)
            {
                float _1794 = (in_var_WorldSpacePos_Distance.w - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                float _1798 = clamp(_1794, 0.0, 1.0);
                float _1822 = atan(-_1127.z, -_1127.x) * 0.159155070781707763671875;
                vec2 _1832 = vec2(_1822, (acos(min(max((_1127.y * FogParams.g_fHorizonScale) + FogParams.g_fHorizonOffset, -1.0), 1.0)) * FogParams.g_fVerticalTextureScale) * 0.31831014156341552734375);
                _1832.x = _1822 + FogParams.g_fTextureUOffset;
                _1847 = vec4(TEXTURE_LOD(SAMPLER_2D(g_xFogColourTexture, g_xFilteredWrap), _1832, (1.0 - _1798) * 8.0).xyz * FogParams.g_fTextureHDRScale, (FogParams.g_fDensityPower > 0.0) ? (pow(_1798, FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _1794).x);
            }
            else
            {
                _1847 = vec4(0.0);
            }
            _1873 = _1847;
        }
        _2075 = _1873;
    }
    out_var_SV_Target = vec4(mix(mix(_1288, _1766.xyz, vec3(clamp(_1293 / in_var_SubsurfaceScatterColourFastFlow_EdgeBlendRadius.w, 0.0, 1.0))).xyz, _2075.xyz * 1.0, vec3(_2075.w)), _1766.w);
}


