// LIGHTS 5
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRangeUnused;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec3 _126;

void main()
{
    ivec2 _136 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _144 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _136, 0).x);
    vec4 _159 = vec4((FrameConstants.g_xCameraTanHalfFOV * _144) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _144, 1.0);
    
    _159.x -= vk3d_params[VIEW].stereo.x * (_144 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];
    
    vec4 _187 = TEXEL_FETCH(g_xLighting_NormalBuffer, _136, 0);
    vec4 _189 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _136, 0);
    vec4 _191 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _136, 0);
    vec3 _195 = normalize((_187.xyz * 2.0) - vec3(1.0));
    vec3 _196 = _189.xyz;
    float _197 = _191.y;
    float _198 = _189.w;
    float _199 = _191.x;
    vec2 _202 = (_191.zw * 2.0) - vec2(1.0);
    vec2 _204 = _202.xy;
    float _208 = sqrt(clamp(1.0 - dot(_204, _204), 0.0, 1.0));
    vec3 _213 = vec3(_202.x, _202.y, _126.z);
    _213.z = (_187.w != 0.0) ? _208 : (-_208);
    vec3 _214 = normalize(in_var_CamDir);
    vec4 _523;
    bool _524;
    do
    {
        bool _236;
        do
        {
            vec4 _219 = _159 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            if (_219.z < 0.0)
            {
                _236 = false;
                break;
            }
            if (any(lessThan(vec2(0.5) - abs((_219.xy / vec2(_219.w)).xy - vec2(0.5)), vec2(0.0))))
            {
                _236 = false;
                break;
            }
            _236 = true;
            break;
        } while(false);
        if (!_236)
        {
            _523 = vec4(0.0, 0.0, 0.0, 1.0);
            _524 = false;
            break;
        }
        vec3 _244 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_159 * FrameConstants.g_xInvView).xyz;
        float _245 = length(_244);
        vec3 _247 = _244 / vec3(_245);
        float _256 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_245 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_256 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _523 = vec4(0.0, 0.0, 0.0, 1.0);
            _524 = false;
            break;
        }
        float _286;
        vec3 _287;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius > 0.0)
        {
            vec3 _273 = -reflect(_214, _195);
            vec3 _276 = (_273 * dot(_244, _273)) - _244;
            _286 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius * inversesqrt(dot(_244, _244));
            _287 = normalize(_244 + (_276 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius / length(_276), 0.0, 1.0)));
        }
        else
        {
            _286 = 0.0;
            _287 = _247;
        }
        float _289 = (_198 != 0.01960784383118152618408203125) ? _199 : 0.0;
        vec3 _291 = normalize(_214 + _287);
        float _293 = clamp(dot(_195, _247), 0.0, 1.0);
        float _295 = clamp(dot(_195, _214), 0.0, 1.0);
        float _297 = clamp(dot(_195, _291), 0.0, 1.0);
        float _299 = clamp(dot(_214, _291), 0.0, 1.0);
        bool _300 = _198 == 0.01568627543747425079345703125;
        vec3 _337;
        vec3 _338;
        vec3 _339;
        if (_300)
        {
            _337 = vec3(0.046999998390674591064453125);
            _338 = vec3(_293);
            _339 = _196;
        }
        else
        {
            vec3 _333;
            vec3 _334;
            vec3 _335;
            if (_198 == 0.007843137718737125396728515625)
            {
                _333 = vec3(0.0280000008642673492431640625);
                _334 = vec3(_293);
                _335 = _196;
            }
            else
            {
                vec3 _329;
                vec3 _330;
                vec3 _331;
                if (_198 == 0.0039215688593685626983642578125)
                {
                    _329 = vec3(0.0280000008642673492431640625);
                    _330 = vec3((clamp((clamp(dot(_247, -_195), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_247, -_214), 0.0, 1.0), 7.0), 0.0, 1.0) * _289) + _293);
                    _331 = _196;
                }
                else
                {
                    vec3 _312 = vec3(_289);
                    _329 = mix(vec3(0.039999999105930328369140625), _196, _312);
                    _330 = vec3(_293);
                    _331 = mix(_196, vec3(0.0), _312);
                }
                _333 = _329;
                _334 = _330;
                _335 = _331;
            }
            _337 = _333;
            _338 = _334;
            _339 = _335;
        }
        vec3 _340 = _338 * _339;
        float _341 = max(_197, 0.0500000007450580596923828125);
        float _342 = _341 * _341;
        vec3 _455;
        if (_300)
        {
            float _393 = clamp(1.0 - _197, 0.0, 1.0);
            float _395 = ((_199 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _401 = pow(_393, 8.0) * 500.0;
            float _404 = pow(_393, 6.0) * 2000.0;
            float _418 = dot(normalize(_213 + (_195 * ((-0.0500000007450580596923828125) + _395))), _291);
            float _438 = dot(normalize(_213 + (_195 * ((0.0500000007450580596923828125 + _395) - (_393 * 0.0500000007450580596923828125)))), _291);
            _455 = _340 + ((((vec3(((smoothstep(-1.0, 0.0, _418) * pow(sqrt(clamp(1.0 - (_418 * _418), 0.0, 1.0)), 20.0 + _401)) * ((((_199 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _393))) * ((_401 + 28.0) * 0.03978873789310455322265625)) * (_196 / vec3(max(dot(_196, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _438) * pow(sqrt(clamp(1.0 - (_438 * _438), 0.0, 1.0)), 50.0 + _404)) * ((((_199 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _393)) * ((_404 + 58.0) * 0.03978873789310455322265625))) * _293) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
        }
        else
        {
            float _348 = clamp(_286 * (1.0 - _342), 0.0, 1.0);
            float _368;
            if (_348 > 0.0)
            {
                float _359 = _342 * _342;
                _368 = _359 / (_359 + (((_348 * 0.25) * ((3.0 * _342) + _348)) / (_299 + 0.001000000047497451305389404296875)));
            }
            else
            {
                _368 = 1.0;
            }
            float _369 = 2.0 - _342;
            float _376 = _342 * _342;
            float _380 = (((_297 * _376) - _297) * _297) + 1.0;
            _455 = _340 + (((((vec3(1.0) - _337) * exp2((((-5.554729938507080078125) * _299) - 6.9831600189208984375) * _299)) + _337) * ((((1.0 / (((_295 * _369) + _342) * ((_293 * _369) + _342))) * (_376 * (1.0 / (_380 * _380)))) * _293) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _368);
        }
        vec3 _505;
        if (_198 == 0.01960784383118152618408203125)
        {
            bool _459 = _199 > 0.4980392158031463623046875;
            vec3 _477 = sqrt(abs(_339));
            float _491 = 0.5 / _342;
            _505 = mix(_455, (_340 + (((((vec3(1.0) - _477) * exp2((((-5.554729938507080078125) * _299) - 6.9831600189208984375) * _299)) + _477) * ((1.0 / ((4.0 * ((_293 + _295) - (_293 * _295))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _491) * pow(1.0 - (_297 * _297), _491)))) * (_293 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_339 * (clamp((clamp(dot(_247, -_195), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_247, -_214), 0.0, 1.0), 7.0), 0.0, 1.0) * (_459 ? 0.5 : 0.0))), vec3((_459 ? (_199 - 0.501960813999176025390625) : _199) * 2.007874011993408203125));
        }
        else
        {
            _505 = _455;
        }
        vec3 _508 = min(_505 * (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * (_256 * clamp(((-dot(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz, _247)) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffOfs) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffScale, 0.0, 1.0))), vec3(1000.0)).xyz;
        vec3 _521;
        if ((_198 == 0.007843137718737125396728515625) && (_144 < 3.0))
        {
            _521 = _508 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_144 - 2.0, 0.0)));
        }
        else
        {
            _521 = _508;
        }
        _523 = vec4(_521.x, _521.y, _521.z, vec4(0.0, 0.0, 0.0, 1.0).w);
        _524 = true;
        break;
    } while(false);
    if (false == _524)
    {
        discard;
    }
    out_var_SV_Target = _523;
}


