// Dynamic Lights
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRange;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fAreaLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 3, binding = 0) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xTextureArray[];
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec3 _200;
vec4 _201;
vec2 _202;

void main()
{
    ivec2 _212 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _220 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _212, 0).x);
    vec4 _235 = vec4((FrameConstants.g_xCameraTanHalfFOV * _220) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _220, 1.0);
    
    _235.x -= vk3d_params[VIEW].stereo.x * (_220 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0]; 
    
    vec4 _284 = TEXEL_FETCH(g_xLighting_NormalBuffer, _212, 0);
    vec4 _286 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _212, 0);
    vec4 _288 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _212, 0);
    vec3 _292 = normalize((_284.xyz * 2.0) - vec3(1.0));
    vec3 _293 = _286.xyz;
    float _294 = _288.y;
    float _295 = _286.w;
    float _296 = _288.x;
    vec2 _299 = (_288.zw * 2.0) - vec2(1.0);
    vec2 _301 = _299.xy;
    float _305 = sqrt(clamp(1.0 - dot(_301, _301), 0.0, 1.0));
    vec3 _310 = vec3(_299.x, _299.y, _200.z);
    _310.z = (_284.w != 0.0) ? _305 : (-_305);
    vec3 _311 = normalize(in_var_CamDir);
    vec4 _1007;
    bool _1008;
    do
    {
        vec4 _336;
        bool _337;
        do
        {
            vec4 _316 = _235 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            float _317 = _316.z;
            if (_317 < 0.0)
            {
                _336 = _201;
                _337 = false;
                break;
            }
            vec2 _324 = _316.xy / vec2(_316.w);
            vec4 _327 = vec4(_324, _317, _316.w);
            if (any(lessThan(vec2(0.5) - abs(_324.xy - vec2(0.5)), vec2(0.0))))
            {
                _336 = _327;
                _337 = false;
                break;
            }
            _336 = _327;
            _337 = true;
            break;
        } while(false);
        if (!_337)
        {
            _1007 = vec4(0.0, 0.0, 0.0, 1.0);
            _1008 = false;
            break;
        }
        vec3 _345 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_235 * FrameConstants.g_xInvView).xyz;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_bCustomBoundingVolumeSet != 0u)
        {
            float _350 = -dot(_345, g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz);
            if ((_350 < g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fMinRange) || (_350 > g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange))
            {
                _1007 = vec4(0.0, 0.0, 0.0, 1.0);
                _1008 = false;
                break;
            }
        }
        float _356 = length(_345);
        vec3 _358 = _345 / vec3(_356);
        float _367 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_356 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_367 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _1007 = vec4(0.0, 0.0, 0.0, 1.0);
            _1008 = false;
            break;
        }
        vec4 _739;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex != (-1))
        {
            float _383 = (_336.z + max(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias * (1.0 - clamp(dot(_292, _358), 0.0, 1.0)), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias)) / _336.w;
            float _389 = (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane;
            float _390 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _391 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _397 = clamp(((_389 / (_390 - ((((1.0 - _383) * 2.0) - 1.0) * _391))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _391, 0.0, 1.0);
            vec4 _738;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength > 0.0)
            {
                float _734;
                if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness > 0.0)
                {
                    float _472 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                    float _473 = sin(_472);
                    float _474 = cos(_472);
                    float _476 = (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov;
                    float _478 = float(uint(0));
                    float _481 = sqrt(_478 + 0.5) * 0.5;
                    float _483 = (_478 * 2.400000095367431640625) + _472;
                    uint _490 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    vec4 _495 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(_490)], g_xBilinearClamp)), _336.xy + (vec2(_481 * cos(_483), _481 * sin(_483)) * _476), FrameConstants.g_fGlobalSamplerBias);
                    float _505 = clamp(((_389 / (_390 - ((((1.0 - _495.x) * 2.0) - 1.0) * _391))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _391, 0.0, 1.0);
                    vec2 _511;
                    if (_505 < _397)
                    {
                        vec2 _509 = _202;
                        _509.x = _505;
                        vec2 _510 = _509;
                        _510.y = 1.0;
                        _511 = _510;
                    }
                    else
                    {
                        _511 = vec2(0.0);
                    }
                    float _513 = float(uint(1));
                    float _516 = sqrt(_513 + 0.5) * 0.5;
                    float _518 = (_513 * 2.400000095367431640625) + _472;
                    vec4 _529 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_490], g_xBilinearClamp), _336.xy + (vec2(_516 * cos(_518), _516 * sin(_518)) * _476), FrameConstants.g_fGlobalSamplerBias);
                    float _539 = clamp(((_389 / (_390 - ((((1.0 - _529.x) * 2.0) - 1.0) * _391))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _391, 0.0, 1.0);
                    vec2 _549;
                    if (_539 < _397)
                    {
                        vec2 _545 = _202;
                        _545.x = _511.x + _539;
                        vec2 _548 = _545;
                        _548.y = _511.y + 1.0;
                        _549 = _548;
                    }
                    else
                    {
                        _549 = _511;
                    }
                    float _551 = float(uint(2));
                    float _554 = sqrt(_551 + 0.5) * 0.5;
                    float _556 = (_551 * 2.400000095367431640625) + _472;
                    vec4 _565 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_490], g_xBilinearClamp), _336.xy + (vec2(_554 * cos(_556), _554 * sin(_556)) * _476), FrameConstants.g_fGlobalSamplerBias);
                    float _575 = clamp(((_389 / (_390 - ((((1.0 - _565.x) * 2.0) - 1.0) * _391))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _391, 0.0, 1.0);
                    vec2 _585;
                    if (_575 < _397)
                    {
                        vec2 _581 = _202;
                        _581.x = _549.x + _575;
                        vec2 _584 = _581;
                        _584.y = _549.y + 1.0;
                        _585 = _584;
                    }
                    else
                    {
                        _585 = _549;
                    }
                    float _587 = float(uint(3));
                    float _590 = sqrt(_587 + 0.5) * 0.5;
                    float _592 = (_587 * 2.400000095367431640625) + _472;
                    vec4 _601 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_490], g_xBilinearClamp), _336.xy + (vec2(_590 * cos(_592), _590 * sin(_592)) * _476), FrameConstants.g_fGlobalSamplerBias);
                    float _611 = clamp(((_389 / (_390 - ((((1.0 - _601.x) * 2.0) - 1.0) * _391))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _391, 0.0, 1.0);
                    vec2 _621;
                    if (_611 < _397)
                    {
                        vec2 _617 = _202;
                        _617.x = _585.x + _611;
                        vec2 _620 = _617;
                        _620.y = _585.y + 1.0;
                        _621 = _620;
                    }
                    else
                    {
                        _621 = _585;
                    }
                    float _733;
                    if (_621.y > 0.0)
                    {
                        float _627 = _621.x / _621.y;
                        float _629 = (_397 - _627) / _627;
                        float _636 = (((clamp(_629 * _629, 0.0, 1.0) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness) * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov) + (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize);
                        _733 = (((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_490)], g_xShadowCmpSampler)), vec3(_336.xy + (vec2(((-0.326211988925933837890625) * _473) - ((-0.40580499172210693359375) * _474), ((-0.40580499172210693359375) * _473) + ((-0.326211988925933837890625) * _474)) * _636), _383)) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2(((-0.840143978595733642578125) * _473) - ((-0.0735799968242645263671875) * _474), ((-0.0735799968242645263671875) * _473) + ((-0.840143978595733642578125) * _474)) * _636), _383))) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2(((-0.695913970470428466796875) * _473) - (0.45713698863983154296875 * _474), (0.45713698863983154296875 * _473) + ((-0.695913970470428466796875) * _474)) * _636), _383))) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2(((-0.203345000743865966796875) * _473) - (0.62071597576141357421875 * _474), (0.62071597576141357421875 * _473) + ((-0.203345000743865966796875) * _474)) * _636), _383))) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2((0.96233999729156494140625 * _473) - ((-0.194983005523681640625) * _474), ((-0.194983005523681640625) * _473) + (0.96233999729156494140625 * _474)) * _636), _383))) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2((0.4734340012073516845703125 * _473) - ((-0.4800260066986083984375) * _474), ((-0.4800260066986083984375) * _473) + (0.4734340012073516845703125 * _474)) * _636), _383))) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2((0.51945602893829345703125 * _473) - (0.76702201366424560546875 * _474), (0.76702201366424560546875 * _473) + (0.51945602893829345703125 * _474)) * _636), _383))) + texture(sampler2DShadow(g_xTextureArray[_490], g_xShadowCmpSampler), vec3(_336.xy + (vec2((0.18546099960803985595703125 * _473) - ((-0.89312398433685302734375) * _474), ((-0.89312398433685302734375) * _473) + (0.18546099960803985595703125 * _474)) * _636), _383))) * 0.125;
                    }
                    else
                    {
                        _733 = 0.0;
                    }
                    _734 = _733;
                }
                else
                {
                    uint _406 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    _734 = (((((((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_406)], g_xShadowCmpSampler)), vec3(_336.xy + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383)) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) + texture(sampler2DShadow(g_xTextureArray[_406], g_xShadowCmpSampler), vec3(_336.xy + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _383))) * 0.083333335816860198974609375;
                }
                vec4 _737 = vec4(0.0, 0.0, 0.0, 1.0);
                _737.w = 1.0 - (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength * _734);
                _738 = _737;
            }
            else
            {
                _738 = vec4(0.0, 0.0, 0.0, 1.0);
            }
            _739 = _738;
        }
        else
        {
            _739 = vec4(0.0, 0.0, 0.0, 1.0);
        }
        vec4 _1006;
        if (_739.w > 0.0)
        {
            float _769;
            vec3 _770;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius > 0.0)
            {
                vec3 _756 = -reflect(_311, _292);
                vec3 _759 = (_756 * dot(_345, _756)) - _345;
                _769 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius * inversesqrt(dot(_345, _345));
                _770 = normalize(_345 + (_759 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius / length(_759), 0.0, 1.0)));
            }
            else
            {
                _769 = 0.0;
                _770 = _358;
            }
            float _772 = (_295 != 0.01960784383118152618408203125) ? _296 : 0.0;
            vec3 _774 = normalize(_311 + _770);
            float _776 = clamp(dot(_292, _358), 0.0, 1.0);
            float _778 = clamp(dot(_292, _311), 0.0, 1.0);
            float _780 = clamp(dot(_292, _774), 0.0, 1.0);
            float _782 = clamp(dot(_311, _774), 0.0, 1.0);
            bool _783 = _295 == 0.01568627543747425079345703125;
            vec3 _820;
            vec3 _821;
            vec3 _822;
            if (_783)
            {
                _820 = vec3(0.046999998390674591064453125);
                _821 = vec3(_776);
                _822 = _293;
            }
            else
            {
                vec3 _816;
                vec3 _817;
                vec3 _818;
                if (_295 == 0.007843137718737125396728515625)
                {
                    _816 = vec3(0.0280000008642673492431640625);
                    _817 = vec3(_776);
                    _818 = _293;
                }
                else
                {
                    vec3 _812;
                    vec3 _813;
                    vec3 _814;
                    if (_295 == 0.0039215688593685626983642578125)
                    {
                        _812 = vec3(0.0280000008642673492431640625);
                        _813 = vec3((clamp((clamp(dot(_358, -_292), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_358, -_311), 0.0, 1.0), 7.0), 0.0, 1.0) * _772) + _776);
                        _814 = _293;
                    }
                    else
                    {
                        vec3 _795 = vec3(_772);
                        _812 = mix(vec3(0.039999999105930328369140625), _293, _795);
                        _813 = vec3(_776);
                        _814 = mix(_293, vec3(0.0), _795);
                    }
                    _816 = _812;
                    _817 = _813;
                    _818 = _814;
                }
                _820 = _816;
                _821 = _817;
                _822 = _818;
            }
            vec3 _823 = _821 * _822;
            float _824 = max(_294, 0.0500000007450580596923828125);
            float _825 = _824 * _824;
            vec3 _938;
            if (_783)
            {
                float _876 = clamp(1.0 - _294, 0.0, 1.0);
                float _878 = ((_296 * 2.0) - 1.0) * 0.300000011920928955078125;
                float _884 = pow(_876, 8.0) * 500.0;
                float _887 = pow(_876, 6.0) * 2000.0;
                float _901 = dot(normalize(_310 + (_292 * ((-0.0500000007450580596923828125) + _878))), _774);
                float _921 = dot(normalize(_310 + (_292 * ((0.0500000007450580596923828125 + _878) - (_876 * 0.0500000007450580596923828125)))), _774);
                _938 = _823 + ((((vec3(((smoothstep(-1.0, 0.0, _901) * pow(sqrt(clamp(1.0 - (_901 * _901), 0.0, 1.0)), 20.0 + _884)) * ((((_296 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _876))) * ((_884 + 28.0) * 0.03978873789310455322265625)) * (_293 / vec3(max(dot(_293, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _921) * pow(sqrt(clamp(1.0 - (_921 * _921), 0.0, 1.0)), 50.0 + _887)) * ((((_296 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _876)) * ((_887 + 58.0) * 0.03978873789310455322265625))) * _776) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
            }
            else
            {
                float _831 = clamp(_769 * (1.0 - _825), 0.0, 1.0);
                float _851;
                if (_831 > 0.0)
                {
                    float _842 = _825 * _825;
                    _851 = _842 / (_842 + (((_831 * 0.25) * ((3.0 * _825) + _831)) / (_782 + 0.001000000047497451305389404296875)));
                }
                else
                {
                    _851 = 1.0;
                }
                float _852 = 2.0 - _825;
                float _859 = _825 * _825;
                float _863 = (((_780 * _859) - _780) * _780) + 1.0;
                _938 = _823 + (((((vec3(1.0) - _820) * exp2((((-5.554729938507080078125) * _782) - 6.9831600189208984375) * _782)) + _820) * ((((1.0 / (((_778 * _852) + _825) * ((_776 * _852) + _825))) * (_859 * (1.0 / (_863 * _863)))) * _776) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _851);
            }
            vec3 _988;
            if (_295 == 0.01960784383118152618408203125)
            {
                bool _942 = _296 > 0.4980392158031463623046875;
                vec3 _960 = sqrt(abs(_822));
                float _974 = 0.5 / _825;
                _988 = mix(_938, (_823 + (((((vec3(1.0) - _960) * exp2((((-5.554729938507080078125) * _782) - 6.9831600189208984375) * _782)) + _960) * ((1.0 / ((4.0 * ((_776 + _778) - (_776 * _778))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _974) * pow(1.0 - (_780 * _780), _974)))) * (_776 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_822 * (clamp((clamp(dot(_358, -_292), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_358, -_311), 0.0, 1.0), 7.0), 0.0, 1.0) * (_942 ? 0.5 : 0.0))), vec3((_942 ? (_296 - 0.501960813999176025390625) : _296) * 2.007874011993408203125));
            }
            else
            {
                _988 = _938;
            }
            vec3 _991 = min(_988 * (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * (_367 * clamp(((-dot(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz, _358)) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffOfs) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffScale, 0.0, 1.0))), vec3(1000.0)).xyz;
            vec3 _1004;
            if ((_295 == 0.007843137718737125396728515625) && (_220 < 3.0))
            {
                _1004 = _991 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_220 - 2.0, 0.0)));
            }
            else
            {
                _1004 = _991;
            }
            _1006 = vec4(_1004.x, _1004.y, _1004.z, _739.w);
        }
        else
        {
            _1006 = _739;
        }
        _1007 = _1006;
        _1008 = true;
        break;
    } while(false);
    if (false == _1008)
    {
        discard;
    }
    out_var_SV_Target = _1007;
}


