#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct PhotonFace
{
    uint uLevel;
    float fSize;
    float fInvSize;
    int iShadowIndex;
};

struct PhotonParams
{
    mat4 m_xLightOrientation;
    mat4 m_xCustomVolumeMatrix;
    vec3 m_xLightPos;
    float m_fLightRadius;
    vec3 m_xLightDir;
    float m_fLightRange;
    vec3 m_xLightColour;
    uint m_bUseLimitAngles;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fLightAngleOuter;
    float m_fRcpLightAngleRange;
    float m_fArcCosLimitAngle;
    float m_fRcpProjCubeLerpDist;
    uint m_bUseProjCube;
    uint m_bUseDualProjCube;
    vec2 m_xDepthProjection;
    float m_fLightMeshRadius;
    float m_fShadowStrength;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fPadding;
    int m_iProjCubeTextureIndex;
    int m_iProjCubeBlurredTextureIndex;
    PhotonFace m_axFace[6];
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_PhotonIndex
{
    uint g_uPhotonIndex;
} PhotonIndex;

layout(set = 1, binding = 89, std430) readonly buffer type_StructuredBuffer_PhotonParams
{
    layout(row_major) PhotonParams _m0[];
} g_axPhotons;

layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;
layout(set = 1, binding = 56) uniform texture2D g_xShadowCubeMapPosX;
layout(set = 1, binding = 57) uniform texture2D g_xShadowCubeMapNegX;
layout(set = 1, binding = 59) uniform texture2D g_xShadowCubeMapNegY;
layout(set = 1, binding = 58) uniform texture2D g_xShadowCubeMapPosY;
layout(set = 1, binding = 60) uniform texture2D g_xShadowCubeMapPosZ;
layout(set = 1, binding = 61) uniform texture2D g_xShadowCubeMapNegZ;

layout(location = 0) out vec4 out_var_SV_Target;

vec2 _79;

void main()
{
    vec4 _349;
    do
    {
        ivec2 _110 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
        float _119 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _110, 0).x);
        vec3 _138 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightPos - (vec4((FrameConstants.g_xCameraTanHalfFOV * _119) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _119, 1.0) * FrameConstants.g_xInvView).xyz;
        float _139 = length(_138);
        vec3 _141 = _138 / vec3(_139);
        float _149 = pow(clamp((1.0 - clamp((_139 - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius) / (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius), 0.0, 1.0)) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffScale, 0.0, 1.0), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffPower);
        float _157;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseLimitAngles != 0u)
        {
            _157 = _149 * clamp((dot(_141, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightDir) + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightAngleOuter) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpLightAngleRange, 0.0, 1.0);
        }
        else
        {
            _157 = _149;
        }
        if ((_157 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            discard;
        }
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowStrength > 0.0)
        {
            vec3 _177 = -_138;
            vec3 _180 = _177;
            _180.y = -_177.y;
            vec3 _181 = abs(_180);
            float _182 = _181.x;
            float _183 = _181.y;
            float _184 = _181.z;
            float _186 = max(_182, max(_183, _184));
            vec3 _188 = _180 / vec3(_186);
            float _193 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xDepthProjection.x + (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xDepthProjection.y / (_186 - max(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowBias * (1.0 - clamp(dot(normalize((TEXEL_FETCH(g_xLighting_NormalBuffer, _110, 0).xyz * 2.0) - vec3(1.0)), _141), 0.0, 1.0)), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowBias)));
            uint _280;
            vec2 _281;
            if (_183 > _182)
            {
                uint _278;
                vec2 _279;
                if (_184 > _183)
                {
                    bool _261 = _188.z < 0.0;
                    vec2 _276;
                    if (_261)
                    {
                        vec2 _272 = _79;
                        _272.x = -_188.x;
                        vec2 _275 = _272;
                        _275.y = -_188.y;
                        _276 = _275;
                    }
                    else
                    {
                        vec2 _266 = _79;
                        _266.x = _188.x;
                        vec2 _269 = _266;
                        _269.y = -_188.y;
                        _276 = _269;
                    }
                    _278 = _261 ? 5u : 4u;
                    _279 = _276;
                }
                else
                {
                    bool _245 = _188.y < 0.0;
                    vec2 _258;
                    if (_245)
                    {
                        vec2 _254 = _79;
                        _254.x = _188.x;
                        vec2 _257 = _254;
                        _257.y = -_188.z;
                        _258 = _257;
                    }
                    else
                    {
                        vec2 _250 = _79;
                        _250.x = _188.x;
                        vec2 _252 = _250;
                        _252.y = _188.z;
                        _258 = _252;
                    }
                    _278 = _245 ? 2u : 3u;
                    _279 = _258;
                }
                _280 = _278;
                _281 = _279;
            }
            else
            {
                uint _238;
                vec2 _239;
                if (_184 > _182)
                {
                    bool _221 = _188.z < 0.0;
                    vec2 _236;
                    if (_221)
                    {
                        vec2 _232 = _79;
                        _232.x = -_188.x;
                        vec2 _235 = _232;
                        _235.y = -_188.y;
                        _236 = _235;
                    }
                    else
                    {
                        vec2 _226 = _79;
                        _226.x = _188.x;
                        vec2 _229 = _226;
                        _229.y = -_188.y;
                        _236 = _229;
                    }
                    _238 = _221 ? 5u : 4u;
                    _239 = _236;
                }
                else
                {
                    bool _203 = _188.x < 0.0;
                    vec2 _218;
                    if (_203)
                    {
                        vec2 _214 = _79;
                        _214.x = _188.z;
                        vec2 _217 = _214;
                        _217.y = -_188.y;
                        _218 = _217;
                    }
                    else
                    {
                        vec2 _209 = _79;
                        _209.x = -_188.z;
                        vec2 _212 = _209;
                        _212.y = -_188.y;
                        _218 = _212;
                    }
                    _238 = uint(_203);
                    _239 = _218;
                }
                _280 = _238;
                _281 = _239;
            }
            vec2 _283 = (_281 * 0.5) + vec2(0.5);
            float _329;
            do
            {
                vec2 _286 = clamp(_283, vec2(0.0), vec2(1.0));
                if (_280 == 0u)
                {
                    _329 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_286, _193));
                    break;
                }
                if (_280 == 1u)
                {
                    _329 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_286, _193));
                    break;
                }
                if (_280 == 2u)
                {
                    _329 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_286, _193));
                    break;
                }
                if (_280 == 3u)
                {
                    _329 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_286, _193));
                    break;
                }
                if (_280 == 4u)
                {
                    _329 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_286, _193));
                    break;
                }
                if (_280 == 5u)
                {
                    _329 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_286, _193));
                    break;
                }
                _329 = 0.0;
                break;
            } while(false);
            vec2 _338 = mix(vec2(0.0), vec2(1.0), greaterThan(fract((_283 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_280].fSize) * 0.5), vec2(0.5)));
            float _344 = 0.20000000298023223876953125 + (abs(_338.x - _338.y) * 0.20000000298023223876953125);
            _349 = mix(vec4(0.0, _344, 0.5, 1.0), vec4(1.0, _344, 0.0, 1.0), vec4(1.0 - (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowStrength * _329)));
            break;
        }
        else
        {
            _349 = vec4(0.0);
            break;
        }
    } while(false);
    out_var_SV_Target = _349;
}

