// LIGHTS 2
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRangeUnused;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 4) uniform sampler g_xTrilinearWrap;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 3, binding = 0) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xTextureArray[];
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec3 _207;
vec4 _208;
vec2 _209;

void main()
{
    ivec2 _219 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _227 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _219, 0).x);
    vec4 _242 = vec4((FrameConstants.g_xCameraTanHalfFOV * _227) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _227, 1.0);
    
   _242.x -= vk3d_params[VIEW].stereo.x * (_227 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0]; 
    
    vec4 _286 = TEXEL_FETCH(g_xLighting_NormalBuffer, _219, 0);
    vec4 _288 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _219, 0);
    vec4 _290 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _219, 0);
    vec3 _294 = normalize((_286.xyz * 2.0) - vec3(1.0));
    vec3 _295 = _288.xyz;
    float _296 = _290.y;
    float _297 = _288.w;
    float _298 = _290.x;
    vec2 _301 = (_290.zw * 2.0) - vec2(1.0);
    vec2 _303 = _301.xy;
    float _307 = sqrt(clamp(1.0 - dot(_303, _303), 0.0, 1.0));
    vec3 _312 = vec3(_301.x, _301.y, _207.z);
    _312.z = (_286.w != 0.0) ? _307 : (-_307);
    vec3 _313 = normalize(in_var_CamDir);
    vec4 _1020;
    bool _1021;
    do
    {
        vec4 _338;
        bool _339;
        do
        {
            vec4 _318 = _242 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            float _319 = _318.z;
            if (_319 < 0.0)
            {
                _338 = _208;
                _339 = false;
                break;
            }
            vec2 _326 = _318.xy / vec2(_318.w);
            vec4 _329 = vec4(_326, _319, _318.w);
            if (any(lessThan(vec2(0.5) - abs(_326.xy - vec2(0.5)), vec2(0.0))))
            {
                _338 = _329;
                _339 = false;
                break;
            }
            _338 = _329;
            _339 = true;
            break;
        } while(false);
        if (!_339)
        {
            _1020 = vec4(0.0, 0.0, 0.0, 1.0);
            _1021 = false;
            break;
        }
        vec3 _347 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_242 * FrameConstants.g_xInvView).xyz;
        float _348 = length(_347);
        vec3 _350 = _347 / vec3(_348);
        float _359 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_348 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_359 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _1020 = vec4(0.0, 0.0, 0.0, 1.0);
            _1021 = false;
            break;
        }
        vec4 _731;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex != (-1))
        {
            float _375 = (_338.z + max(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias * (1.0 - clamp(dot(_294, _350), 0.0, 1.0)), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowBias)) / _338.w;
            float _381 = (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane;
            float _382 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _383 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowFarPlane - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane;
            float _389 = clamp(((_381 / (_382 - ((((1.0 - _375) * 2.0) - 1.0) * _383))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _383, 0.0, 1.0);
            vec4 _730;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength > 0.0)
            {
                float _726;
                if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness > 0.0)
                {
                    float _464 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                    float _465 = sin(_464);
                    float _466 = cos(_464);
                    float _468 = (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov;
                    float _470 = float(uint(0));
                    float _473 = sqrt(_470 + 0.5) * 0.5;
                    float _475 = (_470 * 2.400000095367431640625) + _464;
                    uint _482 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    vec4 _487 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(_482)], g_xBilinearClamp)), _338.xy + (vec2(_473 * cos(_475), _473 * sin(_475)) * _468), FrameConstants.g_fGlobalSamplerBias);
                    float _497 = clamp(((_381 / (_382 - ((((1.0 - _487.x) * 2.0) - 1.0) * _383))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _383, 0.0, 1.0);
                    vec2 _503;
                    if (_497 < _389)
                    {
                        vec2 _501 = _209;
                        _501.x = _497;
                        vec2 _502 = _501;
                        _502.y = 1.0;
                        _503 = _502;
                    }
                    else
                    {
                        _503 = vec2(0.0);
                    }
                    float _505 = float(uint(1));
                    float _508 = sqrt(_505 + 0.5) * 0.5;
                    float _510 = (_505 * 2.400000095367431640625) + _464;
                    vec4 _521 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_482], g_xBilinearClamp), _338.xy + (vec2(_508 * cos(_510), _508 * sin(_510)) * _468), FrameConstants.g_fGlobalSamplerBias);
                    float _531 = clamp(((_381 / (_382 - ((((1.0 - _521.x) * 2.0) - 1.0) * _383))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _383, 0.0, 1.0);
                    vec2 _541;
                    if (_531 < _389)
                    {
                        vec2 _537 = _209;
                        _537.x = _503.x + _531;
                        vec2 _540 = _537;
                        _540.y = _503.y + 1.0;
                        _541 = _540;
                    }
                    else
                    {
                        _541 = _503;
                    }
                    float _543 = float(uint(2));
                    float _546 = sqrt(_543 + 0.5) * 0.5;
                    float _548 = (_543 * 2.400000095367431640625) + _464;
                    vec4 _557 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_482], g_xBilinearClamp), _338.xy + (vec2(_546 * cos(_548), _546 * sin(_548)) * _468), FrameConstants.g_fGlobalSamplerBias);
                    float _567 = clamp(((_381 / (_382 - ((((1.0 - _557.x) * 2.0) - 1.0) * _383))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _383, 0.0, 1.0);
                    vec2 _577;
                    if (_567 < _389)
                    {
                        vec2 _573 = _209;
                        _573.x = _541.x + _567;
                        vec2 _576 = _573;
                        _576.y = _541.y + 1.0;
                        _577 = _576;
                    }
                    else
                    {
                        _577 = _541;
                    }
                    float _579 = float(uint(3));
                    float _582 = sqrt(_579 + 0.5) * 0.5;
                    float _584 = (_579 * 2.400000095367431640625) + _464;
                    vec4 _593 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_482], g_xBilinearClamp), _338.xy + (vec2(_582 * cos(_584), _582 * sin(_584)) * _468), FrameConstants.g_fGlobalSamplerBias);
                    float _603 = clamp(((_381 / (_382 - ((((1.0 - _593.x) * 2.0) - 1.0) * _383))) - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowNearPlane) / _383, 0.0, 1.0);
                    vec2 _613;
                    if (_603 < _389)
                    {
                        vec2 _609 = _209;
                        _609.x = _577.x + _603;
                        vec2 _612 = _609;
                        _612.y = _577.y + 1.0;
                        _613 = _612;
                    }
                    else
                    {
                        _613 = _577;
                    }
                    float _725;
                    if (_613.y > 0.0)
                    {
                        float _619 = _613.x / _613.y;
                        float _621 = (_389 - _619) / _619;
                        float _628 = (((clamp(_621 * _621, 0.0, 1.0) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowSoftness) * 0.02500000037252902984619140625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fProjectorOneOverTanHalfFov) + (2.0 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize);
                        _725 = (((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_482)], g_xShadowCmpSampler)), vec3(_338.xy + (vec2(((-0.326211988925933837890625) * _465) - ((-0.40580499172210693359375) * _466), ((-0.40580499172210693359375) * _465) + ((-0.326211988925933837890625) * _466)) * _628), _375)) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2(((-0.840143978595733642578125) * _465) - ((-0.0735799968242645263671875) * _466), ((-0.0735799968242645263671875) * _465) + ((-0.840143978595733642578125) * _466)) * _628), _375))) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2(((-0.695913970470428466796875) * _465) - (0.45713698863983154296875 * _466), (0.45713698863983154296875 * _465) + ((-0.695913970470428466796875) * _466)) * _628), _375))) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2(((-0.203345000743865966796875) * _465) - (0.62071597576141357421875 * _466), (0.62071597576141357421875 * _465) + ((-0.203345000743865966796875) * _466)) * _628), _375))) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2((0.96233999729156494140625 * _465) - ((-0.194983005523681640625) * _466), ((-0.194983005523681640625) * _465) + (0.96233999729156494140625 * _466)) * _628), _375))) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2((0.4734340012073516845703125 * _465) - ((-0.4800260066986083984375) * _466), ((-0.4800260066986083984375) * _465) + (0.4734340012073516845703125 * _466)) * _628), _375))) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2((0.51945602893829345703125 * _465) - (0.76702201366424560546875 * _466), (0.76702201366424560546875 * _465) + (0.51945602893829345703125 * _466)) * _628), _375))) + texture(sampler2DShadow(g_xTextureArray[_482], g_xShadowCmpSampler), vec3(_338.xy + (vec2((0.18546099960803985595703125 * _465) - ((-0.89312398433685302734375) * _466), ((-0.89312398433685302734375) * _465) + (0.18546099960803985595703125 * _466)) * _628), _375))) * 0.125;
                    }
                    else
                    {
                        _725 = 0.0;
                    }
                    _726 = _725;
                }
                else
                {
                    uint _398 = uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iShadowTextureIndex);
                    _726 = (((((((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_398)], g_xShadowCmpSampler)), vec3(_338.xy + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375)) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) + texture(sampler2DShadow(g_xTextureArray[_398], g_xShadowCmpSampler), vec3(_338.xy + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_f2DFilterSize), _375))) * 0.083333335816860198974609375;
                }
                vec4 _729 = vec4(0.0, 0.0, 0.0, 1.0);
                _729.w = 1.0 - (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fShadowStrength * _726);
                _730 = _729;
            }
            else
            {
                _730 = vec4(0.0, 0.0, 0.0, 1.0);
            }
            _731 = _730;
        }
        else
        {
            _731 = vec4(0.0, 0.0, 0.0, 1.0);
        }
        vec4 _1019;
        if (_731.w > 0.0)
        {
            vec4 _746;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iTextureIndex > 0)
            {
                _746 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iTextureIndex))], g_xTrilinearClamp)), _338.xy, FrameConstants.g_fGlobalSamplerBias);
            }
            else
            {
                _746 = vec4(1.0);
            }
            vec4 _761;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iDetailTextureIndex > 0)
            {
                _761 = _746 * 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_iDetailTextureIndex))], g_xTrilinearWrap)), (_338.xy * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xDetailOffsetScale.zw) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xDetailOffsetScale.xy, FrameConstants.g_fGlobalSamplerBias);
            }
            else
            {
                _761 = _746;
            }
            float _782;
            vec3 _783;
            if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius > 0.0)
            {
                vec3 _769 = -reflect(_313, _294);
                vec3 _772 = (_769 * dot(_347, _769)) - _347;
                _782 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius * inversesqrt(dot(_347, _347));
                _783 = normalize(_347 + (_772 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRadius / length(_772), 0.0, 1.0)));
            }
            else
            {
                _782 = 0.0;
                _783 = _350;
            }
            float _785 = (_297 != 0.01960784383118152618408203125) ? _298 : 0.0;
            vec3 _787 = normalize(_313 + _783);
            float _789 = clamp(dot(_294, _350), 0.0, 1.0);
            float _791 = clamp(dot(_294, _313), 0.0, 1.0);
            float _793 = clamp(dot(_294, _787), 0.0, 1.0);
            float _795 = clamp(dot(_313, _787), 0.0, 1.0);
            bool _796 = _297 == 0.01568627543747425079345703125;
            vec3 _833;
            vec3 _834;
            vec3 _835;
            if (_796)
            {
                _833 = vec3(0.046999998390674591064453125);
                _834 = vec3(_789);
                _835 = _295;
            }
            else
            {
                vec3 _829;
                vec3 _830;
                vec3 _831;
                if (_297 == 0.007843137718737125396728515625)
                {
                    _829 = vec3(0.0280000008642673492431640625);
                    _830 = vec3(_789);
                    _831 = _295;
                }
                else
                {
                    vec3 _825;
                    vec3 _826;
                    vec3 _827;
                    if (_297 == 0.0039215688593685626983642578125)
                    {
                        _825 = vec3(0.0280000008642673492431640625);
                        _826 = vec3((clamp((clamp(dot(_350, -_294), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_350, -_313), 0.0, 1.0), 7.0), 0.0, 1.0) * _785) + _789);
                        _827 = _295;
                    }
                    else
                    {
                        vec3 _808 = vec3(_785);
                        _825 = mix(vec3(0.039999999105930328369140625), _295, _808);
                        _826 = vec3(_789);
                        _827 = mix(_295, vec3(0.0), _808);
                    }
                    _829 = _825;
                    _830 = _826;
                    _831 = _827;
                }
                _833 = _829;
                _834 = _830;
                _835 = _831;
            }
            vec3 _836 = _834 * _835;
            float _837 = max(_296, 0.0500000007450580596923828125);
            float _838 = _837 * _837;
            vec3 _951;
            if (_796)
            {
                float _889 = clamp(1.0 - _296, 0.0, 1.0);
                float _891 = ((_298 * 2.0) - 1.0) * 0.300000011920928955078125;
                float _897 = pow(_889, 8.0) * 500.0;
                float _900 = pow(_889, 6.0) * 2000.0;
                float _914 = dot(normalize(_312 + (_294 * ((-0.0500000007450580596923828125) + _891))), _787);
                float _934 = dot(normalize(_312 + (_294 * ((0.0500000007450580596923828125 + _891) - (_889 * 0.0500000007450580596923828125)))), _787);
                _951 = _836 + ((((vec3(((smoothstep(-1.0, 0.0, _914) * pow(sqrt(clamp(1.0 - (_914 * _914), 0.0, 1.0)), 20.0 + _897)) * ((((_298 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _889))) * ((_897 + 28.0) * 0.03978873789310455322265625)) * (_295 / vec3(max(dot(_295, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _934) * pow(sqrt(clamp(1.0 - (_934 * _934), 0.0, 1.0)), 50.0 + _900)) * ((((_298 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _889)) * ((_900 + 58.0) * 0.03978873789310455322265625))) * _789) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
            }
            else
            {
                float _844 = clamp(_782 * (1.0 - _838), 0.0, 1.0);
                float _864;
                if (_844 > 0.0)
                {
                    float _855 = _838 * _838;
                    _864 = _855 / (_855 + (((_844 * 0.25) * ((3.0 * _838) + _844)) / (_795 + 0.001000000047497451305389404296875)));
                }
                else
                {
                    _864 = 1.0;
                }
                float _865 = 2.0 - _838;
                float _872 = _838 * _838;
                float _876 = (((_793 * _872) - _793) * _793) + 1.0;
                _951 = _836 + (((((vec3(1.0) - _833) * exp2((((-5.554729938507080078125) * _795) - 6.9831600189208984375) * _795)) + _833) * ((((1.0 / (((_791 * _865) + _838) * ((_789 * _865) + _838))) * (_872 * (1.0 / (_876 * _876)))) * _789) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _864);
            }
            vec3 _1001;
            if (_297 == 0.01960784383118152618408203125)
            {
                bool _955 = _298 > 0.4980392158031463623046875;
                vec3 _973 = sqrt(abs(_835));
                float _987 = 0.5 / _838;
                _1001 = mix(_951, (_836 + (((((vec3(1.0) - _973) * exp2((((-5.554729938507080078125) * _795) - 6.9831600189208984375) * _795)) + _973) * ((1.0 / ((4.0 * ((_789 + _791) - (_789 * _791))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _987) * pow(1.0 - (_793 * _793), _987)))) * (_789 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_835 * (clamp((clamp(dot(_350, -_294), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_350, -_313), 0.0, 1.0), 7.0), 0.0, 1.0) * (_955 ? 0.5 : 0.0))), vec3((_955 ? (_298 - 0.501960813999176025390625) : _298) * 2.007874011993408203125));
            }
            else
            {
                _1001 = _951;
            }
            vec3 _1004 = min(_1001 * ((g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * _359) * _761.xyz), vec3(1000.0)).xyz;
            vec3 _1017;
            if ((_297 == 0.007843137718737125396728515625) && (_227 < 3.0))
            {
                _1017 = _1004 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_227 - 2.0, 0.0)));
            }
            else
            {
                _1017 = _1004;
            }
            _1019 = vec4(_1017.x, _1017.y, _1017.z, _731.w);
        }
        else
        {
            _1019 = _731;
        }
        _1020 = _1019;
        _1021 = true;
        break;
    } while(false);
    if (false == _1021)
    {
        discard;
    }
    out_var_SV_Target = _1020;
}


