// Self Shadows
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require

struct StaticMaterial
{
    float m_fMaterialIndex;
    vec3 m_xAlbedoTint;
    vec4 m_xParallaxUVScaleAndSpeed;
    float m_fRefractivity;
    float m_fParallaxScale;
    float m_fParallaxBias;
    float m_fAlbedoDetailUVScale;
    float m_fAlbedoDetailValueScale;
    float m_fNormalDetailUVScale;
    float m_fNormalDetailValueScale;
    float m_fMaterialModel;
    float m_fSlopeShaderUVScale;
    float m_fSlopeShaderStrength;
    float m_fSlopeShaderThreshold;
    float m_fSlopeShaderPower;
    float m_fSlopeShaderThresFactor;
    vec3 m_xSlopeShaderNormal;
    vec3 m_xSlopeShaderForward;
    float m_fPad;
    float m_fInvHeightmapBlendSoftness;
    float m_fDecalNormalThresholdA;
    float m_fDecalNormalThresholdB;
    float m_fFoliageCosClippingAngle;
    float m_fWindResponsiveness;
    uint m_bRefractionBlendBackground;
    uint m_bIsDecal;
    uint m_bIsFoliage;
    uint m_bReflectBackfaceNormalsWithView;
    uint m_bUseBinarySearchForParallax;
    uint m_bUseAlbedo;
    uint m_bUseNormalMap;
    uint m_bUseMetalMap;
    uint m_bUseTransparencyMap;
    uint m_bUseSelfIlluminationMap;
    uint m_bUseParallaxMap;
    uint m_bUseEnvironmentMap;
    uint m_bUseAlbedoDetailMap;
    uint m_bUseNormalDetailMap;
    uint m_bUseSelfIlluminationGraph;
    uint m_bUseRefraction;
    uint m_bUseTransparencyGraph;
    uint m_bUseSlopeShader;
    uint m_bUseHairFlowMap;
    uint m_bUseHairNoiseMap;
    uint m_bHasParallaxMap;
    uint m_bUseSlopeShaderAlbedoMap1;
    uint m_bUseSlopeShaderNormalMap1;
    uint m_bUseSlopeShaderTransparencyMap1;
    uint m_bUseDamageThresholdMap;
    uint m_bUseFaceWrinkles;
    uint m_bUseRippleMap;
    uint m_uAlbedoMap;
    uint m_uAlbedoDetailMap;
    uint m_uNormalMap;
    uint m_uNormalDetailMap;
    uint m_uMetalMap;
    uint m_uTransparencyMap;
    uint m_uSelfIlluminationMap;
    uint m_uParallaxMap;
    uint m_uHairFlowMap;
    uint m_uHairNoiseMap;
    uint m_uSlopeShaderAlbedoMap;
    uint m_uSlopeShaderNormalMap;
    uint m_uSlopeShaderTransparencyMap;
    uint m_uDamageThresholdMap;
    uint m_uWrinkleMask_Compressed;
    uint m_uWrinkleMask_Stretched;
    uint m_uWrinkleMap_Compressed;
    uint m_uWrinkleMap_Stretched;
    uint m_uRippleMap;
    uint m_bIsTransmissive;
    uint m_bIsDeformable;
    float m_fDeformableHeight;
    uint m_uDeformableDetailAlbedoMap;
    uint m_uDeformableDetailNormalMap;
    float m_fDeformableDetailMapUVScale;
    float m_fDeformableDetailBlendPower;
    float m_fPorosity;
    uint m_uStructuredBufferIndex;
};

struct CubemapData
{
    mat4 m_xObbSpace;
    mat4 m_xInvObbSpace;
    vec3 m_xCubemapPosition;
    float m_fPad;
};

struct CSMConstantsStruct
{
    mat4 g_axShadowViewProj[5];
    mat4 g_axInvShadowByViewProj[5];
    mat4 g_axScaledInvShadowByViewProj[5];
    vec4 g_axSize[5];
    vec4 g_axAreaScale[5];
    float g_fBlendArea;
    float g_fStrength;
    uint g_uNumMaps;
    uint g_uPad;
};

struct PhotonFace
{
    uint uLevel;
    float fSize;
    float fInvSize;
    int iShadowIndex;
};

struct PhotonParams
{
    mat4 m_xLightOrientation;
    mat4 m_xCustomVolumeMatrix;
    vec3 m_xLightPos;
    float m_fAreaLightRadius;
    vec3 m_xLightDir;
    float m_fLightRange;
    vec3 m_xLightColour;
    uint m_bUseLimitAngles;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fLightAngleOuter;
    float m_fRcpLightAngleRange;
    float m_fArcCosLimitAngle;
    float m_fRcpProjCubeLerpDist;
    uint m_bUseProjCube;
    uint m_bUseDualProjCube;
    vec2 m_xDepthProjection;
    float m_fLightMeshRadius;
    float m_fShadowStrength;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fPadding;
    int m_iProjCubeTextureIndex;
    int m_iProjCubeBlurredTextureIndex;
    PhotonFace m_axFace[6];
};

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRange;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fAreaLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};

const vec2 _440[8] = vec2[](vec2(-0.695039272308349609375, 0.16170980036258697509765625), vec2(-0.24559019505977630615234375, -0.6526126861572265625), vec2(0.24424530565738677978515625, 0.2412090003490447998046875), vec2(-0.84800231456756591796875, -0.3871740996837615966796875), vec2(-0.3136856853961944580078125, 0.60699260234832763671875), vec2(0.4607562124729156494140625, 0.87447822093963623046875), vec2(0.588717520236968994140625, -0.616069495677947998046875), vec2(0.82326567173004150390625, 0.123084597289562225341796875));


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 7, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 5, binding = 10, std140) uniform type_InstancedConstants
{
    uint g_uInstancedAnimID;
} InstancedConstants;

layout(set = 7, binding = 48, scalar) readonly buffer type_StructuredBuffer_StaticMaterial
{
    StaticMaterial _m0[];
} g_axMaterials;

layout(set = 1, binding = 1, scalar) uniform type_Material1Constants
{
    StaticMaterial g_xStaticMaterial1;
} Material1Constants;

layout(set = 1, binding = 2, scalar) uniform type_Material2Constants
{
    StaticMaterial g_xStaticMaterial2;
} Material2Constants;

layout(set = 7, binding = 6, scalar) uniform type_FogParams
{
    vec3 g_xFogColour;
    float g_fFogNearPlane;
    float g_fRcpFogVolumeDepth;
    float g_fMaxFogDensity;
    uint g_bFogEnabled;
    uint g_bTexturedFogEnabled;
    float g_fHorizonOffset;
    float g_fHorizonScale;
    float g_fTextureHDRScale;
    float g_fTextureUOffset;
    float g_fDensityPower;
    float g_fVerticalTextureScale;
    float g_fHeightFogStrength;
    float g_fHeightFogFalloff;
    float g_fHeightFogHeight;
    vec3 g_xHeightFogColour;
    uint g_bHeightFogEnabled;
    float g_fHeightFogMaxOpacity;
    float g_fHeightFogStartDistance;
    float g_fCosScatteringAngle;
    vec3 g_xSunDirection;
    float g_fDirectScatterStartDistance;
    float g_fDirectScatterFadeDistance;
    vec3 g_xDirectColour;
} FogParams;

layout(set = 2, binding = 3, std140) uniform type_DynamicMaterialConstants
{
    vec3 g_xSelfIlluminationParameters;
    float g_fFogScale;
    vec4 g_xUVScaleOffset;
} DynamicMaterialConstants;

layout(set = 5, binding = 4, std140) uniform type_SHConstants
{
    vec4 g_xRedStaticSH;
    vec4 g_xGreenStaticSH;
    vec4 g_xBlueStaticSH;
    vec3 g_xStaticSpecularDirection;
    int g_iCubeMapIndex;
    vec4 g_xRedDynamicSH;
    vec4 g_xGreenDynamicSH;
    vec4 g_xBlueDynamicSH;
    vec3 g_xDynamicSpecularDirection;
    float g_fSkinAlpha;
} SHConstants;

layout(set = 7, binding = 79, std430) readonly buffer type_StructuredBuffer_CubemapData
{
    layout(row_major) CubemapData _m0[];
} g_axPBRCubemapDatas;

layout(set = 6, binding = 7, std140) uniform type_ProjectedCloudParams
{
    uint g_bSunlightCloudsEnabled;
    float g_fSunlightCloudsVelocityX;
    float g_fSunlightCloudsVelocityY;
    float g_fSunlightCloudsScale;
    vec4 g_xSunlightCloudsProjectionDir;
} ProjectedCloudParams;

layout(set = 7, binding = 11, std140) uniform type_HairDebugParams
{
    float g_fHybridHairAlphaCutOff;
    uint g_bClipOpaquePixelsFromAlphaPass;
} HairDebugParams;

layout(set = 6, binding = 13, std140) uniform type_CSMParams
{
    layout(row_major) CSMConstantsStruct g_xCSMData;
} CSMParams;

layout(set = 6, binding = 12, scalar) uniform type_DynamicLightsParams
{
    vec3 g_xSunDir;
    float g_fPad1;
    vec3 g_xSunColour;
    uint g_uNumPhotons;
    uint g_uNumProjectors;
    uvec3 g_xPad2;
    float g_fPhotonSpatialHashScale;
    float g_fProjectorSpatialHashScale;
} DynamicLightsParams;

layout(set = 6, binding = 89, std430) readonly buffer type_StructuredBuffer_PhotonParams
{
    layout(row_major) PhotonParams _m0[];
} g_axPhotons;

layout(set = 6, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 6, binding = 91, std430) readonly buffer type_StructuredBuffer_uint
{
    uint _m0[];
} g_axPhotonSpatialHash;

layout(set = 6, binding = 92, std430) readonly buffer g_axProjectorSpatialHash
{
    uint _m0[];
} g_axProjectorSpatialHash_1;

layout(set = 7, binding = 80, std430) readonly buffer type_StructuredBuffer_v4float
{
    vec4 _m0[];
} g_xInstanceTexture;

layout(set = 0, binding = 1) uniform sampler g_xPointClamp;
layout(set = 0, binding = 2) uniform sampler g_xBilinearWrap;
layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 4) uniform sampler g_xTrilinearWrap;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 0, binding = 6) uniform sampler g_xFilteredWrap;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 8, binding = 0) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xTextureArray[];
layout(set = 7, binding = 49) uniform texture1D g_xFogTexture;
layout(set = 7, binding = 81) uniform TEXTURE_2D g_xFogColourTexture;
layout(set = 7, binding = 88) uniform TEXTURE_2D g_xMaterialsTexture;
layout(set = 7, binding = 50) uniform TEXTURE_2D g_xSceneTexture;
layout(set = 3, binding = 51) uniform TEXTURE_2D g_xDepthTexture;
layout(set = 7, binding = 77) uniform textureCubeArray g_axPBRCubeMaps;
layout(set = 6, binding = 100) uniform TEXTURE_2D g_xLighting_Clouds;
layout(set = 6, binding = 93) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_axCSMMap0;
layout(set = 6, binding = 94) uniform texture2D g_axCSMMap1;
layout(set = 6, binding = 95) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_axCSMMap2;
layout(set = 6, binding = 96) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_axCSMMap3;
layout(set = 6, binding = 97) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_axCSMMap4;

layout(location = 0) in vec4 in_var_NORMAL0;
layout(location = 1) in vec3 in_var_TANGENT0;
layout(location = 2) in vec3 in_var_BINORMAL0;
layout(location = 3) in vec4 in_var_TEXCOORD0;
layout(location = 4) in vec4 in_var_TEXCOORD1;
layout(location = 0) out vec4 out_var_SV_TARGET;

vec3 _500;
vec4 _501;
vec2 _502;
vec2 _503;
vec3 _504;

void main()
{
    vec3 _517 = normalize(in_var_NORMAL0.xyz);
    vec3 _519 = normalize(in_var_TEXCOORD1.xyz);
    vec3 _521 = normalize(in_var_BINORMAL0);
    mat3 _522 = mat3(normalize(in_var_TANGENT0), _521, _517);
    vec4 _777;
    if ((g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u) || (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u))
    {
        vec3 _535 = _519 * _522;
        vec4 _776;
        if ((g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u) || (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u))
        {
            vec4 _775;
            if (((g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u) && (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseBinarySearchForParallax != 0u)) || ((g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u) && (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseBinarySearchForParallax != 0u)))
            {
                vec3 _690 = normalize((-_535) / vec3(_535.z));
                _690.z = 1.0;
                vec3 _692;
                vec4 _697;
                _692 = _690;
                _697 = in_var_TEXCOORD0.xyxy;
                vec3 _693;
                float _696;
                vec4 _698;
                float _695 = 0.0;
                int _699 = 0;
                for (; _699 < 12; _692 = _693, _695 = _696, _697 = _698, _699++)
                {
                    float _732;
                    if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u)
                    {
                        _732 = ((1.0 - 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uParallaxMap)], g_xFilteredWrap)), (_697.xy * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.xy) + (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.zw * FrameConstants.g_fScaledTime), FrameConstants.g_fGlobalSamplerBias).x) * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fParallaxScale) + g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fParallaxBias;
                    }
                    else
                    {
                        _732 = 0.0;
                    }
                    float _761;
                    if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u)
                    {
                        _761 = ((1.0 - 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uParallaxMap)], g_xFilteredWrap)), (_697.zw * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.xy) + (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.zw * FrameConstants.g_fScaledTime), FrameConstants.g_fGlobalSamplerBias).x) * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fParallaxScale) + g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fParallaxBias;
                    }
                    else
                    {
                        _761 = 0.0;
                    }
                    _693 = _692 * 0.5;
                    if (_695 < mix(_761, _732, in_var_NORMAL0.w))
                    {
                        _696 = _695 + _693.z;
                        _698 = _697 + _693.xyxy;
                    }
                    else
                    {
                        _696 = _695 - _693.z;
                        _698 = _697 - _693.xyxy;
                    }
                }
                _775 = _697;
            }
            else
            {
                float _586;
                if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u)
                {
                    _586 = (
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uParallaxMap)], g_xFilteredWrap)), (in_var_TEXCOORD0.xy * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.xy) + (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.zw * FrameConstants.g_fScaledTime), FrameConstants.g_fGlobalSamplerBias).x * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fParallaxScale) + g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fParallaxBias;
                }
                else
                {
                    _586 = 0.0;
                }
                float _614;
                if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u)
                {
                    _614 = (
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uParallaxMap)], g_xFilteredWrap)), (in_var_TEXCOORD0.xy * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.xy) + (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.zw * FrameConstants.g_fScaledTime), FrameConstants.g_fGlobalSamplerBias).x * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fParallaxScale) + g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fParallaxBias;
                }
                else
                {
                    _614 = 0.0;
                }
                float _615 = mix(_614, _586, in_var_NORMAL0.w);
                vec2 _617 = _535.xy * _615;
                vec4 _619 = in_var_TEXCOORD0.xyxy + _617.xyxy;
                float _647;
                if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u)
                {
                    _647 = (
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uParallaxMap)], g_xFilteredWrap)), (_619.xy * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.xy) + (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.zw * FrameConstants.g_fScaledTime), FrameConstants.g_fGlobalSamplerBias).x * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fParallaxScale) + g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fParallaxBias;
                }
                else
                {
                    _647 = 0.0;
                }
                float _675;
                if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseParallaxMap != 0u)
                {
                    _675 = (
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uParallaxMap)], g_xFilteredWrap)), (_619.zw * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.xy) + (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xParallaxUVScaleAndSpeed.zw * FrameConstants.g_fScaledTime), FrameConstants.g_fGlobalSamplerBias).x * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fParallaxScale) + g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fParallaxBias;
                }
                else
                {
                    _675 = 0.0;
                }
                _775 = in_var_TEXCOORD0.xyxy + (_617 * clamp((-_615) / (mix(_675, _647, in_var_NORMAL0.w) - (2.0 * _615)), 0.0, 1.0)).xyxy;
            }
            _776 = _775;
        }
        else
        {
            _776 = in_var_TEXCOORD0.xyxy;
        }
        _777 = _776;
    }
    else
    {
        _777 = in_var_TEXCOORD0.xyxy;
    }
    float _808;
    do
    {
        if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseTransparencyMap != 0u)
        {
            vec4 _794 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uTransparencyMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias);
            float _795 = _794.w;
            float _807;
            if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseTransparencyMap != 0u)
            {
                _807 = mix(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uTransparencyMap)], g_xFilteredWrap)), _777.zw, FrameConstants.g_fGlobalSamplerBias).w, _795, in_var_NORMAL0.w);
            }
            else
            {
                _807 = _795;
            }
            _808 = _807;
            break;
        }
        else
        {
            _808 = 1.0;
            break;
        }
    } while(false);
    float _820;
    if (HairDebugParams.g_bClipOpaquePixelsFromAlphaPass != 0u)
    {
        if ((HairDebugParams.g_fHybridHairAlphaCutOff - _808) < 0.0)
        {
            discard;
        }
        _820 = clamp(_808 / HairDebugParams.g_fHybridHairAlphaCutOff, 0.0, 1.0);
    }
    else
    {
        _820 = _808;
    }
    vec3 _919;
    do
    {
        if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseNormalMap != 0u)
        {
            vec4 _835 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uNormalMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias);
            vec2 _838 = (_835.xy * 2.0) - vec2(1.0);
            vec3 _860;
            if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseNormalDetailMap != 0u)
            {
                vec2 _858 = _838.xy + (((
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uNormalDetailMap)], g_xFilteredWrap)), _777.xy * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fNormalDetailUVScale, FrameConstants.g_fGlobalSamplerBias).xy * 2.0) - vec2(1.0)) * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fNormalDetailValueScale);
                _860 = vec3(_858.x, _858.y, _500.z);
            }
            else
            {
                _860 = vec3(_838.x, _838.y, _500.z);
            }
            vec3 _863 = _860;
            _863.y = -_860.y;
            vec3 _869 = _863;
            _869.z = sqrt(clamp(1.0 - dot(_863.xy, _863.xy), 0.0, 1.0));
            vec3 _870 = _522 * _869;
            vec3 _917;
            if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseNormalMap != 0u)
            {
                vec4 _879 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uNormalMap)], g_xFilteredWrap)), _777.zw, FrameConstants.g_fGlobalSamplerBias);
                vec2 _882 = (_879.xy * 2.0) - vec2(1.0);
                vec3 _904;
                if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseNormalDetailMap != 0u)
                {
                    vec2 _902 = _882.xy + (((
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uNormalDetailMap)], g_xFilteredWrap)), _777.zw * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fNormalDetailUVScale, FrameConstants.g_fGlobalSamplerBias).xy * 2.0) - vec2(1.0)) * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fNormalDetailValueScale);
                    _904 = vec3(_902.x, _902.y, _500.z);
                }
                else
                {
                    _904 = vec3(_882.x, _882.y, _500.z);
                }
                vec3 _907 = _904;
                _907.y = -_904.y;
                vec3 _913 = _907;
                _913.z = sqrt(clamp(1.0 - dot(_907.xy, _907.xy), 0.0, 1.0));
                _917 = mix(_522 * _913, _870, vec3(in_var_NORMAL0.w));
            }
            else
            {
                _917 = _870;
            }
            _919 = normalize(_917);
            break;
        }
        else
        {
            _919 = _517;
            break;
        }
    } while(false);
    vec3 _923 = FrameConstants.g_xCamPos - in_var_TEXCOORD1.xyz;
    vec4 _968 = vec4(_919, 1.0);
    vec3 _970 = _500;
    _970.x = dot(_968, SHConstants.g_xRedStaticSH);
    vec3 _972 = _970;
    _972.y = dot(_968, SHConstants.g_xGreenStaticSH);
    vec3 _974 = _972;
    _974.z = dot(_968, SHConstants.g_xBlueStaticSH);
    vec3 _976 = max(_974, vec3(0.0)) * 2.0;
    vec4 _1079;
    if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseAlbedo != 0u)
    {
        vec4 _995 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uAlbedoMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias);
        vec4 _1024;
        if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseAlbedoDetailMap != 0u)
        {
            vec3 _1007 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uAlbedoDetailMap)], g_xFilteredWrap)), _777.xy * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fAlbedoDetailUVScale, FrameConstants.g_fGlobalSamplerBias).xyz;
            vec3 _1008 = _995.xyz;
            vec3 _1022 = clamp(mix(_1008, mix((_1007 * 2.0) * _1008, vec3(1.0) - (((vec3(1.0) - _1007) * 2.0) * (vec3(1.0) - _1008)), step(vec3(0.5), _1008)), vec3(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fAlbedoDetailValueScale)), vec3(0.0), vec3(1.0));
            _1024 = vec4(_1022.x, _1022.y, _1022.z, _995.w);
        }
        else
        {
            _1024 = _995;
        }
        vec3 _1028 = _1024.xyz * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xAlbedoTint;
        vec4 _1029 = vec4(_1028.x, _1028.y, _1028.z, _1024.w);
        vec4 _1075;
        if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseAlbedo != 0u)
        {
            vec4 _1038 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uAlbedoMap)], g_xFilteredWrap)), _777.zw, FrameConstants.g_fGlobalSamplerBias);
            vec4 _1067;
            if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseAlbedoDetailMap != 0u)
            {
                vec3 _1050 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uAlbedoDetailMap)], g_xFilteredWrap)), _777.zw * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fAlbedoDetailUVScale, FrameConstants.g_fGlobalSamplerBias).xyz;
                vec3 _1051 = _1038.xyz;
                vec3 _1065 = clamp(mix(_1051, mix((_1050 * 2.0) * _1051, vec3(1.0) - (((vec3(1.0) - _1050) * 2.0) * (vec3(1.0) - _1051)), step(vec3(0.5), _1051)), vec3(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_fAlbedoDetailValueScale)), vec3(0.0), vec3(1.0));
                _1067 = vec4(_1065.x, _1065.y, _1065.z, _1038.w);
            }
            else
            {
                _1067 = _1038;
            }
            vec3 _1071 = _1067.xyz * g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_xAlbedoTint;
            _1075 = mix(vec4(_1071.x, _1071.y, _1071.z, _1067.w), _1029, vec4(in_var_NORMAL0.w));
        }
        else
        {
            _1075 = _1029;
        }
        vec4 _1078 = vec4(_1075.x, _1075.y, _1075.z, _501.w);
        _1078.w = _1075.w;
        _1079 = _1078;
    }
    else
    {
        vec4 _988 = vec4(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xAlbedoTint.x, g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xAlbedoTint.y, g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_xAlbedoTint.z, _501.w);
        _988.w = 1.0;
        _1079 = _988;
    }
    float _1084 = g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fMaterialModel * 0.0039215688593685626983642578125;
    vec4 _1216;
    if (_1084 == 0.0)
    {
        float _1212;
        do
        {
            float _1196;
            if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseMetalMap != 0u)
            {
                _1196 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uMetalMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias).w;
            }
            else
            {
                _1196 = 0.0;
            }
            if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseMetalMap != 0u)
            {
                _1212 = mix(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uMetalMap)], g_xFilteredWrap)), _777.zw, FrameConstants.g_fGlobalSamplerBias).w, _1196, in_var_NORMAL0.w);
                break;
            }
            else
            {
                _1212 = _1196;
                break;
            }
        } while(false);
        vec3 _1214 = _976 * (1.0 - _1212);
        _1216 = vec4(_1214.x, _1214.y, _1214.z, _501.w);
    }
    else
    {
        vec4 _1180;
        if (_1084 == 0.0039215688593685626983642578125)
        {
            float _1165;
            if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseMetalMap != 0u)
            {
                _1165 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uMetalMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias).w;
            }
            else
            {
                _1165 = 1.0;
            }
            vec3 _1178 = _976 + (_1079.xyz * (_1165 * clamp((clamp(dot(SHConstants.g_xStaticSpecularDirection, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(SHConstants.g_xStaticSpecularDirection, -_519), 0.0, 1.0), 7.0), 0.0, 1.0)));
            _1180 = vec4(_1178.x, _1178.y, _1178.z, _501.w);
        }
        else
        {
            vec4 _1151;
            if (_1084 == 0.01960784383118152618408203125)
            {
                float _1129;
                do
                {
                    float _1113;
                    if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseMetalMap != 0u)
                    {
                        _1113 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uMetalMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias).w;
                    }
                    else
                    {
                        _1113 = 0.5;
                    }
                    if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseMetalMap != 0u)
                    {
                        _1129 = mix(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uMetalMap)], g_xFilteredWrap)), _777.zw, FrameConstants.g_fGlobalSamplerBias).w, _1113, in_var_NORMAL0.w);
                        break;
                    }
                    else
                    {
                        _1129 = _1113;
                        break;
                    }
                } while(false);
                vec3 _1149 = mix(_976 * 1.0, _976 + ((_1079.xyz * clamp((clamp(dot(SHConstants.g_xStaticSpecularDirection, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(SHConstants.g_xStaticSpecularDirection, -_519), 0.0, 1.0), 7.0), 0.0, 1.0)) * ((g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bIsTransmissive != 0u) ? 0.5 : 0.0)), vec3(_1129));
                _1151 = vec4(_1149.x, _1149.y, _1149.z, _501.w);
            }
            else
            {
                _1151 = vec4(_976.x, _976.y, _976.z, _501.w);
            }
            _1180 = _1151;
        }
        _1216 = _1180;
    }
    float _1217 = dot(_919, _519);
    float _1238;
    do
    {
        if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseSelfIlluminationGraph != 0u)
        {
            _1238 = (TEXTURE_LOD(SAMPLER_2D(g_xMaterialsTexture, g_xTrilinearClamp), vec2((min(1.0, _1217) * 0.498046875) + 0.5, g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fMaterialIndex), 0.0) * 2.0).y;
            break;
        }
        else
        {
            _1238 = 1.0;
            break;
        }
    } while(false);
    vec3 _1267;
    if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseSelfIlluminationMap != 0u)
    {
        vec4 _1252 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uSelfIlluminationMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias);
        float _1253 = _1252.w;
        float _1265;
        if (g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_bUseSelfIlluminationMap != 0u)
        {
            _1265 = mix(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material2Constants.g_xStaticMaterial2.m_uStructuredBufferIndex].m_uSelfIlluminationMap)], g_xFilteredWrap)), _777.zw, FrameConstants.g_fGlobalSamplerBias).w, _1253, in_var_NORMAL0.w);
        }
        else
        {
            _1265 = _1253;
        }
        _1267 = DynamicMaterialConstants.g_xSelfIlluminationParameters * _1265;
    }
    else
    {
        _1267 = DynamicMaterialConstants.g_xSelfIlluminationParameters;
    }
    vec3 _1270 = _1216.xyz + (_1267 * _1238);
    float _1274 = clamp(dot(_519, normalize(_519 + SHConstants.g_xStaticSpecularDirection)), 0.0, 1.0);
    float _1281 = (dot(_976, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)) * 4.0) * (1.0 / ((3.0 * (_1274 * _1274)) + 1.0));
    float _1298;
    do
    {
        if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseHairNoiseMap != 0u)
        {
            _1298 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uHairNoiseMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias).w;
            break;
        }
        else
        {
            _1298 = 0.5;
            break;
        }
    } while(false);
    bool _1299 = _1298 < 0.0039215688593685626983642578125;
    vec4 _1516;
    vec3 _1517;
    if (_1299)
    {
        vec3 _1450 = _1079.xyz * _1270.xyz;
        vec3 _1510;
        if (SHConstants.g_iCubeMapIndex >= 0)
        {
            vec4 _1474 = vec4((vec4(normalize(reflect(-_519, _919)), 0.0) * g_axPBRCubemapDatas._m0[uint(SHConstants.g_iCubeMapIndex)].m_xObbSpace).xyz * vec3(1.0, -1.0, 1.0), float(SHConstants.g_iCubeMapIndex));
            vec4 _1491 = (vec4(-1.0, -0.0274999998509883880615234375, -0.572000026702880859375, 0.02199999988079071044921875) * (_1079.w * _1079.w)) + vec4(1.0, 0.0425000004470348358154296875, 1.03999996185302734375, -0.039999999105930328369140625);
            float _1492 = _1491.x;
            vec2 _1502 = (vec2(-1.03999996185302734375, 1.03999996185302734375) * ((min(_1492 * _1492, exp2((-9.27999973297119140625) * clamp(_1217, 0.0, 1.0))) * _1492) + _1491.y)) + _1491.zw;
            _1510 = _1450 + ((textureLod(samplerCubeArray(g_axPBRCubeMaps, g_xTrilinearClamp), _1474, _1079.w * 6.0).xyz * clamp(_1281 / max(0.00999999977648258209228515625, dot(textureLod(samplerCubeArray(g_axPBRCubeMaps, g_xTrilinearClamp), _1474, 6.0).xyz, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375))), 0.0, 1.0)) * ((vec3(0.0280000008642673492431640625) * _1502.x) + vec3(_1502.y)));
        }
        else
        {
            _1510 = _1450;
        }
        _1516 = vec4(min(_1510, vec3(1000.0)), _820);
        _1517 = SHConstants.g_xStaticSpecularDirection;
    }
    else
    {
        vec3 _1375;
        do
        {
            if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseHairFlowMap != 0u)
            {
                vec2 _1319 = (
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_uHairFlowMap)], g_xFilteredWrap)), _777.xy, FrameConstants.g_fGlobalSamplerBias).xy * 2.0) - vec2(1.0);
                vec3 _1323 = vec3(_1319.x, _1319.y, _500.z);
                _1323.y = -_1319.y;
                vec3 _1324 = _1323;
                _1324.z = 0.0;
                vec4 _1332 = vec4(cross(_517, _919), dot(_517, _919) + 1.0);
                float _1333 = length(_1332);
                vec4 _1338 = mix(_1332, _1332 / vec4(_1333), bvec4(_1333 > 0.0));
                vec3 _1340 = _1338.xyz * 2.0;
                vec3 _1342 = _1340 * _1338.w;
                vec3 _1344 = _1340 * _1338.x;
                vec3 _1347 = _1338.yyz * _1340.yzz;
                float _1348 = _1347.x;
                float _1349 = _1347.z;
                float _1352 = _1344.y;
                float _1353 = _1342.z;
                float _1355 = _1344.z;
                float _1356 = _1342.y;
                float _1359 = _1344.x;
                float _1362 = _1347.y;
                float _1363 = _1342.x;
                _1375 = normalize(mat3(vec3(1.0 - (_1348 + _1349), _1352 + _1353, _1355 - _1356), vec3(_1352 - _1353, 1.0 - (_1359 + _1349), _1362 + _1363), vec3(_1355 + _1356, _1362 - _1363, 1.0 - (_1359 + _1348))) * (_522 * _1324));
                break;
            }
            else
            {
                _1375 = _521;
                break;
            }
        } while(false);
        vec4 _1381 = vec4(_1079.xyz * _1270.xyz, _820);
        float _1385 = clamp(1.0 - _1079.w, 0.0, 1.0);
        float _1387 = ((_1298 * 2.0) - 1.0) * 0.300000011920928955078125;
        float _1393 = pow(_1385, 8.0) * 500.0;
        float _1396 = pow(_1385, 6.0) * 2000.0;
        float _1410 = dot(normalize(_1375 + (_919 * ((-0.0500000007450580596923828125) + _1387))), _519);
        float _1430 = dot(normalize(_1375 + (_919 * ((0.0500000007450580596923828125 + _1387) - (_1385 * 0.0500000007450580596923828125)))), _519);
        vec3 _1447 = _1381.xyz + ((((vec3(((smoothstep(-1.0, 0.0, _1410) * pow(sqrt(clamp(1.0 - (_1410 * _1410), 0.0, 1.0)), 20.0 + _1393)) * ((((_1298 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _1385))) * ((_1393 + 28.0) * 0.03978873789310455322265625)) * (_1079.xyz / vec3(max(dot(_1079.xyz, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _1430) * pow(sqrt(clamp(1.0 - (_1430 * _1430), 0.0, 1.0)), 50.0 + _1396)) * ((((_1298 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _1385)) * ((_1396 + 58.0) * 0.03978873789310455322265625))) * _1281) * 0.5);
        _1516 = vec4(_1447.x, _1447.y, _1447.z, _1381.w);
        _1517 = _1375;
    }
    float _1518 = _1299 ? 0.007843137718737125396728515625 : _1084;
    vec3 _1520 = normalize(FrameConstants.g_xCamPos - _923);
    vec4 _2123;
    if (any(greaterThan(DynamicLightsParams.g_xSunColour, vec3(0.0))))
    {
        float _1526 = (_1518 != 0.01960784383118152618408203125) ? _1298 : 0.0;
        vec3 _1528 = normalize(_1520 + DynamicLightsParams.g_xSunDir);
        float _1530 = clamp(dot(_919, DynamicLightsParams.g_xSunDir), 0.0, 1.0);
        float _1532 = clamp(dot(_919, _1520), 0.0, 1.0);
        float _1534 = clamp(dot(_919, _1528), 0.0, 1.0);
        float _1536 = clamp(dot(_1520, _1528), 0.0, 1.0);
        bool _1537 = _1518 == 0.01568627543747425079345703125;
        vec3 _1574;
        vec3 _1575;
        vec3 _1576;
        if (_1537)
        {
            _1574 = vec3(0.046999998390674591064453125);
            _1575 = vec3(_1530);
            _1576 = _1079.xyz;
        }
        else
        {
            vec3 _1570;
            vec3 _1571;
            vec3 _1572;
            if (_1518 == 0.007843137718737125396728515625)
            {
                _1570 = vec3(0.0280000008642673492431640625);
                _1571 = vec3(_1530);
                _1572 = _1079.xyz;
            }
            else
            {
                vec3 _1566;
                vec3 _1567;
                vec3 _1568;
                if (_1518 == 0.0039215688593685626983642578125)
                {
                    _1566 = vec3(0.0280000008642673492431640625);
                    _1567 = vec3((clamp((clamp(dot(DynamicLightsParams.g_xSunDir, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(DynamicLightsParams.g_xSunDir, -_1520), 0.0, 1.0), 7.0), 0.0, 1.0) * _1526) + _1530);
                    _1568 = _1079.xyz;
                }
                else
                {
                    vec3 _1549 = vec3(_1526);
                    _1566 = mix(vec3(0.039999999105930328369140625), _1079.xyz, _1549);
                    _1567 = vec3(_1530);
                    _1568 = mix(_1079.xyz, vec3(0.0), _1549);
                }
                _1570 = _1566;
                _1571 = _1567;
                _1572 = _1568;
            }
            _1574 = _1570;
            _1575 = _1571;
            _1576 = _1572;
        }
        vec3 _1577 = _1575 * _1576;
        float _1578 = max(_1079.w, 0.0500000007450580596923828125);
        float _1579 = _1578 * _1578;
        vec3 _1675;
        if (_1537)
        {
            float _1613 = clamp(1.0 - _1079.w, 0.0, 1.0);
            float _1615 = ((_1298 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _1621 = pow(_1613, 8.0) * 500.0;
            float _1624 = pow(_1613, 6.0) * 2000.0;
            float _1638 = dot(normalize(_1517 + (_919 * ((-0.0500000007450580596923828125) + _1615))), _1528);
            float _1658 = dot(normalize(_1517 + (_919 * ((0.0500000007450580596923828125 + _1615) - (_1613 * 0.0500000007450580596923828125)))), _1528);
            _1675 = _1577 + ((((vec3(((smoothstep(-1.0, 0.0, _1638) * pow(sqrt(clamp(1.0 - (_1638 * _1638), 0.0, 1.0)), 20.0 + _1621)) * ((((_1298 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _1613))) * ((_1621 + 28.0) * 0.03978873789310455322265625)) * (_1079.xyz / vec3(max(dot(_1079.xyz, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _1658) * pow(sqrt(clamp(1.0 - (_1658 * _1658), 0.0, 1.0)), 50.0 + _1624)) * ((((_1298 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _1613)) * ((_1624 + 58.0) * 0.03978873789310455322265625))) * _1530) * 1.0);
        }
        else
        {
            float _1590 = 2.0 - _1579;
            float _1597 = _1579 * _1579;
            float _1601 = (((_1534 * _1597) - _1534) * _1534) + 1.0;
            _1675 = _1577 + (((((vec3(1.0) - _1574) * exp2((((-5.554729938507080078125) * _1536) - 6.9831600189208984375) * _1536)) + _1574) * (((1.0 / (((_1532 * _1590) + _1579) * ((_1530 * _1590) + _1579))) * (_1597 * (1.0 / (_1601 * _1601)))) * _1530)) * 1.0);
        }
        vec3 _1724;
        if (_1518 == 0.01960784383118152618408203125)
        {
            bool _1679 = _1298 > 0.4980392158031463623046875;
            vec3 _1697 = sqrt(abs(_1576));
            float _1711 = 0.5 / _1579;
            _1724 = mix(_1675, (_1577 + (((((vec3(1.0) - _1697) * exp2((((-5.554729938507080078125) * _1536) - 6.9831600189208984375) * _1536)) + _1697) * ((1.0 / ((4.0 * ((_1530 + _1532) - (_1530 * _1532))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _1711) * pow(1.0 - (_1534 * _1534), _1711)))) * _1530)) + (_1576 * (clamp((clamp(dot(DynamicLightsParams.g_xSunDir, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(DynamicLightsParams.g_xSunDir, -_1520), 0.0, 1.0), 7.0), 0.0, 1.0) * (_1679 ? 0.5 : 0.0))), vec3((_1679 ? (_1298 - 0.501960813999176025390625) : _1298) * 2.007874011993408203125));
        }
        else
        {
            _1724 = _1675;
        }
        vec3 _1726 = min(_1724 * DynamicLightsParams.g_xSunColour, vec3(1000.0));
        vec4 _1731 = vec4(_923, 1.0);
        float _1783;
        if (ProjectedCloudParams.g_bSunlightCloudsEnabled != 0u)
        {
            float _1782;
            if (abs(ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y) > 9.9999999747524270787835121154785e-07)
            {
                vec3 _1744 = _1731.xyz;
                float _1747 = (vec3(0.0, -10000.0, 0.0) - _1744).y / ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y;
                float _1781;
                if (_1747 > 0.0)
                {
                    vec3 _1753 = _1744 + (ProjectedCloudParams.g_xSunlightCloudsProjectionDir.xyz * _1747);
                    vec2 _1762 = _1753.xz;
                    _1762.x = _1753.x + (FrameConstants.g_fScaledTime * ProjectedCloudParams.g_fSunlightCloudsVelocityX);
                    vec2 _1768 = _1762;
                    _1768.y = _1753.z + (FrameConstants.g_fScaledTime * ProjectedCloudParams.g_fSunlightCloudsVelocityY);
                    vec2 _1772 = _1768 / vec2(ProjectedCloudParams.g_fSunlightCloudsScale);
                    vec2 _1775 = _1772;
                    _1775.x = _1772.x * (-1.0);
                    _1781 = TEXTURE_LOD(SAMPLER_2D(g_xLighting_Clouds, g_xBilinearWrap), _1775, 0.0).x;
                }
                else
                {
                    _1781 = 1.0;
                }
                _1782 = _1781;
            }
            else
            {
                _1782 = 1.0;
            }
            _1783 = _1782;
        }
        else
        {
            _1783 = 1.0;
        }
        vec4 _1784 = vec4(_1726.x, _1726.y, _1726.z, vec4(0.0, 0.0, 0.0, 1.0).w);
        _1784.w = _1783;
        bool _1786 = _1518 == 0.007843137718737125396728515625;
        vec3 _1797;
        if (_1786)
        {
            _1797 = _1784.xyz + (((_1079.xyz * vec3(0.0194447897374629974365234375, 1.6805228369776159524917602539062e-05, 3.1907067266079280898338765837252e-11)) * clamp(0.300000011920928955078125 + dot(DynamicLightsParams.g_xSunDir, -_919), 0.0, 1.0)) * DynamicLightsParams.g_xSunColour);
        }
        else
        {
            _1797 = _1784.xyz;
        }
        vec3 _1810;
        if (_1786 && (in_var_TEXCOORD1.w < 3.0))
        {
            _1810 = _1797.xyz * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(in_var_TEXCOORD1.w - 2.0, 0.0)));
        }
        else
        {
            _1810 = _1797.xyz;
        }
        vec4 _2116;
        uint _1813 = 0u;
        for (;;)
        {
            if (_1813 < CSMParams.g_xCSMData.g_uNumMaps)
            {
                vec4 _1821 = _1731 * CSMParams.g_xCSMData.g_axShadowViewProj[_1813];
                vec2 _1824 = (_1821.xy * vec2(0.5, -0.5)) + vec2(0.5);
                float _1827 = clamp(_1821.z + 1.525902189314365386962890625e-05, 0.0, 1.0);
                vec2 _1828 = _1824.xy;
                if (all(lessThan(abs(_1828 - vec2(0.5)), vec2(0.5))))
                {
                    float _1956;
                    switch (_1813)
                    {
                        case 1u:
                        {
                            float _1911 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[1].x;
                            float _1917;
                            _1917 = texture(sampler2DShadow(g_axCSMMap1, g_xShadowCmpSampler), vec3(_1828, _1827));
                            for (int _1920 = 0; _1920 < 8; )
                            {
                                _1917 += textureLod(sampler2DShadow(g_axCSMMap1, g_xShadowCmpSampler), vec3(_1828 + ((_440[_1920] * _1911) / vec2(CSMParams.g_xCSMData.g_axSize[1].x)), _1827), 0.0);
                                _1920++;
                                continue;
                            }
                            _1956 = _1917 * 0.111111111938953399658203125;
                            break;
                        }
                        case 2u:
                        {
                            float _1888 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[2].x;
                            float _1894;
                            _1894 = texture(sampler2DShadow(g_axCSMMap2, g_xShadowCmpSampler), vec3(_1828, _1827));
                            for (int _1897 = 0; _1897 < 8; )
                            {
                                _1894 += textureLod(sampler2DShadow(g_axCSMMap2, g_xShadowCmpSampler), vec3(_1828 + ((_440[_1897] * _1888) / vec2(CSMParams.g_xCSMData.g_axSize[2].x)), _1827), 0.0);
                                _1897++;
                                continue;
                            }
                            _1956 = _1894 * 0.111111111938953399658203125;
                            break;
                        }
                        case 3u:
                        {
                            float _1865 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[3].x;
                            float _1871;
                            _1871 = texture(sampler2DShadow(g_axCSMMap3, g_xShadowCmpSampler), vec3(_1828, _1827));
                            for (int _1874 = 0; _1874 < 8; )
                            {
                                _1871 += textureLod(sampler2DShadow(g_axCSMMap3, g_xShadowCmpSampler), vec3(_1828 + ((_440[_1874] * _1865) / vec2(CSMParams.g_xCSMData.g_axSize[3].x)), _1827), 0.0);
                                _1874++;
                                continue;
                            }
                            _1956 = _1871 * 0.111111111938953399658203125;
                            break;
                        }
                        case 4u:
                        {
                            float _1842 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[4].x;
                            float _1848;
                            _1848 = texture(sampler2DShadow(g_axCSMMap4, g_xShadowCmpSampler), vec3(_1828, _1827));
                            for (int _1851 = 0; _1851 < 8; )
                            {
                                _1848 += textureLod(sampler2DShadow(g_axCSMMap4, g_xShadowCmpSampler), vec3(_1828 + ((_440[_1851] * _1842) / vec2(CSMParams.g_xCSMData.g_axSize[4].x)), _1827), 0.0);
                                _1851++;
                                continue;
                            }
                            _1956 = _1848 * 0.111111111938953399658203125;
                            break;
                        }
                        default:
                        {
                            float _1934 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[0].x;
                            float _1940;
                            _1940 = texture(sampler2DShadow(g_axCSMMap0, g_xShadowCmpSampler), vec3(_1828, _1827));
                            for (int _1943 = 0; _1943 < 8; )
                            {
                                _1940 += textureLod(sampler2DShadow(g_axCSMMap0, g_xShadowCmpSampler), vec3(_1828 + ((_440[_1943] * _1934) / vec2(CSMParams.g_xCSMData.g_axSize[0].x)), _1827), 0.0);
                                _1943++;
                                continue;
                            }
                            _1956 = _1940 * 0.111111111938953399658203125;
                            break;
                        }
                    }
                    uint _1957 = _1813 + 1u;
                    float _2111;
                    if (_1957 < CSMParams.g_xCSMData.g_uNumMaps)
                    {
                        vec2 _1961 = vec2(1.0) - _1828;
                        vec4 _1973 = _1731 * CSMParams.g_xCSMData.g_axShadowViewProj[_1957];
                        float _1979 = clamp(_1973.z + 1.525902189314365386962890625e-05, 0.0, 1.0);
                        vec2 _1980 = ((_1973.xy * vec2(0.5, -0.5)) + vec2(0.5)).xy;
                        float _2110;
                        if (all(lessThan(abs(_1980 - vec2(0.5)), vec2(0.5))))
                        {
                            float _2108;
                            switch (_1957)
                            {
                                case 1u:
                                {
                                    float _2063 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[1].x;
                                    float _2069;
                                    _2069 = texture(sampler2DShadow(g_axCSMMap1, g_xShadowCmpSampler), vec3(_1980, _1979));
                                    for (int _2072 = 0; _2072 < 8; )
                                    {
                                        _2069 += textureLod(sampler2DShadow(g_axCSMMap1, g_xShadowCmpSampler), vec3(_1980 + ((_440[_2072] * _2063) / vec2(CSMParams.g_xCSMData.g_axSize[1].x)), _1979), 0.0);
                                        _2072++;
                                        continue;
                                    }
                                    _2108 = _2069 * 0.111111111938953399658203125;
                                    break;
                                }
                                case 2u:
                                {
                                    float _2040 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[2].x;
                                    float _2046;
                                    _2046 = texture(sampler2DShadow(g_axCSMMap2, g_xShadowCmpSampler), vec3(_1980, _1979));
                                    for (int _2049 = 0; _2049 < 8; )
                                    {
                                        _2046 += textureLod(sampler2DShadow(g_axCSMMap2, g_xShadowCmpSampler), vec3(_1980 + ((_440[_2049] * _2040) / vec2(CSMParams.g_xCSMData.g_axSize[2].x)), _1979), 0.0);
                                        _2049++;
                                        continue;
                                    }
                                    _2108 = _2046 * 0.111111111938953399658203125;
                                    break;
                                }
                                case 3u:
                                {
                                    float _2017 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[3].x;
                                    float _2023;
                                    _2023 = texture(sampler2DShadow(g_axCSMMap3, g_xShadowCmpSampler), vec3(_1980, _1979));
                                    for (int _2026 = 0; _2026 < 8; )
                                    {
                                        _2023 += textureLod(sampler2DShadow(g_axCSMMap3, g_xShadowCmpSampler), vec3(_1980 + ((_440[_2026] * _2017) / vec2(CSMParams.g_xCSMData.g_axSize[3].x)), _1979), 0.0);
                                        _2026++;
                                        continue;
                                    }
                                    _2108 = _2023 * 0.111111111938953399658203125;
                                    break;
                                }
                                case 4u:
                                {
                                    float _1994 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[4].x;
                                    float _2000;
                                    _2000 = texture(sampler2DShadow(g_axCSMMap4, g_xShadowCmpSampler), vec3(_1980, _1979));
                                    for (int _2003 = 0; _2003 < 8; )
                                    {
                                        _2000 += textureLod(sampler2DShadow(g_axCSMMap4, g_xShadowCmpSampler), vec3(_1980 + ((_440[_2003] * _1994) / vec2(CSMParams.g_xCSMData.g_axSize[4].x)), _1979), 0.0);
                                        _2003++;
                                        continue;
                                    }
                                    _2108 = _2000 * 0.111111111938953399658203125;
                                    break;
                                }
                                default:
                                {
                                    float _2086 = 1.5 * CSMParams.g_xCSMData.g_axAreaScale[0].x;
                                    float _2092;
                                    _2092 = texture(sampler2DShadow(g_axCSMMap0, g_xShadowCmpSampler), vec3(_1980, _1979));
                                    for (int _2095 = 0; _2095 < 8; )
                                    {
                                        _2092 += textureLod(sampler2DShadow(g_axCSMMap0, g_xShadowCmpSampler), vec3(_1980 + ((_440[_2095] * _2086) / vec2(CSMParams.g_xCSMData.g_axSize[0].x)), _1979), 0.0);
                                        _2095++;
                                        continue;
                                    }
                                    _2108 = _2092 * 0.111111111938953399658203125;
                                    break;
                                }
                            }
                            _2110 = mix(_2108, _1956, clamp(min(min(_1824.x, _1824.y), min(_1961.x, _1961.y)) * 20.0, 0.0, 1.0));
                        }
                        else
                        {
                            _2110 = _1956;
                        }
                        _2111 = _2110;
                    }
                    else
                    {
                        _2111 = _1956;
                    }
                    vec4 _2115 = vec4(_1810.x, _1810.y, _1810.z, _1784.w);
                    _2115.w = _1783 * (1.0 - (_2111 * CSMParams.g_xCSMData.g_fStrength));
                    _2116 = _2115;
                    break;
                }
                _1813++;
                continue;
            }
            else
            {
                _2116 = vec4(_1810.x, _1810.y, _1810.z, _1784.w);
                break;
            }
        }
        vec3 _2121 = _1516.xyz + (_2116.xyz * _2116.w);
        _2123 = vec4(_2121.x, _2121.y, _2121.z, _1516.w);
    }
    else
    {
        _2123 = _1516;
    }
    vec4 _3028;
    if (DynamicLightsParams.g_uNumPhotons != 0u)
    {
        vec3 _2127 = _923 * DynamicLightsParams.g_fPhotonSpatialHashScale;
        uint _2143 = uint(((int(trunc(_2127.x)) * 73856093) ^ (int(trunc(_2127.y)) * 19349663)) ^ (int(trunc(_2127.z)) * 83492791)) % 7919u;
        uint _2153;
        uint _2154;
        if (g_axPhotonSpatialHash._m0[_2143] == 0u)
        {
            _2153 = 0u;
            _2154 = g_axPhotonSpatialHash._m0[_2143];
        }
        else
        {
            _2153 = g_axPhotonSpatialHash._m0[g_axPhotonSpatialHash._m0[_2143]];
            _2154 = g_axPhotonSpatialHash._m0[_2143] + 1u;
        }
        uint _2155 = _2154 + _2153;
        vec2 _2157;
        vec4 _2160;
        _2157 = _502;
        _2160 = _2123;
        uint _2163;
        vec2 _2158;
        vec4 _2161;
        for (uint _2162 = _2154; _2162 < _2155; _2157 = _2158, _2160 = _2161, _2162 = _2163)
        {
            _2163 = _2162 + 1u;
            vec4 _3017;
            bool _3018;
            do
            {
                vec3 _2220 = g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_xLightPos - _923;
                float _2221 = length(_2220);
                vec3 _2223 = _2220 / vec3(_2221);
                float _2231 = pow(clamp((1.0 - clamp((_2221 - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fAreaLightRadius) / (g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fLightRange - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fAreaLightRadius), 0.0, 1.0)) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fLightFalloffScale, 0.0, 1.0), g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fLightFalloffPower);
                float _2239;
                if (g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_bUseLimitAngles != 0u)
                {
                    _2239 = _2231 * clamp((dot(_2223, g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_xLightDir) + g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fLightAngleOuter) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fRcpLightAngleRange, 0.0, 1.0);
                }
                else
                {
                    _2239 = _2231;
                }
                if (_2239 < 9.9999997473787516355514526367188e-05)
                {
                    _2158 = _2157;
                    _3017 = vec4(0.0, 0.0, 0.0, 1.0);
                    _3018 = false;
                    break;
                }
                float _2244 = clamp(dot(_919, _2223), 0.0, 1.0);
                vec3 _2248 = -_2220;
                vec3 _2251 = _2248;
                _2251.y = -_2248.y;
                vec3 _2252 = abs(_2251);
                float _2253 = _2252.x;
                float _2254 = _2252.y;
                float _2255 = _2252.z;
                float _2257 = max(_2253, max(_2254, _2255));
                vec3 _2259 = _2251 / vec3(_2257);
                float _2264 = g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_xDepthProjection.x + (g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_xDepthProjection.y / (_2257 - max(g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowBias * (1.0 - _2244), g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowBias)));
                float _2269 = (2.0 * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowFarPlane;
                float _2270 = g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowFarPlane + g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane;
                float _2271 = g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowFarPlane - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane;
                float _2277 = clamp(((_2269 / (_2270 - ((((1.0 - _2264) * 2.0) - 1.0) * _2271))) - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane) / _2271, 0.0, 1.0);
                vec2 _2364;
                int _2365;
                if (_2254 > _2253)
                {
                    vec2 _2362;
                    int _2363;
                    if (_2255 > _2254)
                    {
                        bool _2345 = _2259.z < 0.0;
                        vec2 _2360;
                        if (_2345)
                        {
                            vec2 _2356 = _2157;
                            _2356.x = -_2259.x;
                            vec2 _2359 = _2356;
                            _2359.y = -_2259.y;
                            _2360 = _2359;
                        }
                        else
                        {
                            vec2 _2350 = _2157;
                            _2350.x = _2259.x;
                            vec2 _2353 = _2350;
                            _2353.y = -_2259.y;
                            _2360 = _2353;
                        }
                        _2362 = _2360;
                        _2363 = _2345 ? g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[5].iShadowIndex : g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[4].iShadowIndex;
                    }
                    else
                    {
                        bool _2329 = _2259.y < 0.0;
                        vec2 _2342;
                        if (_2329)
                        {
                            vec2 _2338 = _2157;
                            _2338.x = _2259.x;
                            vec2 _2341 = _2338;
                            _2341.y = -_2259.z;
                            _2342 = _2341;
                        }
                        else
                        {
                            vec2 _2334 = _2157;
                            _2334.x = _2259.x;
                            vec2 _2336 = _2334;
                            _2336.y = _2259.z;
                            _2342 = _2336;
                        }
                        _2362 = _2342;
                        _2363 = _2329 ? g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[2].iShadowIndex : g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[3].iShadowIndex;
                    }
                    _2364 = _2362;
                    _2365 = _2363;
                }
                else
                {
                    vec2 _2322;
                    int _2323;
                    if (_2255 > _2253)
                    {
                        bool _2305 = _2259.z < 0.0;
                        vec2 _2320;
                        if (_2305)
                        {
                            vec2 _2316 = _2157;
                            _2316.x = -_2259.x;
                            vec2 _2319 = _2316;
                            _2319.y = -_2259.y;
                            _2320 = _2319;
                        }
                        else
                        {
                            vec2 _2310 = _2157;
                            _2310.x = _2259.x;
                            vec2 _2313 = _2310;
                            _2313.y = -_2259.y;
                            _2320 = _2313;
                        }
                        _2322 = _2320;
                        _2323 = _2305 ? g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[5].iShadowIndex : g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[4].iShadowIndex;
                    }
                    else
                    {
                        bool _2287 = _2259.x < 0.0;
                        vec2 _2302;
                        if (_2287)
                        {
                            vec2 _2298 = _2157;
                            _2298.x = _2259.z;
                            vec2 _2301 = _2298;
                            _2301.y = -_2259.y;
                            _2302 = _2301;
                        }
                        else
                        {
                            vec2 _2293 = _2157;
                            _2293.x = -_2259.z;
                            vec2 _2296 = _2293;
                            _2296.y = -_2259.y;
                            _2302 = _2296;
                        }
                        _2322 = _2302;
                        _2323 = _2287 ? g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[1].iShadowIndex : g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[0].iShadowIndex;
                    }
                    _2364 = _2322;
                    _2365 = _2323;
                }
                vec2 _2367 = (_2364 * 0.5) + vec2(0.5);
                bool _2368 = _2365 > (-1);
                vec4 _2782;
                if (_2368)
                {
                    float _2778;
                    do
                    {
                        if (_2365 == (-1))
                        {
                            _2778 = 0.0;
                            break;
                        }
                        else
                        {
                            float _2777;
                            if (g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowSoftness > 0.0)
                            {
                                float _2485 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                                float _2486 = g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowSoftness * 0.02500000037252902984619140625;
                                float _2488 = float(uint(0));
                                float _2490 = sqrt(_2488 + 0.5);
                                float _2491 = _2490 * 0.5;
                                float _2493 = (_2488 * 2.400000095367431640625) + _2485;
                                float _2494 = sin(_2493);
                                float _2495 = cos(_2493);
                                uint _2502 = uint(_2365);
                                vec4 _2506 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(_2502)], g_xBilinearClamp)), clamp(_2367 + (vec2(_2491 * _2495, _2491 * _2494) * _2486), vec2(0.0), vec2(1.0)), FrameConstants.g_fGlobalSamplerBias);
                                float _2517 = clamp(((_2269 / (_2270 - ((((1.0 - (_2368 ? _2506.x : 0.0)) * 2.0) - 1.0) * _2271))) - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane) / _2271, 0.0, 1.0);
                                vec2 _2523;
                                if (_2517 < _2277)
                                {
                                    vec2 _2521 = _503;
                                    _2521.x = _2517;
                                    vec2 _2522 = _2521;
                                    _2522.y = 1.0;
                                    _2523 = _2522;
                                }
                                else
                                {
                                    _2523 = vec2(0.0);
                                }
                                float _2525 = float(uint(1));
                                float _2527 = sqrt(_2525 + 0.5);
                                float _2528 = _2527 * 0.5;
                                float _2530 = (_2525 * 2.400000095367431640625) + _2485;
                                float _2531 = sin(_2530);
                                float _2532 = cos(_2530);
                                vec4 _2542 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_2502], g_xBilinearClamp), clamp(_2367 + (vec2(_2528 * _2532, _2528 * _2531) * _2486), vec2(0.0), vec2(1.0)), FrameConstants.g_fGlobalSamplerBias);
                                float _2553 = clamp(((_2269 / (_2270 - ((((1.0 - (_2368 ? _2542.x : 0.0)) * 2.0) - 1.0) * _2271))) - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane) / _2271, 0.0, 1.0);
                                vec2 _2563;
                                if (_2553 < _2277)
                                {
                                    vec2 _2559 = _503;
                                    _2559.x = _2523.x + _2553;
                                    vec2 _2562 = _2559;
                                    _2562.y = _2523.y + 1.0;
                                    _2563 = _2562;
                                }
                                else
                                {
                                    _2563 = _2523;
                                }
                                float _2565 = float(uint(2));
                                float _2567 = sqrt(_2565 + 0.5);
                                float _2568 = _2567 * 0.5;
                                float _2570 = (_2565 * 2.400000095367431640625) + _2485;
                                float _2571 = sin(_2570);
                                float _2572 = cos(_2570);
                                vec4 _2580 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_2502], g_xBilinearClamp), clamp(_2367 + (vec2(_2568 * _2572, _2568 * _2571) * _2486), vec2(0.0), vec2(1.0)), FrameConstants.g_fGlobalSamplerBias);
                                float _2591 = clamp(((_2269 / (_2270 - ((((1.0 - (_2368 ? _2580.x : 0.0)) * 2.0) - 1.0) * _2271))) - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane) / _2271, 0.0, 1.0);
                                vec2 _2601;
                                if (_2591 < _2277)
                                {
                                    vec2 _2597 = _503;
                                    _2597.x = _2563.x + _2591;
                                    vec2 _2600 = _2597;
                                    _2600.y = _2563.y + 1.0;
                                    _2601 = _2600;
                                }
                                else
                                {
                                    _2601 = _2563;
                                }
                                float _2603 = float(uint(3));
                                float _2605 = sqrt(_2603 + 0.5);
                                float _2606 = _2605 * 0.5;
                                float _2608 = (_2603 * 2.400000095367431640625) + _2485;
                                float _2609 = sin(_2608);
                                float _2610 = cos(_2608);
                                vec4 _2618 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_2502], g_xBilinearClamp), clamp(_2367 + (vec2(_2606 * _2610, _2606 * _2609) * _2486), vec2(0.0), vec2(1.0)), FrameConstants.g_fGlobalSamplerBias);
                                float _2629 = clamp(((_2269 / (_2270 - ((((1.0 - (_2368 ? _2618.x : 0.0)) * 2.0) - 1.0) * _2271))) - g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowNearPlane) / _2271, 0.0, 1.0);
                                vec2 _2639;
                                if (_2629 < _2277)
                                {
                                    vec2 _2635 = _503;
                                    _2635.x = _2601.x + _2629;
                                    vec2 _2638 = _2635;
                                    _2638.y = _2601.y + 1.0;
                                    _2639 = _2638;
                                }
                                else
                                {
                                    _2639 = _2601;
                                }
                                float _2776;
                                if (_2639.y > 0.0)
                                {
                                    float _2645 = _2639.x / _2639.y;
                                    float _2647 = (_2277 - _2645) / _2645;
                                    float _2655 = ((clamp(_2647 * _2647, 0.0, 1.0) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowSoftness) * 0.02500000037252902984619140625) + (2.0 * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize);
                                    float _2656 = _2490 * 0.3535533845424652099609375;
                                    float _2666 = _2527 * 0.3535533845424652099609375;
                                    float _2677 = _2567 * 0.3535533845424652099609375;
                                    float _2688 = _2605 * 0.3535533845424652099609375;
                                    float _2700 = float(uint(4));
                                    float _2703 = sqrt(_2700 + 0.5) * 0.3535533845424652099609375;
                                    float _2705 = (_2700 * 2.400000095367431640625) + _2485;
                                    float _2719 = float(uint(5));
                                    float _2722 = sqrt(_2719 + 0.5) * 0.3535533845424652099609375;
                                    float _2724 = (_2719 * 2.400000095367431640625) + _2485;
                                    float _2738 = float(uint(6));
                                    float _2741 = sqrt(_2738 + 0.5) * 0.3535533845424652099609375;
                                    float _2743 = (_2738 * 2.400000095367431640625) + _2485;
                                    float _2757 = float(uint(7));
                                    float _2760 = sqrt(_2757 + 0.5) * 0.3535533845424652099609375;
                                    float _2762 = (_2757 * 2.400000095367431640625) + _2485;
                                    _2776 = ((((((((_2368 ? texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_2502)], g_xShadowCmpSampler)), vec3(clamp(_2367 + (vec2(_2656 * _2495, _2656 * _2494) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2666 * _2532, _2666 * _2531) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2677 * _2572, _2677 * _2571) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2688 * _2610, _2688 * _2609) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2703 * cos(_2705), _2703 * sin(_2705)) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2722 * cos(_2724), _2722 * sin(_2724)) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2741 * cos(_2743), _2741 * sin(_2743)) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2502], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(_2760 * cos(_2762), _2760 * sin(_2762)) * _2655), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) * 0.125;
                                }
                                else
                                {
                                    _2776 = 0.0;
                                }
                                _2777 = _2776;
                            }
                            else
                            {
                                uint _2386 = uint(_2365);
                                _2777 = ((((((((((((_2368 ? texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_2386)], g_xShadowCmpSampler)), vec3(clamp(_2367 + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) + (_2368 ? texture(sampler2DShadow(g_xTextureArray[_2386], g_xShadowCmpSampler), vec3(clamp(_2367 + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_axFace[_2365].fInvSize), vec2(0.0), vec2(1.0)), _2264)) : 0.0)) * 0.083333335816860198974609375;
                            }
                            _2778 = _2777;
                            break;
                        }
                    } while(false);
                    vec4 _2781 = vec4(0.0, 0.0, 0.0, 1.0);
                    _2781.w = 1.0 - (g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fShadowStrength * _2778);
                    _2782 = _2781;
                }
                else
                {
                    _2782 = vec4(0.0, 0.0, 0.0, 1.0);
                }
                if (_2782.w > 0.0)
                {
                    vec3 _2787 = g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_xLightColour * _2239;
                    float _2789 = (_1518 != 0.01960784383118152618408203125) ? _1298 : 0.0;
                    vec3 _2791 = normalize(_1520 + _2223);
                    float _2793 = clamp(dot(_919, _1520), 0.0, 1.0);
                    float _2795 = clamp(dot(_919, _2791), 0.0, 1.0);
                    float _2797 = clamp(dot(_1520, _2791), 0.0, 1.0);
                    bool _2798 = _1518 == 0.01568627543747425079345703125;
                    vec3 _2835;
                    vec3 _2836;
                    vec3 _2837;
                    if (_2798)
                    {
                        _2835 = vec3(0.046999998390674591064453125);
                        _2836 = vec3(_2244);
                        _2837 = _1079.xyz;
                    }
                    else
                    {
                        vec3 _2831;
                        vec3 _2832;
                        vec3 _2833;
                        if (_1518 == 0.007843137718737125396728515625)
                        {
                            _2831 = vec3(0.0280000008642673492431640625);
                            _2832 = vec3(_2244);
                            _2833 = _1079.xyz;
                        }
                        else
                        {
                            vec3 _2827;
                            vec3 _2828;
                            vec3 _2829;
                            if (_1518 == 0.0039215688593685626983642578125)
                            {
                                _2827 = vec3(0.0280000008642673492431640625);
                                _2828 = vec3((clamp((clamp(dot(_2223, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_2223, -_1520), 0.0, 1.0), 7.0), 0.0, 1.0) * _2789) + _2244);
                                _2829 = _1079.xyz;
                            }
                            else
                            {
                                vec3 _2810 = vec3(_2789);
                                _2827 = mix(vec3(0.039999999105930328369140625), _1079.xyz, _2810);
                                _2828 = vec3(_2244);
                                _2829 = mix(_1079.xyz, vec3(0.0), _2810);
                            }
                            _2831 = _2827;
                            _2832 = _2828;
                            _2833 = _2829;
                        }
                        _2835 = _2831;
                        _2836 = _2832;
                        _2837 = _2833;
                    }
                    vec3 _2838 = _2836 * _2837;
                    float _2839 = max(_1079.w, 0.0500000007450580596923828125);
                    float _2840 = _2839 * _2839;
                    vec3 _2937;
                    if (_2798)
                    {
                        float _2875 = clamp(1.0 - _1079.w, 0.0, 1.0);
                        float _2877 = ((_1298 * 2.0) - 1.0) * 0.300000011920928955078125;
                        float _2883 = pow(_2875, 8.0) * 500.0;
                        float _2886 = pow(_2875, 6.0) * 2000.0;
                        float _2900 = dot(normalize(_1517 + (_919 * ((-0.0500000007450580596923828125) + _2877))), _2791);
                        float _2920 = dot(normalize(_1517 + (_919 * ((0.0500000007450580596923828125 + _2877) - (_2875 * 0.0500000007450580596923828125)))), _2791);
                        _2937 = _2838 + ((((vec3(((smoothstep(-1.0, 0.0, _2900) * pow(sqrt(clamp(1.0 - (_2900 * _2900), 0.0, 1.0)), 20.0 + _2883)) * ((((_1298 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _2875))) * ((_2883 + 28.0) * 0.03978873789310455322265625)) * (_1079.xyz / vec3(max(dot(_1079.xyz, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _2920) * pow(sqrt(clamp(1.0 - (_2920 * _2920), 0.0, 1.0)), 50.0 + _2886)) * ((((_1298 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _2875)) * ((_2886 + 58.0) * 0.03978873789310455322265625))) * _2244) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fSpecularStrength);
                    }
                    else
                    {
                        float _2851 = 2.0 - _2840;
                        float _2858 = _2840 * _2840;
                        float _2862 = (((_2795 * _2858) - _2795) * _2795) + 1.0;
                        _2937 = _2838 + (((((vec3(1.0) - _2835) * exp2((((-5.554729938507080078125) * _2797) - 6.9831600189208984375) * _2797)) + _2835) * ((((1.0 / (((_2793 * _2851) + _2840) * ((_2244 * _2851) + _2840))) * (_2858 * (1.0 / (_2862 * _2862)))) * _2244) * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fSpecularStrength)) * 1.0);
                    }
                    vec3 _2987;
                    if (_1518 == 0.01960784383118152618408203125)
                    {
                        bool _2941 = _1298 > 0.4980392158031463623046875;
                        vec3 _2959 = sqrt(abs(_2837));
                        float _2973 = 0.5 / _2840;
                        _2987 = mix(_2937, (_2838 + (((((vec3(1.0) - _2959) * exp2((((-5.554729938507080078125) * _2797) - 6.9831600189208984375) * _2797)) + _2959) * ((1.0 / ((4.0 * ((_2244 + _2793) - (_2244 * _2793))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _2973) * pow(1.0 - (_2795 * _2795), _2973)))) * (_2244 * g_axPhotons._m0[g_axPhotonSpatialHash._m0[_2162]].m_fSpecularStrength))) + (_2837 * (clamp((clamp(dot(_2223, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_2223, -_1520), 0.0, 1.0), 7.0), 0.0, 1.0) * (_2941 ? 0.5 : 0.0))), vec3((_2941 ? (_1298 - 0.501960813999176025390625) : _1298) * 2.007874011993408203125));
                    }
                    else
                    {
                        _2987 = _2937;
                    }
                    vec3 _2990 = min(_2987 * _2787, vec3(1000.0)).xyz;
                    bool _2991 = _1518 == 0.007843137718737125396728515625;
                    vec3 _3002;
                    if (_2991)
                    {
                        _3002 = _2990 + (((_1079.xyz * vec3(0.0194447897374629974365234375, 1.6805228369776159524917602539062e-05, 3.1907067266079280898338765837252e-11)) * clamp(0.300000011920928955078125 + dot(_2223, -_919), 0.0, 1.0)) * _2787);
                    }
                    else
                    {
                        _3002 = _2990;
                    }
                    vec3 _3015;
                    if (_2991 && (in_var_TEXCOORD1.w < 3.0))
                    {
                        _3015 = _3002.xyz * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(in_var_TEXCOORD1.w - 2.0, 0.0)));
                    }
                    else
                    {
                        _3015 = _3002.xyz;
                    }
                    _2158 = _2367;
                    _3017 = vec4(_3015.x, _3015.y, _3015.z, _2782.w);
                    _3018 = true;
                    break;
                }
                _2158 = _2367;
                _3017 = _2782;
                _3018 = false;
                break;
            } while(false);
            if (true == _3018)
            {
                vec3 _3026 = _2160.xyz + (_3017.xyz * _3017.w);
                _2161 = vec4(_3026.x, _3026.y, _3026.z, _2160.w);
            }
            else
            {
                _2161 = _2160;
            }
        }
        _3028 = _2160;
    }
    else
    {
        _3028 = _2123;
    }
    vec4 _3886;
    if (DynamicLightsParams.g_uNumProjectors != 0u)
    {
        vec4 _3045 = vec4((FrameConstants.g_xCameraTanHalfFOV * in_var_TEXCOORD1.w) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), in_var_TEXCOORD1.w, 1.0);
        
        _3045.x -= vk3d_params[VIEW].stereo.x * (in_var_TEXCOORD1.w - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0]; 
        
        vec3 _3046 = _923 * DynamicLightsParams.g_fProjectorSpatialHashScale;
        uint _3062 = uint(((int(trunc(_3046.x)) * 73856093) ^ (int(trunc(_3046.y)) * 19349663)) ^ (int(trunc(_3046.z)) * 83492791)) % 7919u;
        uint _3072;
        uint _3073;
        if (g_axProjectorSpatialHash_1._m0[_3062] == 0u)
        {
            _3072 = 0u;
            _3073 = g_axProjectorSpatialHash_1._m0[_3062];
        }
        else
        {
            _3072 = g_axProjectorSpatialHash_1._m0[g_axProjectorSpatialHash_1._m0[_3062]];
            _3073 = g_axProjectorSpatialHash_1._m0[_3062] + 1u;
        }
        uint _3074 = _3073 + _3072;
        vec4 _3076;
        vec4 _3079;
        _3076 = _501;
        _3079 = _3028;
        uint _3082;
        vec4 _3077;
        vec4 _3080;
        for (uint _3081 = _3073; _3081 < _3074; _3076 = _3077, _3079 = _3080, _3081 = _3082)
        {
            _3082 = _3081 + 1u;
            vec4 _3875;
            bool _3876;
            do
            {
                vec4 _3172;
                bool _3173;
                do
                {
                    vec4 _3152 = _3045 * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xProjectorMatrix;
                    float _3153 = _3152.z;
                    if (_3153 < 0.0)
                    {
                        _3172 = _3076;
                        _3173 = false;
                        break;
                    }
                    vec2 _3160 = _3152.xy / vec2(_3152.w);
                    vec4 _3163 = vec4(_3160, _3153, _3152.w);
                    if (any(lessThan(vec2(0.5) - abs(_3160.xy - vec2(0.5)), vec2(0.0))))
                    {
                        _3172 = _3163;
                        _3173 = false;
                        break;
                    }
                    _3172 = _3163;
                    _3173 = true;
                    break;
                } while(false);
                if (!_3173)
                {
                    _3077 = _3172;
                    _3875 = vec4(0.0, 0.0, 0.0, 1.0);
                    _3876 = false;
                    break;
                }
                vec4 _3179 = _3045 * FrameConstants.g_xInvView;
                vec3 _3181 = g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xProjectorPos - _3179.xyz;
                if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_bCustomBoundingVolumeSet != 0u)
                {
                    bool _3197;
                    do
                    {
                        if (any(greaterThanEqual(abs((vec4(_3179.xyz, 1.0) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xWorldToBoundingSpace).xyz), vec3(1.0))))
                        {
                            _3197 = false;
                            break;
                        }
                        _3197 = true;
                        break;
                    } while(false);
                    if (!_3197)
                    {
                        _3077 = _3172;
                        _3875 = vec4(0.0, 0.0, 0.0, 1.0);
                        _3876 = false;
                        break;
                    }
                    float _3203 = -dot(_3181, g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xProjectionDirection.xyz);
                    if ((_3203 < g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fMinRange) || (_3203 > g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fLightRange))
                    {
                        _3077 = _3172;
                        _3875 = vec4(0.0, 0.0, 0.0, 1.0);
                        _3876 = false;
                        break;
                    }
                }
                float _3209 = length(_3181);
                vec3 _3211 = _3181 / vec3(_3209);
                float _3220 = g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fDistanceFade * pow(clamp((1.0 - clamp((_3209 - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fAreaLightRadius) / (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fLightRange - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fAreaLightRadius), 0.0, 1.0)) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fLightFalloffPower);
                if ((_3220 - 9.9999997473787516355514526367188e-05) < 0.0)
                {
                    _3077 = _3172;
                    _3875 = vec4(0.0, 0.0, 0.0, 1.0);
                    _3876 = false;
                    break;
                }
                vec4 _3593;
                vec4 _3594;
                if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iShadowTextureIndex != (-1))
                {
                    float _3236 = (_3172.z + max(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowBias * (1.0 - clamp(dot(_919, _3211), 0.0, 1.0)), g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowBias)) / _3172.w;
                    vec4 _3237 = _3172;
                    _3237.z = _3236;
                    float _3243 = (2.0 * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowFarPlane;
                    float _3244 = g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowFarPlane + g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane;
                    float _3245 = g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowFarPlane - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane;
                    float _3251 = clamp(((_3243 / (_3244 - ((((1.0 - _3236) * 2.0) - 1.0) * _3245))) - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane) / _3245, 0.0, 1.0);
                    vec4 _3592;
                    if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowStrength > 0.0)
                    {
                        float _3588;
                        if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowSoftness > 0.0)
                        {
                            float _3326 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                            float _3327 = sin(_3326);
                            float _3328 = cos(_3326);
                            float _3330 = (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowSoftness * 0.02500000037252902984619140625) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fProjectorOneOverTanHalfFov;
                            float _3332 = float(uint(0));
                            float _3335 = sqrt(_3332 + 0.5) * 0.5;
                            float _3337 = (_3332 * 2.400000095367431640625) + _3326;
                            uint _3344 = uint(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iShadowTextureIndex);
                            vec4 _3349 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(nonuniformEXT(sampler2D(g_xTextureArray[nonuniformEXT(_3344)], g_xBilinearClamp)), _3237.xy + (vec2(_3335 * cos(_3337), _3335 * sin(_3337)) * _3330), FrameConstants.g_fGlobalSamplerBias);
                            float _3359 = clamp(((_3243 / (_3244 - ((((1.0 - _3349.x) * 2.0) - 1.0) * _3245))) - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane) / _3245, 0.0, 1.0);
                            vec2 _3365;
                            if (_3359 < _3251)
                            {
                                vec2 _3363 = _503;
                                _3363.x = _3359;
                                vec2 _3364 = _3363;
                                _3364.y = 1.0;
                                _3365 = _3364;
                            }
                            else
                            {
                                _3365 = vec2(0.0);
                            }
                            float _3367 = float(uint(1));
                            float _3370 = sqrt(_3367 + 0.5) * 0.5;
                            float _3372 = (_3367 * 2.400000095367431640625) + _3326;
                            vec4 _3383 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_3344], g_xBilinearClamp), _3237.xy + (vec2(_3370 * cos(_3372), _3370 * sin(_3372)) * _3330), FrameConstants.g_fGlobalSamplerBias);
                            float _3393 = clamp(((_3243 / (_3244 - ((((1.0 - _3383.x) * 2.0) - 1.0) * _3245))) - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane) / _3245, 0.0, 1.0);
                            vec2 _3403;
                            if (_3393 < _3251)
                            {
                                vec2 _3399 = _503;
                                _3399.x = _3365.x + _3393;
                                vec2 _3402 = _3399;
                                _3402.y = _3365.y + 1.0;
                                _3403 = _3402;
                            }
                            else
                            {
                                _3403 = _3365;
                            }
                            float _3405 = float(uint(2));
                            float _3408 = sqrt(_3405 + 0.5) * 0.5;
                            float _3410 = (_3405 * 2.400000095367431640625) + _3326;
                            vec4 _3419 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_3344], g_xBilinearClamp), _3237.xy + (vec2(_3408 * cos(_3410), _3408 * sin(_3410)) * _3330), FrameConstants.g_fGlobalSamplerBias);
                            float _3429 = clamp(((_3243 / (_3244 - ((((1.0 - _3419.x) * 2.0) - 1.0) * _3245))) - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane) / _3245, 0.0, 1.0);
                            vec2 _3439;
                            if (_3429 < _3251)
                            {
                                vec2 _3435 = _503;
                                _3435.x = _3403.x + _3429;
                                vec2 _3438 = _3435;
                                _3438.y = _3403.y + 1.0;
                                _3439 = _3438;
                            }
                            else
                            {
                                _3439 = _3403;
                            }
                            float _3441 = float(uint(3));
                            float _3444 = sqrt(_3441 + 0.5) * 0.5;
                            float _3446 = (_3441 * 2.400000095367431640625) + _3326;
                            vec4 _3455 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xTextureArray[_3344], g_xBilinearClamp), _3237.xy + (vec2(_3444 * cos(_3446), _3444 * sin(_3446)) * _3330), FrameConstants.g_fGlobalSamplerBias);
                            float _3465 = clamp(((_3243 / (_3244 - ((((1.0 - _3455.x) * 2.0) - 1.0) * _3245))) - g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowNearPlane) / _3245, 0.0, 1.0);
                            vec2 _3475;
                            if (_3465 < _3251)
                            {
                                vec2 _3471 = _503;
                                _3471.x = _3439.x + _3465;
                                vec2 _3474 = _3471;
                                _3474.y = _3439.y + 1.0;
                                _3475 = _3474;
                            }
                            else
                            {
                                _3475 = _3439;
                            }
                            float _3587;
                            if (_3475.y > 0.0)
                            {
                                float _3481 = _3475.x / _3475.y;
                                float _3483 = (_3251 - _3481) / _3481;
                                float _3490 = (((clamp(_3483 * _3483, 0.0, 1.0) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowSoftness) * 0.02500000037252902984619140625) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fProjectorOneOverTanHalfFov) + (2.0 * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize);
                                _3587 = (((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_3344)], g_xShadowCmpSampler)), vec3(_3237.xy + (vec2(((-0.326211988925933837890625) * _3327) - ((-0.40580499172210693359375) * _3328), ((-0.40580499172210693359375) * _3327) + ((-0.326211988925933837890625) * _3328)) * _3490), _3236)) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(((-0.840143978595733642578125) * _3327) - ((-0.0735799968242645263671875) * _3328), ((-0.0735799968242645263671875) * _3327) + ((-0.840143978595733642578125) * _3328)) * _3490), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(((-0.695913970470428466796875) * _3327) - (0.45713698863983154296875 * _3328), (0.45713698863983154296875 * _3327) + ((-0.695913970470428466796875) * _3328)) * _3490), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(((-0.203345000743865966796875) * _3327) - (0.62071597576141357421875 * _3328), (0.62071597576141357421875 * _3327) + ((-0.203345000743865966796875) * _3328)) * _3490), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2((0.96233999729156494140625 * _3327) - ((-0.194983005523681640625) * _3328), ((-0.194983005523681640625) * _3327) + (0.96233999729156494140625 * _3328)) * _3490), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2((0.4734340012073516845703125 * _3327) - ((-0.4800260066986083984375) * _3328), ((-0.4800260066986083984375) * _3327) + (0.4734340012073516845703125 * _3328)) * _3490), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2((0.51945602893829345703125 * _3327) - (0.76702201366424560546875 * _3328), (0.76702201366424560546875 * _3327) + (0.51945602893829345703125 * _3328)) * _3490), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3344], g_xShadowCmpSampler), vec3(_3237.xy + (vec2((0.18546099960803985595703125 * _3327) - ((-0.89312398433685302734375) * _3328), ((-0.89312398433685302734375) * _3327) + (0.18546099960803985595703125 * _3328)) * _3490), _3236))) * 0.125;
                            }
                            else
                            {
                                _3587 = 0.0;
                            }
                            _3588 = _3587;
                        }
                        else
                        {
                            uint _3260 = uint(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iShadowTextureIndex);
                            _3588 = (((((((((((texture(nonuniformEXT(sampler2DShadow(g_xTextureArray[nonuniformEXT(_3260)], g_xShadowCmpSampler)), vec3(_3237.xy + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236)) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) + texture(sampler2DShadow(g_xTextureArray[_3260], g_xShadowCmpSampler), vec3(_3237.xy + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_f2DFilterSize), _3236))) * 0.083333335816860198974609375;
                        }
                        vec4 _3591 = vec4(0.0, 0.0, 0.0, 1.0);
                        _3591.w = 1.0 - (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fShadowStrength * _3588);
                        _3592 = _3591;
                    }
                    else
                    {
                        _3592 = vec4(0.0, 0.0, 0.0, 1.0);
                    }
                    _3593 = _3237;
                    _3594 = _3592;
                }
                else
                {
                    _3593 = _3172;
                    _3594 = vec4(0.0, 0.0, 0.0, 1.0);
                }
                vec4 _3874;
                if (_3594.w > 0.0)
                {
                    vec3 _3654;
                    if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_bIsProjector != 0u)
                    {
                        vec4 _3618;
                        if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iTextureIndex > 0)
                        {
                            _3618 = textureLod(nonuniformEXT(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iTextureIndex))], g_xTrilinearClamp)), _3593.xy, 0.0);
                        }
                        else
                        {
                            _3618 = vec4(1.0);
                        }
                        bool _3624 = g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iCausticTextureIndex > 0;
                        float _3632;
                        if (_3624)
                        {
                            _3632 = textureLod(nonuniformEXT(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iCausticTextureIndex))], g_xTrilinearWrap)), (_3593.xy * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xCausticOffsets.z) + g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xCausticOffsets.xy, 0.0).x;
                        }
                        else
                        {
                            _3632 = 1.0;
                        }
                        vec4 _3644;
                        if (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iDetailTextureIndex > 0)
                        {
                            _3644 = _3618 * textureLod(nonuniformEXT(
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
sampler2D(g_xTextureArray[nonuniformEXT(uint(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_iDetailTextureIndex))], g_xTrilinearWrap)), (_3593.xy * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xDetailOffsetScale.zw) + g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xDetailOffsetScale.xy, 0.0);
                        }
                        else
                        {
                            _3644 = _3618;
                        }
                        vec4 _3650;
                        if (_3624)
                        {
                            _3650 = clamp(_3644 * min(_3632, 1.0), vec4(0.0), vec4(0.1500000059604644775390625));
                        }
                        else
                        {
                            _3650 = _3644;
                        }
                        _3654 = (g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xLightColour * _3220) * _3650.xyz;
                    }
                    else
                    {
                        _3654 = g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xLightColour * (_3220 * clamp(((-dot(g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_xProjectionDirection.xyz, _3211)) + g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fAngularFalloffOfs) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fAngularFalloffScale, 0.0, 1.0));
                    }
                    float _3656 = (_1518 != 0.01960784383118152618408203125) ? _1298 : 0.0;
                    vec3 _3658 = normalize(_1520 + _3211);
                    float _3660 = clamp(dot(_919, _3211), 0.0, 1.0);
                    float _3662 = clamp(dot(_919, _1520), 0.0, 1.0);
                    float _3664 = clamp(dot(_919, _3658), 0.0, 1.0);
                    float _3666 = clamp(dot(_1520, _3658), 0.0, 1.0);
                    bool _3667 = _1518 == 0.01568627543747425079345703125;
                    vec3 _3704;
                    vec3 _3705;
                    vec3 _3706;
                    if (_3667)
                    {
                        _3704 = vec3(0.046999998390674591064453125);
                        _3705 = vec3(_3660);
                        _3706 = _1079.xyz;
                    }
                    else
                    {
                        vec3 _3700;
                        vec3 _3701;
                        vec3 _3702;
                        if (_1518 == 0.007843137718737125396728515625)
                        {
                            _3700 = vec3(0.0280000008642673492431640625);
                            _3701 = vec3(_3660);
                            _3702 = _1079.xyz;
                        }
                        else
                        {
                            vec3 _3696;
                            vec3 _3697;
                            vec3 _3698;
                            if (_1518 == 0.0039215688593685626983642578125)
                            {
                                _3696 = vec3(0.0280000008642673492431640625);
                                _3697 = vec3((clamp((clamp(dot(_3211, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_3211, -_1520), 0.0, 1.0), 7.0), 0.0, 1.0) * _3656) + _3660);
                                _3698 = _1079.xyz;
                            }
                            else
                            {
                                vec3 _3679 = vec3(_3656);
                                _3696 = mix(vec3(0.039999999105930328369140625), _1079.xyz, _3679);
                                _3697 = vec3(_3660);
                                _3698 = mix(_1079.xyz, vec3(0.0), _3679);
                            }
                            _3700 = _3696;
                            _3701 = _3697;
                            _3702 = _3698;
                        }
                        _3704 = _3700;
                        _3705 = _3701;
                        _3706 = _3702;
                    }
                    vec3 _3707 = _3705 * _3706;
                    float _3708 = max(_1079.w, 0.0500000007450580596923828125);
                    float _3709 = _3708 * _3708;
                    vec3 _3806;
                    if (_3667)
                    {
                        float _3744 = clamp(1.0 - _1079.w, 0.0, 1.0);
                        float _3746 = ((_1298 * 2.0) - 1.0) * 0.300000011920928955078125;
                        float _3752 = pow(_3744, 8.0) * 500.0;
                        float _3755 = pow(_3744, 6.0) * 2000.0;
                        float _3769 = dot(normalize(_1517 + (_919 * ((-0.0500000007450580596923828125) + _3746))), _3658);
                        float _3789 = dot(normalize(_1517 + (_919 * ((0.0500000007450580596923828125 + _3746) - (_3744 * 0.0500000007450580596923828125)))), _3658);
                        _3806 = _3707 + ((((vec3(((smoothstep(-1.0, 0.0, _3769) * pow(sqrt(clamp(1.0 - (_3769 * _3769), 0.0, 1.0)), 20.0 + _3752)) * ((((_1298 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _3744))) * ((_3752 + 28.0) * 0.03978873789310455322265625)) * (_1079.xyz / vec3(max(dot(_1079.xyz, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _3789) * pow(sqrt(clamp(1.0 - (_3789 * _3789), 0.0, 1.0)), 50.0 + _3755)) * ((((_1298 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _3744)) * ((_3755 + 58.0) * 0.03978873789310455322265625))) * _3660) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fSpecularStrength);
                    }
                    else
                    {
                        float _3720 = 2.0 - _3709;
                        float _3727 = _3709 * _3709;
                        float _3731 = (((_3664 * _3727) - _3664) * _3664) + 1.0;
                        _3806 = _3707 + (((((vec3(1.0) - _3704) * exp2((((-5.554729938507080078125) * _3666) - 6.9831600189208984375) * _3666)) + _3704) * ((((1.0 / (((_3662 * _3720) + _3709) * ((_3660 * _3720) + _3709))) * (_3727 * (1.0 / (_3731 * _3731)))) * _3660) * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fSpecularStrength)) * 1.0);
                    }
                    vec3 _3856;
                    if (_1518 == 0.01960784383118152618408203125)
                    {
                        bool _3810 = _1298 > 0.4980392158031463623046875;
                        vec3 _3828 = sqrt(abs(_3706));
                        float _3842 = 0.5 / _3709;
                        _3856 = mix(_3806, (_3707 + (((((vec3(1.0) - _3828) * exp2((((-5.554729938507080078125) * _3666) - 6.9831600189208984375) * _3666)) + _3828) * ((1.0 / ((4.0 * ((_3660 + _3662) - (_3660 * _3662))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _3842) * pow(1.0 - (_3664 * _3664), _3842)))) * (_3660 * g_axProjectors._m0[g_axProjectorSpatialHash_1._m0[_3081]].m_fSpecularStrength))) + (_3706 * (clamp((clamp(dot(_3211, -_919), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_3211, -_1520), 0.0, 1.0), 7.0), 0.0, 1.0) * (_3810 ? 0.5 : 0.0))), vec3((_3810 ? (_1298 - 0.501960813999176025390625) : _1298) * 2.007874011993408203125));
                    }
                    else
                    {
                        _3856 = _3806;
                    }
                    vec3 _3859 = min(_3856 * _3654, vec3(1000.0)).xyz;
                    vec3 _3872;
                    if ((_1518 == 0.007843137718737125396728515625) && (in_var_TEXCOORD1.w < 3.0))
                    {
                        _3872 = _3859 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(in_var_TEXCOORD1.w - 2.0, 0.0)));
                    }
                    else
                    {
                        _3872 = _3859;
                    }
                    _3874 = vec4(_3872.x, _3872.y, _3872.z, _3594.w);
                }
                else
                {
                    _3874 = _3594;
                }
                _3077 = _3593;
                _3875 = _3874;
                _3876 = true;
                break;
            } while(false);
            if (true == _3876)
            {
                vec3 _3884 = _3079.xyz + (_3875.xyz * _3875.w);
                _3080 = vec4(_3884.x, _3884.y, _3884.z, _3079.w);
            }
            else
            {
                _3080 = _3079;
            }
        }
        _3886 = _3079;
    }
    else
    {
        _3886 = _3028;
    }
    vec4 _3956;
    if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bUseRefraction != 0u)
    {
        vec3 _3901 = _919 * mat3(FrameConstants.g_xView[0].xyz, FrameConstants.g_xView[1].xyz, FrameConstants.g_xView[2].xyz);
        vec2 _3915 = (gl_FragCoord.xy + ((vec2(-_3901.x, -_3901.y) * g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_fRefractivity) / vec2(in_var_TEXCOORD1.w))) * FrameConstants.g_xRcpScreenDims;
        vec2 _3933;
        if ((FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
SAMPLE_TEXTURE1(SAMPLER_2D(g_xDepthTexture, g_xPointClamp), _3915, FrameConstants.g_fGlobalSamplerBias).x)) < in_var_TEXCOORD1.w)
        {
            _3933 = gl_FragCoord.xy * FrameConstants.g_xRcpScreenDims;
        }
        else
        {
            _3933 = _3915;
        }
        vec4 _3937 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
SAMPLE_TEXTURE1(SAMPLER_2D(g_xSceneTexture, g_xTrilinearClamp), _3933, FrameConstants.g_fGlobalSamplerBias);
        vec3 _3938 = _3937.xyz;
        vec4 _3955;
        if (g_axMaterials._m0[Material1Constants.g_xStaticMaterial1.m_uStructuredBufferIndex].m_bRefractionBlendBackground != 0u)
        {
            vec3 _3953 = mix(_3938, _3886.xyz, vec3(_3886.w));
            _3955 = vec4(_3953.x, _3953.y, _3953.z, _3886.w);
        }
        else
        {
            vec3 _3948 = _3938 + (_3886.xyz * _3886.w);
            _3955 = vec4(_3948.x, _3948.y, _3948.z, _3886.w);
        }
        _3956 = _3955;
    }
    else
    {
        _3956 = _3886;
    }
    bool _3959 = FogParams.g_bFogEnabled != 0u;
    bool _3962 = FogParams.g_bHeightFogEnabled != 0u;
    vec4 _4290;
    if (_3959 || _3962)
    {
        bool _3968 = DynamicMaterialConstants.g_fFogScale < 0.0;
        float _3974;
        float _3975;
        if (_3968)
        {
            _3974 = -DynamicMaterialConstants.g_fFogScale;
            _3975 = FrameConstants.g_xCameraFarClipValues.x;
        }
        else
        {
            _3974 = 1.0;
            _3975 = in_var_TEXCOORD1.w;
        }
        vec4 _4277;
        if (_3962)
        {
            vec3 _4093 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _3975) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _3975, 1.0) * FrameConstants.g_xInvView).xyz - FrameConstants.g_xCamPos;
            float _4094 = length(_4093);
            vec3 _4096 = -normalize(_4093);
            vec4 _4200;
            if (_4094 > FogParams.g_fHeightFogStartDistance)
            {
                float _4104 = max(_4094 - FogParams.g_fHeightFogStartDistance, 0.0);
                float _4128;
                if (_4094 >= FrameConstants.g_xCameraFarClipValues.x)
                {
                    _4128 = float(int(sign(_4096.y))) * 1.57079601287841796875;
                }
                else
                {
                    _4128 = atan((((_4096.y * _4104) - FrameConstants.g_xCamPos.y) - FogParams.g_fHeightFogHeight) / FogParams.g_fHeightFogFalloff);
                }
                float _4129 = _4096.y;
                float _4164;
                if (abs(_4129) <= 9.9999999747524270787835121154785e-07)
                {
                    float _4159 = (-FrameConstants.g_xCamPos.y) - FogParams.g_fHeightFogHeight;
                    _4164 = (((FogParams.g_fHeightFogStrength * FogParams.g_fHeightFogFalloff) * _4104) * FogParams.g_fHeightFogFalloff) / ((_4159 * _4159) + (FogParams.g_fHeightFogFalloff * FogParams.g_fHeightFogFalloff));
                }
                else
                {
                    _4164 = ((FogParams.g_fHeightFogStrength * FogParams.g_fHeightFogFalloff) * (_4128 - atan(((-FogParams.g_fHeightFogHeight) - FrameConstants.g_xCamPos.y) / FogParams.g_fHeightFogFalloff))) / _4129;
                }
                float _4175 = clamp(max(-dot(FogParams.g_xSunDirection, _4096), 0.0) - FogParams.g_fCosScatteringAngle, 0.0, 1.0) / (1.0 - FogParams.g_fCosScatteringAngle);
                _4200 = vec4(FogParams.g_xHeightFogColour + ((FogParams.g_xDirectColour * clamp(max(_4094 - FogParams.g_fDirectScatterStartDistance, 0.0) / FogParams.g_fDirectScatterFadeDistance, 0.0, 1.0)) * (_4175 * _4175)), clamp(_4164, 0.0, 1.0) * FogParams.g_fHeightFogMaxOpacity);
            }
            else
            {
                _4200 = vec4(0.0);
            }
            vec4 _4276;
            if (FogParams.g_bTexturedFogEnabled != 0u)
            {
                float _4211 = (_3975 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                float _4215 = clamp(_4211, 0.0, 1.0);
                float _4225 = (FogParams.g_fDensityPower > 0.0) ? (pow(_4215, FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _4211).x;
                float _4240 = atan(-_519.z, -_519.x) * 0.159155070781707763671875;
                vec2 _4250 = vec2(_4240, (acos(min(max((_519.y * FogParams.g_fHorizonScale) + FogParams.g_fHorizonOffset, -1.0), 1.0)) * FogParams.g_fVerticalTextureScale) * 0.31831014156341552734375);
                _4250.x = _4240 + FogParams.g_fTextureUOffset;
                _4276 = vec4(mix(vec4(TEXTURE_LOD(SAMPLER_2D(g_xFogColourTexture, g_xFilteredWrap), _4250, (1.0 - _4215) * 8.0).xyz * FogParams.g_fTextureHDRScale, _4225).xyz, _4200.xyz, vec3(_4200.w)), clamp(_4200.w + _4225, 0.0, 1.0));
            }
            else
            {
                _4276 = _4200;
            }
            _4277 = _4276;
        }
        else
        {
            vec4 _4074;
            if (_3959)
            {
                float _4055 = (_3975 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                _4074 = vec4(FogParams.g_xFogColour, (FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_4055, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _4055).x);
            }
            else
            {
                vec4 _4047;
                if (FogParams.g_bTexturedFogEnabled != 0u)
                {
                    float _3993 = (_3975 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                    float _3997 = clamp(_3993, 0.0, 1.0);
                    float _4022 = atan(-_519.z, -_519.x) * 0.159155070781707763671875;
                    vec2 _4032 = vec2(_4022, (acos(min(max((_519.y * FogParams.g_fHorizonScale) + FogParams.g_fHorizonOffset, -1.0), 1.0)) * FogParams.g_fVerticalTextureScale) * 0.31831014156341552734375);
                    _4032.x = _4022 + FogParams.g_fTextureUOffset;
                    _4047 = vec4(TEXTURE_LOD(SAMPLER_2D(g_xFogColourTexture, g_xFilteredWrap), _4032, (1.0 - _3997) * 8.0).xyz * FogParams.g_fTextureHDRScale, (FogParams.g_fDensityPower > 0.0) ? (pow(_3997, FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _3993).x);
                }
                else
                {
                    _4047 = vec4(0.0);
                }
                _4074 = _4047;
            }
            _4277 = _4074;
        }
        _4290 = vec4(mix(_3956.xyz, _4277.xyz * (_3968 ? 1.0 : DynamicMaterialConstants.g_fFogScale), vec3(_4277.w * _3974)), _3956.w);
    }
    else
    {
        _4290 = _3956;
    }
    vec4 _4295 = _4290;
    _4295.w = _4290.w * SHConstants.g_fSkinAlpha;
    uint _4301 = floatBitsToUint(g_xInstanceTexture._m0[InstancedConstants.g_uInstancedAnimID + 3u].w);
    vec4 _4323;
    if (((_4301 >> 24u) & 1u) != 0u)
    {
        vec3 _4310 = _504;
        _4310.x = float((_4301 & 16711680u) >> 16u);
        vec3 _4314 = _4310;
        _4314.y = float((_4301 & 65280u) >> 8u);
        vec3 _4317 = _4314;
        _4317.z = float(_4301 & 255u);
        vec3 _4321 = (_4317 * vec3(0.0039215688593685626983642578125)) * dot(_4295.xyz, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375));
        _4323 = vec4(_4321.x, _4321.y, _4321.z, _4295.w);
    }
    else
    {
        _4323 = _4295;
    }
    out_var_SV_TARGET = _4323;
}


