// Flashlight
// Dynamic Light
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct ProjectorParams
{
    mat4 m_xProjectorMatrix;
    mat4 m_xWorldToBoundingSpace;
    vec4 m_xProjectionDirection;
    vec4 m_xDetailOffsetScale;
    vec4 m_xCausticOffsets;
    vec3 m_xProjectorPos;
    float m_fPad;
    vec4 m_xOrientation1;
    vec4 m_xOrientation2;
    vec4 m_xOrientation3;
    vec4 m_xPosition_UVAspect;
    vec4 m_xOffsets;
    vec3 m_xLightColour;
    float m_fDistanceFade;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fAngularFalloffOfs;
    float m_fAngularFalloffScale;
    float m_fMinRange;
    float m_fLightRange;
    float m_fShadowStrength;
    uint m_bCustomBoundingVolumeSet;
    float m_f2DFilterSize;
    float m_fCubeFilterSize;
    float m_fCubeClampWidth;
    int m_iShadowTextureIndex;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fProjectorOneOverTanHalfFov;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fAreaLightRadius;
    uint m_uShadowPoolLevel;
    int m_iTextureIndex;
    int m_iDetailTextureIndex;
    int m_iCausticTextureIndex;
    uint m_bIsProjector;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_ProjectorIndex
{
    uint g_uProjectorIndex;
} ProjectorIndex;

layout(set = 1, binding = 90, std430) readonly buffer type_StructuredBuffer_ProjectorParams
{
    layout(row_major) ProjectorParams _m0[];
} g_axProjectors;

layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) in vec3 in_var_CamDir;
layout(location = 0) out vec4 out_var_SV_Target;

vec3 _128;

void main()
{
    ivec2 _138 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _146 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _138, 0).x);
    vec4 _161 = vec4((FrameConstants.g_xCameraTanHalfFOV * _146) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _146, 1.0);
    
   _161.x -= vk3d_params[VIEW].stereo.x * (_146 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0]; 
    
    vec4 _194 = TEXEL_FETCH(g_xLighting_NormalBuffer, _138, 0);
    vec4 _196 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _138, 0);
    vec4 _198 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _138, 0);
    vec3 _202 = normalize((_194.xyz * 2.0) - vec3(1.0));
    vec3 _203 = _196.xyz;
    float _204 = _198.y;
    float _205 = _196.w;
    float _206 = _198.x;
    vec2 _209 = (_198.zw * 2.0) - vec2(1.0);
    vec2 _211 = _209.xy;
    float _215 = sqrt(clamp(1.0 - dot(_211, _211), 0.0, 1.0));
    vec3 _220 = vec3(_209.x, _209.y, _128.z);
    _220.z = (_194.w != 0.0) ? _215 : (-_215);
    vec3 _221 = normalize(in_var_CamDir);
    vec4 _540;
    bool _541;
    do
    {
        bool _243;
        do
        {
            vec4 _226 = _161 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorMatrix;
            if (_226.z < 0.0)
            {
                _243 = false;
                break;
            }
            if (any(lessThan(vec2(0.5) - abs((_226.xy / vec2(_226.w)).xy - vec2(0.5)), vec2(0.0))))
            {
                _243 = false;
                break;
            }
            _243 = true;
            break;
        } while(false);
        if (!_243)
        {
            _540 = vec4(0.0, 0.0, 0.0, 1.0);
            _541 = false;
            break;
        }
        vec3 _251 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectorPos - (_161 * FrameConstants.g_xInvView).xyz;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_bCustomBoundingVolumeSet != 0u)
        {
            float _256 = -dot(_251, g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz);
            if ((_256 < g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fMinRange) || (_256 > g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange))
            {
                _540 = vec4(0.0, 0.0, 0.0, 1.0);
                _541 = false;
                break;
            }
        }
        float _262 = length(_251);
        vec3 _264 = _251 / vec3(_262);
        float _273 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fDistanceFade * pow(clamp((1.0 - clamp((_262 - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius) / (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightRange - g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius), 0.0, 1.0)) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffScale, 0.0, 1.0), g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fLightFalloffPower);
        if ((_273 - 9.9999997473787516355514526367188e-05) < 0.0)
        {
            _540 = vec4(0.0, 0.0, 0.0, 1.0);
            _541 = false;
            break;
        }
        float _303;
        vec3 _304;
        if (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius > 0.0)
        {
            vec3 _290 = -reflect(_221, _202);
            vec3 _293 = (_290 * dot(_251, _290)) - _251;
            _303 = g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius * inversesqrt(dot(_251, _251));
            _304 = normalize(_251 + (_293 * clamp(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAreaLightRadius / length(_293), 0.0, 1.0)));
        }
        else
        {
            _303 = 0.0;
            _304 = _264;
        }
        float _306 = (_205 != 0.01960784383118152618408203125) ? _206 : 0.0;
        vec3 _308 = normalize(_221 + _304);
        float _310 = clamp(dot(_202, _264), 0.0, 1.0);
        float _312 = clamp(dot(_202, _221), 0.0, 1.0);
        float _314 = clamp(dot(_202, _308), 0.0, 1.0);
        float _316 = clamp(dot(_221, _308), 0.0, 1.0);
        bool _317 = _205 == 0.01568627543747425079345703125;
        vec3 _354;
        vec3 _355;
        vec3 _356;
        if (_317)
        {
            _354 = vec3(0.046999998390674591064453125);
            _355 = vec3(_310);
            _356 = _203;
        }
        else
        {
            vec3 _350;
            vec3 _351;
            vec3 _352;
            if (_205 == 0.007843137718737125396728515625)
            {
                _350 = vec3(0.0280000008642673492431640625);
                _351 = vec3(_310);
                _352 = _203;
            }
            else
            {
                vec3 _346;
                vec3 _347;
                vec3 _348;
                if (_205 == 0.0039215688593685626983642578125)
                {
                    _346 = vec3(0.0280000008642673492431640625);
                    _347 = vec3((clamp((clamp(dot(_264, -_202), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_264, -_221), 0.0, 1.0), 7.0), 0.0, 1.0) * _306) + _310);
                    _348 = _203;
                }
                else
                {
                    vec3 _329 = vec3(_306);
                    _346 = mix(vec3(0.039999999105930328369140625), _203, _329);
                    _347 = vec3(_310);
                    _348 = mix(_203, vec3(0.0), _329);
                }
                _350 = _346;
                _351 = _347;
                _352 = _348;
            }
            _354 = _350;
            _355 = _351;
            _356 = _352;
        }
        vec3 _357 = _355 * _356;
        float _358 = max(_204, 0.0500000007450580596923828125);
        float _359 = _358 * _358;
        vec3 _472;
        if (_317)
        {
            float _410 = clamp(1.0 - _204, 0.0, 1.0);
            float _412 = ((_206 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _418 = pow(_410, 8.0) * 500.0;
            float _421 = pow(_410, 6.0) * 2000.0;
            float _435 = dot(normalize(_220 + (_202 * ((-0.0500000007450580596923828125) + _412))), _308);
            float _455 = dot(normalize(_220 + (_202 * ((0.0500000007450580596923828125 + _412) - (_410 * 0.0500000007450580596923828125)))), _308);
            _472 = _357 + ((((vec3(((smoothstep(-1.0, 0.0, _435) * pow(sqrt(clamp(1.0 - (_435 * _435), 0.0, 1.0)), 20.0 + _418)) * ((((_206 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _410))) * ((_418 + 28.0) * 0.03978873789310455322265625)) * (_203 / vec3(max(dot(_203, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _455) * pow(sqrt(clamp(1.0 - (_455 * _455), 0.0, 1.0)), 50.0 + _421)) * ((((_206 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _410)) * ((_421 + 58.0) * 0.03978873789310455322265625))) * _310) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength);
        }
        else
        {
            float _365 = clamp(_303 * (1.0 - _359), 0.0, 1.0);
            float _385;
            if (_365 > 0.0)
            {
                float _376 = _359 * _359;
                _385 = _376 / (_376 + (((_365 * 0.25) * ((3.0 * _359) + _365)) / (_316 + 0.001000000047497451305389404296875)));
            }
            else
            {
                _385 = 1.0;
            }
            float _386 = 2.0 - _359;
            float _393 = _359 * _359;
            float _397 = (((_314 * _393) - _314) * _314) + 1.0;
            _472 = _357 + (((((vec3(1.0) - _354) * exp2((((-5.554729938507080078125) * _316) - 6.9831600189208984375) * _316)) + _354) * ((((1.0 / (((_312 * _386) + _359) * ((_310 * _386) + _359))) * (_393 * (1.0 / (_397 * _397)))) * _310) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength)) * _385);
        }
        vec3 _522;
        if (_205 == 0.01960784383118152618408203125)
        {
            bool _476 = _206 > 0.4980392158031463623046875;
            vec3 _494 = sqrt(abs(_356));
            float _508 = 0.5 / _359;
            _522 = mix(_472, (_357 + (((((vec3(1.0) - _494) * exp2((((-5.554729938507080078125) * _316) - 6.9831600189208984375) * _316)) + _494) * ((1.0 / ((4.0 * ((_310 + _312) - (_310 * _312))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _508) * pow(1.0 - (_314 * _314), _508)))) * (_310 * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fSpecularStrength))) + (_356 * (clamp((clamp(dot(_264, -_202), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_264, -_221), 0.0, 1.0), 7.0), 0.0, 1.0) * (_476 ? 0.5 : 0.0))), vec3((_476 ? (_206 - 0.501960813999176025390625) : _206) * 2.007874011993408203125));
        }
        else
        {
            _522 = _472;
        }
        vec3 _525 = min(_522 * (g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xLightColour * (_273 * clamp(((-dot(g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_xProjectionDirection.xyz, _264)) + g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffOfs) * g_axProjectors._m0[ProjectorIndex.g_uProjectorIndex].m_fAngularFalloffScale, 0.0, 1.0))), vec3(1000.0)).xyz;
        vec3 _538;
        if ((_205 == 0.007843137718737125396728515625) && (_146 < 3.0))
        {
            _538 = _525 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_146 - 2.0, 0.0)));
        }
        else
        {
            _538 = _525;
        }
        _540 = vec4(_538.x, _538.y, _538.z, vec4(0.0, 0.0, 0.0, 1.0).w);
        _541 = true;
        break;
    } while(false);
    if (false == _541)
    {
        discard;
    }
    out_var_SV_Target = _540;
}


